@@ -1,8 +1,4 @@
 #!/bin/sh -e
-
-# workaround dnf bug
-export FORCE_COLUMNS=100
-
 #install RPM packages
 dnf install -y dist/*.rpm
 dnf install -y gcovr
@@ -17,6 +13,9 @@ dbus-daemon --system --fork
 /usr/lib/polkit-1/polkitd &
 sleep 5
 
+# Enable testing capturing emulation data
+fwupdtool emulation-tag 08d460be0f1f9f128413f816022a6439e0078018
+
 # run the daemon startup to check it can start
 /usr/libexec/fwupd/fwupd --immediate-exit --verbose
 
