@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -9,9 +9,12 @@
 #include <gio/gio.h>
 
 #include "fu-bios-settings.h"
+#include "fu-common-struct.h"
 #include "fu-common.h"
+#include "fu-efi-hard-drive-device-path.h"
+#include "fu-efivars.h"
 #include "fu-firmware.h"
-#include "fu-smbios.h"
+#include "fu-smbios-struct.h"
 
 #define FU_TYPE_CONTEXT (fu_context_get_type())
 G_DECLARE_DERIVABLE_TYPE(FuContext, fu_context, FU, CONTEXT, GObject)
@@ -20,13 +23,55 @@ struct _FuContextClass {
 	GObjectClass parent_class;
 	/* signals */
 	void (*security_changed)(FuContext *self);
+	void (*housekeeping)(FuContext *self);
 };
 
+/**
+ * FuContextQuirkSource:
+ *
+ * The source of the quirk data, ordered by how good the data is.
+ **/
+typedef enum {
+	/**
+	 * FU_CONTEXT_QUIRK_SOURCE_DEVICE:
+	 *
+	 * From the device itself, perhaps a USB descriptor.
+	 *
+	 * Since: 2.0.2
+	 **/
+	FU_CONTEXT_QUIRK_SOURCE_DEVICE,
+	/**
+	 * FU_CONTEXT_QUIRK_SOURCE_FILE:
+	 *
+	 * From an internal `.quirk` file.
+	 *
+	 * Since: 2.0.2
+	 **/
+	FU_CONTEXT_QUIRK_SOURCE_FILE,
+	/**
+	 * FU_CONTEXT_QUIRK_SOURCE_DB:
+	 *
+	 * From the database, populated from `usb.ids` and `pci.ids`.
+	 *
+	 * Since: 2.0.2
+	 **/
+	FU_CONTEXT_QUIRK_SOURCE_DB,
+	/**
+	 * FU_CONTEXT_QUIRK_SOURCE_FALLBACK:
+	 *
+	 * A good fallback, perhaps from the PCI class information.
+	 *
+	 * Since: 2.0.2
+	 **/
+	FU_CONTEXT_QUIRK_SOURCE_FALLBACK,
+} FuContextQuirkSource;
+
 /**
  * FuContextLookupIter:
  * @self: a #FuContext
  * @key: a key
  * @value: a value
+ * @source: a #FuContextQuirkSource, e.g. %FU_CONTEXT_QUIRK_SOURCE_DB
  * @user_data: user data
  *
  * The context lookup iteration callback.
@@ -34,59 +79,81 @@ struct _FuContextClass {
 typedef void (*FuContextLookupIter)(FuContext *self,
 				    const gchar *key,
 				    const gchar *value,
+				    FuContextQuirkSource source,
 				    gpointer user_data);
 
-/**
- * FU_CONTEXT_FLAG_NONE:
- *
- * No flags set.
- *
- * Since: 1.8.5
- **/
-#define FU_CONTEXT_FLAG_NONE (0u)
-
-/**
- * FU_CONTEXT_FLAG_SAVE_EVENTS:
- *
- * Save events so that they can be replayed to emulate devices.
- *
- * Since: 1.8.5
- **/
-#define FU_CONTEXT_FLAG_SAVE_EVENTS (1u << 0)
-
-/**
- * FU_CONTEXT_FLAG_SYSTEM_INHIBIT:
- *
- * All devices are not updatable due to a system-wide inhibit.
- *
- * Since: 1.8.10
- **/
-#define FU_CONTEXT_FLAG_SYSTEM_INHIBIT (1u << 1)
-
-/**
- * FU_CONTEXT_FLAG_LOADED_HWINFO:
- *
- * Hardware information has been loaded with a call to fu_context_load_hwinfo().
- *
- * Since: 1.9.10
- **/
-#define FU_CONTEXT_FLAG_LOADED_HWINFO (1u << 2)
-
-/**
- * FU_CONTEXT_FLAG_IGNORE_EFIVARS_FREE_SPACE:
- *
- * Ignore the efivars free space requirement for db, dbx, KEK and PK updates.
- *
- * Since: 1.9.31, backported from 2.0.12
- **/
-#define FU_CONTEXT_FLAG_IGNORE_EFIVARS_FREE_SPACE (1u << 6)
-
 /**
  * FuContextFlags:
  *
  * The context flags.
  **/
-typedef guint64 FuContextFlags;
+typedef enum {
+	/**
+	 * FU_CONTEXT_FLAG_NONE:
+	 *
+	 * No flags set.
+	 *
+	 * Since: 1.8.5
+	 **/
+	FU_CONTEXT_FLAG_NONE = 0,
+	/**
+	 * FU_CONTEXT_FLAG_SAVE_EVENTS:
+	 *
+	 * Save events so that they can be replayed to emulate devices.
+	 *
+	 * Since: 1.8.5
+	 **/
+	FU_CONTEXT_FLAG_SAVE_EVENTS = 1u << 0,
+	/**
+	 * FU_CONTEXT_FLAG_SYSTEM_INHIBIT:
+	 *
+	 * All devices are not updatable due to a system-wide inhibit.
+	 *
+	 * Since: 1.8.10
+	 **/
+	FU_CONTEXT_FLAG_SYSTEM_INHIBIT = 1u << 1,
+	/**
+	 * FU_CONTEXT_FLAG_LOADED_HWINFO:
+	 *
+	 * Hardware information has been loaded with a call to fu_context_load_hwinfo().
+	 *
+	 * Since: 1.9.10
+	 **/
+	FU_CONTEXT_FLAG_LOADED_HWINFO = 1u << 2,
+	/**
+	 * FU_CONTEXT_FLAG_INHIBIT_VOLUME_MOUNT:
+	 *
+	 * Do not allow mounting volumes, usually set in self tests.
+	 *
+	 * Since: 2.0.2
+	 **/
+	FU_CONTEXT_FLAG_INHIBIT_VOLUME_MOUNT = 1u << 3,
+	/**
+	 * FU_CONTEXT_FLAG_FDE_BITLOCKER:
+	 *
+	 * Bitlocker style full disk encryption is in use
+	 *
+	 * Since: 2.0.5
+	 **/
+	FU_CONTEXT_FLAG_FDE_BITLOCKER = 1u << 4,
+	/**
+	 * FU_CONTEXT_FLAG_FDE_SNAPD:
+	 *
+	 * Snapd style full disk encryption is in use
+	 *
+	 * Since: 2.0.5
+	 **/
+	FU_CONTEXT_FLAG_FDE_SNAPD = 1u << 5,
+
+	/**
+	 * FU_CONTEXT_FLAG_LOADED_UNKNOWN:
+	 *
+	 * Unknown flag value.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_FLAG_LOADED_UNKNOWN = G_MAXUINT64,
+} FuContextFlags;
 
 void
 fu_context_add_flag(FuContext *context, FuContextFlags flag) G_GNUC_NON_NULL(1);
@@ -169,6 +236,15 @@ fu_context_get_bios_setting(FuContext *self, const gchar *name) G_GNUC_NON_NULL(
 GPtrArray *
 fu_context_get_esp_volumes(FuContext *self, GError **error) G_GNUC_WARN_UNUSED_RESULT
     G_GNUC_NON_NULL(1);
+FuVolume *
+fu_context_get_default_esp(FuContext *self, GError **error) G_GNUC_WARN_UNUSED_RESULT
+    G_GNUC_NON_NULL(1);
+FuVolume *
+fu_context_get_esp_volume_by_hard_drive_device_path(FuContext *self,
+						    FuEfiHardDriveDevicePath *dp,
+						    GError **error) G_GNUC_WARN_UNUSED_RESULT
+    G_GNUC_NON_NULL(1, 2);
+
 FuFirmware *
 fu_context_get_fdt(FuContext *self, GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1);
 FuSmbiosChassisKind
@@ -177,7 +253,57 @@ void
 fu_context_set_esp_location(FuContext *self, const gchar *location);
 const gchar *
 fu_context_get_esp_location(FuContext *self);
+FuEfivars *
+fu_context_get_efivars(FuContext *self) G_GNUC_NON_NULL(1);
 
-gboolean
-fu_context_efivars_check_free_space(FuContext *self, gsize count, GError **error)
+/**
+ * FuContextEspFileFlags:
+ *
+ * The flags to use when loading files in the ESP.
+ **/
+typedef enum {
+	/**
+	 * FU_CONTEXT_ESP_FILE_FLAG_NONE:
+	 *
+	 * No flags set.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_ESP_FILE_FLAG_NONE = 0,
+	/**
+	 * FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_FIRST_STAGE:
+	 *
+	 * Include 1st stage bootloaders like shim.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_FIRST_STAGE = 1 << 0,
+	/**
+	 * FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_SECOND_STAGE:
+	 *
+	 * Include 2nd stage bootloaders like shim.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_SECOND_STAGE = 1 << 1,
+	/**
+	 * FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_REVOCATIONS:
+	 *
+	 * Include revokcations, for example the `revocations.efi` file used by shim.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_ESP_FILE_FLAG_INCLUDE_REVOCATIONS = 1 << 2,
+	/**
+	 * FU_CONTEXT_ESP_FILE_FLAG_UNKNOWN:
+	 *
+	 * Unknown flag value.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_CONTEXT_ESP_FILE_FLAG_UNKNOWN = G_MAXUINT64,
+} FuContextEspFileFlags;
+
+GPtrArray *
+fu_context_get_esp_files(FuContext *self, FuContextEspFileFlags flags, GError **error)
     G_GNUC_NON_NULL(1);
