@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2015 Richard Hughes <richard@hughsie.com>
+ * Copyright 2015 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -9,7 +9,9 @@
 #include <fwupd.h>
 
 #include "fu-context.h"
+#include "fu-device-event.h"
 #include "fu-device-locker.h"
+#include "fu-device-struct.h"
 #include "fu-firmware.h"
 #include "fu-progress.h"
 #include "fu-security-attrs.h"
@@ -41,7 +43,8 @@ struct _FuDeviceClass {
 	gboolean (*probe)(FuDevice *self, GError **error) G_GNUC_WARN_UNUSED_RESULT;
 	gboolean (*rescan)(FuDevice *self, GError **error) G_GNUC_WARN_UNUSED_RESULT;
 	FuFirmware *(*prepare_firmware)(FuDevice *self,
-					GBytes *fw,
+					GInputStream *stream,
+					FuProgress *progress,
 					FwupdInstallFlags flags,
 					GError **error)G_GNUC_WARN_UNUSED_RESULT;
 	gboolean (*set_quirk_kv)(FuDevice *self,
@@ -87,26 +90,179 @@ struct _FuDeviceClass {
 	void (*set_progress)(FuDevice *self, FuProgress *progress);
 	void (*invalidate)(FuDevice *self);
 	gchar *(*convert_version)(FuDevice *self, guint64 version_raw);
+	void (*register_flags)(FuDevice *self);
 #endif
 };
 
 /**
- * FuDeviceInstanceFlags:
- * @FU_DEVICE_INSTANCE_FLAG_NONE:		No flags set
- * @FU_DEVICE_INSTANCE_FLAG_VISIBLE:		Show to the user
- * @FU_DEVICE_INSTANCE_FLAG_QUIRKS:		Match against quirk files
- * @FU_DEVICE_INSTANCE_FLAG_GENERIC:		Generic GUID added by a baseclass
+ * FuDeviceIncorporateFlags:
  *
- * The flags to use when interacting with a device instance
+ * The flags to use when incorporating a device instance.
  **/
 typedef enum {
-	FU_DEVICE_INSTANCE_FLAG_NONE = 0,
-	FU_DEVICE_INSTANCE_FLAG_VISIBLE = 1 << 0,
-	FU_DEVICE_INSTANCE_FLAG_QUIRKS = 1 << 1,
-	FU_DEVICE_INSTANCE_FLAG_GENERIC = 1 << 2,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_BASECLASS:
+	 *
+	 * Set baseclass properties.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_BASECLASS = 1ull << 0,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_SUPERCLASS:
+	 *
+	 * Set superclass properties, implemented using `->incorporate()`.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_SUPERCLASS = 1ull << 1,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_VENDOR:
+	 *
+	 * Set vendor.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_VENDOR = 1ull << 2,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_VENDOR_IDS:
+	 *
+	 * Set vendor IDs.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_VENDOR_IDS = 1ull << 3,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_PHYSICAL_ID:
+	 *
+	 * Set the physical ID.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_PHYSICAL_ID = 1ull << 4,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_LOGICAL_ID:
+	 *
+	 * Set the logical ID.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_LOGICAL_ID = 1ull << 5,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_BACKEND_ID:
+	 *
+	 * Set the backend ID.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_BACKEND_ID = 1ull << 6,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_REMOVE_DELAY:
+	 *
+	 * Set the remove delay.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_REMOVE_DELAY = 1ull << 7,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_ACQUIESCE_DELAY:
+	 *
+	 * Set the acquiesce delay.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_ACQUIESCE_DELAY = 1ull << 8,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_ICONS:
+	 *
+	 * Set the icons.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_ICONS = 1ull << 9,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_UPDATE_ERROR:
+	 *
+	 * Set the update error.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_UPDATE_ERROR = 1ull << 10,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_UPDATE_STATE:
+	 *
+	 * Set the update state.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_UPDATE_STATE = 1ull << 11,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_VID:
+	 *
+	 * Set the vendor ID.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_VID = 1ull << 12,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_PID:
+	 *
+	 * Set the product ID.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_PID = 1ull << 13,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_UPDATE_MESSAGE:
+	 *
+	 * Set the update message.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_UPDATE_MESSAGE = 1ull << 14,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_UPDATE_IMAGE:
+	 *
+	 * Set the update image.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_UPDATE_IMAGE = 1ull << 15,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_EVENTS:
+	 *
+	 * Add the device events.
+	 *
+	 * Since: 2.0.0
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_EVENTS = 1ull << 16,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_INSTANCE_IDS:
+	 *
+	 * Set the instance IDs.
+	 *
+	 * Since: 2.0.4
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_INSTANCE_IDS = 1ull << 17,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_POSSIBLE_PLUGINS:
+	 *
+	 * Set the possible plugins.
+	 *
+	 * Since: 2.0.6
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_POSSIBLE_PLUGINS = 1ull << 18,
+	/**
+	 * FU_DEVICE_INCORPORATE_FLAG_GTYPE:
+	 *
+	 * Set the device GType.
+	 *
+	 * Since: 2.0.6
+	 **/
+	FU_DEVICE_INCORPORATE_FLAG_GTYPE = 1ull << 19,
 	/*< private >*/
-	FU_DEVICE_INSTANCE_FLAG_LAST
-} FuDeviceInstanceFlags;
+	FU_DEVICE_INCORPORATE_FLAG_ALL = G_MAXUINT64,
+} FuDeviceIncorporateFlags;
 
 /**
  * FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE:
@@ -144,32 +300,29 @@ FuDevice *
 fu_device_new(FuContext *ctx);
 
 /* helpful casting macros */
-#define fu_device_has_flag(d, v)	   fwupd_device_has_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_flag(d, v)	   fwupd_device_has_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_request_flag(d, v)   fwupd_device_has_request_flag(FWUPD_DEVICE(d), v)
-#define fu_device_add_request_flag(d, v)   fwupd_device_add_request_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_instance_id(d, v)	   fwupd_device_has_instance_id(FWUPD_DEVICE(d), v)
-#define fu_device_has_vendor_id(d, v)	   fwupd_device_has_vendor_id(FWUPD_DEVICE(d), v)
-#define fu_device_has_protocol(d, v)	   fwupd_device_has_protocol(FWUPD_DEVICE(d), v)
-#define fu_device_has_checksum(d, v)	   fwupd_device_has_checksum(FWUPD_DEVICE(d), v)
-#define fu_device_add_checksum(d, v)	   fwupd_device_add_checksum(FWUPD_DEVICE(d), v)
-#define fu_device_add_release(d, v)	   fwupd_device_add_release(FWUPD_DEVICE(d), v)
-#define fu_device_add_icon(d, v)	   fwupd_device_add_icon(FWUPD_DEVICE(d), v)
-#define fu_device_has_icon(d, v)	   fwupd_device_has_icon(FWUPD_DEVICE(d), v)
-#define fu_device_add_issue(d, v)	   fwupd_device_add_issue(FWUPD_DEVICE(d), v)
-#define fu_device_set_created(d, v)	   fwupd_device_set_created(FWUPD_DEVICE(d), v)
-#define fu_device_set_description(d, v)	   fwupd_device_set_description(FWUPD_DEVICE(d), v)
-#define fu_device_set_flags(d, v)	   fwupd_device_set_flags(FWUPD_DEVICE(d), v)
-#define fu_device_set_modified(d, v)	   fwupd_device_set_modified(FWUPD_DEVICE(d), v)
-#define fu_device_set_plugin(d, v)	   fwupd_device_set_plugin(FWUPD_DEVICE(d), v)
-#define fu_device_set_serial(d, v)	   fwupd_device_set_serial(FWUPD_DEVICE(d), v)
-#define fu_device_set_summary(d, v)	   fwupd_device_set_summary(FWUPD_DEVICE(d), v)
-#define fu_device_set_branch(d, v)	   fwupd_device_set_branch(FWUPD_DEVICE(d), v)
-#define fu_device_set_update_message(d, v) fwupd_device_set_update_message(FWUPD_DEVICE(d), v)
-#define fu_device_set_update_image(d, v)   fwupd_device_set_update_image(FWUPD_DEVICE(d), v)
-#define fu_device_set_update_error(d, v)   fwupd_device_set_update_error(FWUPD_DEVICE(d), v)
-#define fu_device_add_vendor_id(d, v)	   fwupd_device_add_vendor_id(FWUPD_DEVICE(d), v)
-#define fu_device_add_protocol(d, v)	   fwupd_device_add_protocol(FWUPD_DEVICE(d), v)
+#define fu_device_has_flag(d, v)	 fwupd_device_has_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_flag(d, v)	 fwupd_device_has_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_request_flag(d, v) fwupd_device_has_request_flag(FWUPD_DEVICE(d), v)
+#define fu_device_add_request_flag(d, v) fwupd_device_add_request_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_vendor_id(d, v)	 fwupd_device_has_vendor_id(FWUPD_DEVICE(d), v)
+#define fu_device_has_protocol(d, v)	 fwupd_device_has_protocol(FWUPD_DEVICE(d), v)
+#define fu_device_has_checksum(d, v)	 fwupd_device_has_checksum(FWUPD_DEVICE(d), v)
+#define fu_device_add_checksum(d, v)	 fwupd_device_add_checksum(FWUPD_DEVICE(d), v)
+#define fu_device_add_release(d, v)	 fwupd_device_add_release(FWUPD_DEVICE(d), v)
+#define fu_device_add_icon(d, v)	 fwupd_device_add_icon(FWUPD_DEVICE(d), v)
+#define fu_device_has_icon(d, v)	 fwupd_device_has_icon(FWUPD_DEVICE(d), v)
+#define fu_device_add_issue(d, v)	 fwupd_device_add_issue(FWUPD_DEVICE(d), v)
+#define fu_device_set_description(d, v)	 fwupd_device_set_description(FWUPD_DEVICE(d), v)
+#define fu_device_set_flags(d, v)	 fwupd_device_set_flags(FWUPD_DEVICE(d), v)
+#define fu_device_set_plugin(d, v)	 fwupd_device_set_plugin(FWUPD_DEVICE(d), v)
+#define fu_device_set_serial(d, v)	 fwupd_device_set_serial(FWUPD_DEVICE(d), v)
+#define fu_device_set_summary(d, v)	 fwupd_device_set_summary(FWUPD_DEVICE(d), v)
+#define fu_device_set_branch(d, v)	 fwupd_device_set_branch(FWUPD_DEVICE(d), v)
+#define fu_device_set_update_error(d, v) fwupd_device_set_update_error(FWUPD_DEVICE(d), v)
+#define fu_device_add_vendor_id(d, v)	 fwupd_device_add_vendor_id(FWUPD_DEVICE(d), v)
+#define fu_device_add_protocol(d, v)	 fwupd_device_add_protocol(FWUPD_DEVICE(d), v)
+#define fu_device_set_version_lowest_raw(d, v)                                                     \
+	fwupd_device_set_version_lowest_raw(FWUPD_DEVICE(d), v)
 #define fu_device_set_version_bootloader_raw(d, v)                                                 \
 	fwupd_device_set_version_bootloader_raw(FWUPD_DEVICE(d), v)
 #define fu_device_set_version_build_date(d, v)                                                     \
@@ -178,8 +331,6 @@ fu_device_new(FuContext *ctx);
 #define fu_device_set_install_duration(d, v) fwupd_device_set_install_duration(FWUPD_DEVICE(d), v)
 #define fu_device_get_checksums(d)	     fwupd_device_get_checksums(FWUPD_DEVICE(d))
 #define fu_device_get_flags(d)		     fwupd_device_get_flags(FWUPD_DEVICE(d))
-#define fu_device_get_created(d)	     fwupd_device_get_created(FWUPD_DEVICE(d))
-#define fu_device_get_modified(d)	     fwupd_device_get_modified(FWUPD_DEVICE(d))
 #define fu_device_get_guids(d)		     fwupd_device_get_guids(FWUPD_DEVICE(d))
 #define fu_device_get_guid_default(d)	     fwupd_device_get_guid_default(FWUPD_DEVICE(d))
 #define fu_device_get_instance_ids(d)	     fwupd_device_get_instance_ids(FWUPD_DEVICE(d))
@@ -194,8 +345,6 @@ fu_device_new(FuContext *ctx);
 #define fu_device_get_plugin(d)		     fwupd_device_get_plugin(FWUPD_DEVICE(d))
 #define fu_device_get_update_error(d)	     fwupd_device_get_update_error(FWUPD_DEVICE(d))
 #define fu_device_get_update_state(d)	     fwupd_device_get_update_state(FWUPD_DEVICE(d))
-#define fu_device_get_update_message(d)	     fwupd_device_get_update_message(FWUPD_DEVICE(d))
-#define fu_device_get_update_image(d)	     fwupd_device_get_update_image(FWUPD_DEVICE(d))
 #define fu_device_get_vendor(d)		     fwupd_device_get_vendor(FWUPD_DEVICE(d))
 #define fu_device_get_version(d)	     fwupd_device_get_version(FWUPD_DEVICE(d))
 #define fu_device_get_version_lowest(d)	     fwupd_device_get_version_lowest(FWUPD_DEVICE(d))
@@ -217,303 +366,238 @@ fu_device_new(FuContext *ctx);
 #define fu_device_set_percentage(d, v)	    fwupd_device_set_percentage(FWUPD_DEVICE(d), v)
 
 /**
- * FuDeviceInternalFlags:
- *
- * The device internal flags.
- **/
-typedef guint64 FuDeviceInternalFlags;
-
-/**
- * FU_DEVICE_INTERNAL_FLAG_NONE:
- *
- * No flags set.
- *
- * Since: 1.5.5
- */
-#define FU_DEVICE_INTERNAL_FLAG_NONE (0)
-
-/**
- * FU_DEVICE_INTERNAL_FLAG_UNKNOWN:
- *
- * Unknown flag value.
- *
- * Since: 1.5.5
- */
-#define FU_DEVICE_INTERNAL_FLAG_UNKNOWN G_MAXUINT64
-
-/**
- * FU_DEVICE_INTERNAL_FLAG_NO_AUTO_INSTANCE_IDS:
+ * FU_DEVICE_PRIVATE_FLAG_NO_AUTO_INSTANCE_IDS:
  *
  * Do not add instance IDs from the device baseclass.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_AUTO_INSTANCE_IDS (1ull << 0)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_AUTO_INSTANCE_IDS "no-auto-instance-ids"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ENSURE_SEMVER:
+ * FU_DEVICE_PRIVATE_FLAG_ENSURE_SEMVER:
  *
  * Ensure the version is a valid semantic version, e.g. numbers separated with dots.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_ENSURE_SEMVER (1ull << 1)
-
+#define FU_DEVICE_PRIVATE_FLAG_ENSURE_SEMVER "ensure-semver"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ONLY_SUPPORTED:
+ * FU_DEVICE_PRIVATE_FLAG_ONLY_SUPPORTED:
  *
  * Only devices supported in the metadata will be opened
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_ONLY_SUPPORTED (1ull << 2)
-
+#define FU_DEVICE_PRIVATE_FLAG_ONLY_SUPPORTED "only-supported"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_NAME:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_NAME:
  *
  * Set the device name from the metadata `name` if available.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_NAME (1ull << 3)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_NAME "md-set-name"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_NAME_CATEGORY:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_NAME_CATEGORY:
  *
  * Set the device name from the metadata `category` if available.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_NAME_CATEGORY (1ull << 4)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_NAME_CATEGORY "md-set-name-category"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_VERFMT:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_VERFMT:
  *
- * Set the device version format from the metadata if available.
+ * Set the device version format from the metadata or history database if available.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_VERFMT (1ull << 5)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_VERFMT "md-set-verfmt"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_ICON:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_ICON:
  *
  * Set the device icon from the metadata if available.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_ICON (1ull << 6)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_ICON "md-set-icon"
 /**
- * FU_DEVICE_INTERNAL_FLAG_RETRY_OPEN:
+ * FU_DEVICE_PRIVATE_FLAG_RETRY_OPEN:
  *
  * Retry the device open up to 5 times if it fails.
  *
  * Since: 1.5.5
  */
-#define FU_DEVICE_INTERNAL_FLAG_RETRY_OPEN (1ull << 7)
-
+#define FU_DEVICE_PRIVATE_FLAG_RETRY_OPEN "retry-open"
 /**
- * FU_DEVICE_INTERNAL_FLAG_REPLUG_MATCH_GUID:
+ * FU_DEVICE_PRIVATE_FLAG_REPLUG_MATCH_GUID:
  *
  * Match GUIDs on device replug where the physical and logical IDs will be different.
  *
  * Since: 1.5.8
  */
-#define FU_DEVICE_INTERNAL_FLAG_REPLUG_MATCH_GUID (1ull << 8)
-
+#define FU_DEVICE_PRIVATE_FLAG_REPLUG_MATCH_GUID "replug-match-guid"
 /**
- * FU_DEVICE_INTERNAL_FLAG_INHERIT_ACTIVATION:
+ * FU_DEVICE_PRIVATE_FLAG_INHERIT_ACTIVATION:
  *
  * Inherit activation status from the history database on startup.
  *
  * Since: 1.5.9
  */
-#define FU_DEVICE_INTERNAL_FLAG_INHERIT_ACTIVATION (1ull << 9)
-
+#define FU_DEVICE_PRIVATE_FLAG_INHERIT_ACTIVATION "inherit-activation"
 /**
- * FU_DEVICE_INTERNAL_FLAG_IS_OPEN:
+ * FU_DEVICE_PRIVATE_FLAG_IS_OPEN:
  *
  * The device opened successfully and ready to use.
  *
  * Since: 1.6.1
  */
-#define FU_DEVICE_INTERNAL_FLAG_IS_OPEN (1ull << 10)
-
+#define FU_DEVICE_PRIVATE_FLAG_IS_OPEN "is-open"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_SERIAL_NUMBER:
+ * FU_DEVICE_PRIVATE_FLAG_NO_SERIAL_NUMBER:
  *
  * Do not attempt to read the device serial number.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_SERIAL_NUMBER (1ull << 11)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_SERIAL_NUMBER "no-serial-number"
 /**
- * FU_DEVICE_INTERNAL_FLAG_AUTO_PARENT_CHILDREN:
+ * FU_DEVICE_PRIVATE_FLAG_AUTO_PARENT_CHILDREN:
  *
  * Automatically assign the parent for children of this device.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_AUTO_PARENT_CHILDREN (1ull << 12)
-
+#define FU_DEVICE_PRIVATE_FLAG_AUTO_PARENT_CHILDREN "auto-parent-children"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ATTACH_EXTRA_RESET:
+ * FU_DEVICE_PRIVATE_FLAG_ATTACH_EXTRA_RESET:
  *
  * Device needs resetting twice for attach after the firmware update.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_ATTACH_EXTRA_RESET (1ull << 13)
-
+#define FU_DEVICE_PRIVATE_FLAG_ATTACH_EXTRA_RESET "attach-extra-reset"
 /**
- * FU_DEVICE_INTERNAL_FLAG_INHIBIT_CHILDREN:
+ * FU_DEVICE_PRIVATE_FLAG_INHIBIT_CHILDREN:
  *
  * Children of the device are inhibited by the parent.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_INHIBIT_CHILDREN (1ull << 14)
-
+#define FU_DEVICE_PRIVATE_FLAG_INHIBIT_CHILDREN "inhibit-children"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_AUTO_REMOVE_CHILDREN:
+ * FU_DEVICE_PRIVATE_FLAG_NO_AUTO_REMOVE_CHILDREN:
  *
  * Do not auto-remove children in the device list.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_AUTO_REMOVE_CHILDREN (1ull << 15)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_AUTO_REMOVE_CHILDREN "no-auto-remove-children"
 /**
- * FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_OPEN:
+ * FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_OPEN:
  *
  * Use parent to open and close the device.
  *
  * Since: 1.6.2
  */
-#define FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_OPEN (1ull << 16)
-
+#define FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_OPEN "use-parent-for-open"
 /**
- * FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_BATTERY:
+ * FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_BATTERY:
  *
  * Use parent for the battery level and threshold.
  *
  * Since: 1.6.3
  */
-#define FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_BATTERY (1ull << 17)
-
+#define FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_BATTERY "use-parent-for-battery"
 /**
- * FU_DEVICE_INTERNAL_FLAG_USE_PROXY_FALLBACK:
+ * FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FALLBACK:
  *
  * Use parent for the battery level and threshold.
  *
  * Since: 1.6.4
  */
-#define FU_DEVICE_INTERNAL_FLAG_USE_PROXY_FALLBACK (1ull << 18)
-
+#define FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FALLBACK "use-proxy-fallback"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_AUTO_REMOVE:
+ * FU_DEVICE_PRIVATE_FLAG_NO_AUTO_REMOVE:
  *
  * The device is not auto removed.
  *
  * Since 1.7.3
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_AUTO_REMOVE (1ull << 19)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_AUTO_REMOVE "no-auto-remove"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_VENDOR:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_VENDOR:
  *
  * Set the device vendor from the metadata `developer_name` if available.
  *
  * Since: 1.7.4
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_VENDOR (1ull << 20)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_VENDOR "md-set-vendor"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_LID_CLOSED:
+ * FU_DEVICE_PRIVATE_FLAG_NO_LID_CLOSED:
  *
  * Do not allow updating when the laptop lid is closed.
  *
  * Since: 1.7.4
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_LID_CLOSED (1ull << 21)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_LID_CLOSED "no-lid-closed"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_PROBE:
+ * FU_DEVICE_PRIVATE_FLAG_NO_PROBE:
  *
  * Do not probe this device.
  *
  * Since: 1.7.6
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_PROBE (1ull << 22)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_PROBE "no-probe"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_SIGNED:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_SIGNED:
  *
  * Set the signed/unsigned payload from the metadata if available.
  *
  * Since: 1.7.6
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_SIGNED (1ull << 23)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_SIGNED "md-set-signed"
 /**
- * FU_DEVICE_INTERNAL_AUTO_PAUSE_POLLING:
+ * FU_DEVICE_PRIVATE_FLAG_AUTO_PAUSE_POLLING:
  *
  * Pause polling when reading or writing to the device
  *
  * Since: 1.8.1
  */
-#define FU_DEVICE_INTERNAL_AUTO_PAUSE_POLLING (1ull << 24)
-
+#define FU_DEVICE_PRIVATE_FLAG_AUTO_PAUSE_POLLING "auto-pause-polling"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ONLY_WAIT_FOR_REPLUG:
+ * FU_DEVICE_PRIVATE_FLAG_DELAYED_REMOVAL:
  *
- * Only use the device removal delay when explicitly waiting for a replug, rather than every time
- * the device is removed.
+ * Use the device removal delay every time the device is removed, rather than when explicitly
+ * waiting for a replug.
  *
- * Since: 1.8.1
+ * Since: 2.0.3
  */
-#define FU_DEVICE_INTERNAL_FLAG_ONLY_WAIT_FOR_REPLUG (1ull << 25)
-
+#define FU_DEVICE_PRIVATE_FLAG_DELAYED_REMOVAL "delayed-removal"
 /**
- * FU_DEVICE_INTERNAL_FLAG_IGNORE_SYSTEM_POWER:
+ * FU_DEVICE_PRIVATE_FLAG_IGNORE_SYSTEM_POWER:
  *
  * Allow updating firmware when the system power is otherwise too low.
  * This is only really useful when updating the system battery firmware.
  *
  * Since: 1.8.11
  */
-#define FU_DEVICE_INTERNAL_FLAG_IGNORE_SYSTEM_POWER (1ull << 26)
-
+#define FU_DEVICE_PRIVATE_FLAG_IGNORE_SYSTEM_POWER "ignore-system-power"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_PROBE_COMPLETE:
- *
- * Do not deallocate resources typically only required during `->probe`.
- *
- * Note: the daemon will not actually free or unref device resources, but the plugin should
- * still use this flag. After a a few releases and a lot of testing we'll actually flip the switch.
- *
- * Since: 1.8.12
- */
-#define FU_DEVICE_INTERNAL_FLAG_NO_PROBE_COMPLETE (1ull << 27)
-
-/**
- * FU_DEVICE_INTERNAL_FLAG_SAVE_INTO_BACKUP_REMOTE:
+ * FU_DEVICE_PRIVATE_FLAG_SAVE_INTO_BACKUP_REMOTE:
  *
  * Save the cabinet archive to persistent storage remote before starting the update process.
  *
- * This is useful when the network device is being updated, and different blobs inside the archive
- * could be required in different scenarios. For instance, if the user installs a firmware update
- * for a specific network device and then changes the SIM -- it might be they need the archive
- * again and have no internet access.
+ * This is useful when the network device is being updated, and different blobs inside the
+ * archive could be required in different scenarios. For instance, if the user installs a
+ * firmware update for a specific network device and then changes the SIM -- it might be
+ * they need the archive again and have no internet access.
  *
  * Since: 1.8.13
  */
-#define FU_DEVICE_INTERNAL_FLAG_SAVE_INTO_BACKUP_REMOTE (1ull << 28)
-
+#define FU_DEVICE_PRIVATE_FLAG_SAVE_INTO_BACKUP_REMOTE "save-into-backup-remote"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_FLAGS:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_FLAGS:
  *
  * Set the device flags from the metadata if available.
  *
@@ -522,19 +606,17 @@ typedef guint64 FuDeviceInternalFlags;
  *
  * Since: 1.9.1
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_FLAGS (1ull << 29)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_FLAGS "md-set-flags"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_SET_VERSION:
+ * FU_DEVICE_PRIVATE_FLAG_MD_SET_VERSION:
  *
  * Set the device version from the metadata if available.
  *
  * Since: 1.9.1
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_SET_VERSION (1ull << 30)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_SET_VERSION "md-set-version"
 /**
- * FU_DEVICE_INTERNAL_FLAG_MD_ONLY_CHECKSUM:
+ * FU_DEVICE_PRIVATE_FLAG_MD_ONLY_CHECKSUM:
  *
  * Only use the metadata *checksum* to set device attributes.
  *
@@ -542,125 +624,181 @@ typedef guint64 FuDeviceInternalFlags;
  *
  * Since: 1.9.1
  */
-#define FU_DEVICE_INTERNAL_FLAG_MD_ONLY_CHECKSUM (1ull << 31)
-
+#define FU_DEVICE_PRIVATE_FLAG_MD_ONLY_CHECKSUM "md-only-checksum"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ADD_INSTANCE_ID_REV:
+ * FU_DEVICE_PRIVATE_FLAG_ADD_INSTANCE_ID_REV:
  *
  * Add the `_REV` instance ID suffix.
  *
  * Since: 1.9.3
  */
-#define FU_DEVICE_INTERNAL_FLAG_ADD_INSTANCE_ID_REV (1ull << 32)
-
+#define FU_DEVICE_PRIVATE_FLAG_ADD_INSTANCE_ID_REV "add-instance-id-rev"
 /**
- * FU_DEVICE_INTERNAL_FLAG_UNCONNECTED:
+ * FU_DEVICE_PRIVATE_FLAG_UNCONNECTED:
  *
  * The device is not connected and is probably awaiting replug.
  *
  * Since: 1.9.4
  */
-#define FU_DEVICE_INTERNAL_FLAG_UNCONNECTED (1ull << 33)
-
+#define FU_DEVICE_PRIVATE_FLAG_UNCONNECTED "unconnected"
 /**
- * FU_DEVICE_INTERNAL_FLAG_DISPLAY_REQUIRED:
+ * FU_DEVICE_PRIVATE_FLAG_DISPLAY_REQUIRED:
  *
  * The device requires a display to be plugged in.
  *
  * Since: 1.9.6
  */
-#define FU_DEVICE_INTERNAL_FLAG_DISPLAY_REQUIRED (1ull << 34)
-
+#define FU_DEVICE_PRIVATE_FLAG_DISPLAY_REQUIRED "display-required"
 /**
- * FU_DEVICE_INTERNAL_FLAG_UPDATE_PENDING:
+ * FU_DEVICE_PRIVATE_FLAG_UPDATE_PENDING:
  *
  * The device has an update that is waiting to be applied.
  *
  * Since: 1.9.7
  */
-#define FU_DEVICE_INTERNAL_FLAG_UPDATE_PENDING (1ull << 35)
-
+#define FU_DEVICE_PRIVATE_FLAG_UPDATE_PENDING "update-pending"
 /**
- * FU_DEVICE_INTERNAL_FLAG_NO_GENERIC_GUIDS:
+ * FU_DEVICE_PRIVATE_FLAG_NO_GENERIC_GUIDS:
  *
  * Do not add generic GUIDs from outside the plugin.
  *
  * Since: 1.9.8
  */
-#define FU_DEVICE_INTERNAL_FLAG_NO_GENERIC_GUIDS (1ull << 36)
-
+#define FU_DEVICE_PRIVATE_FLAG_NO_GENERIC_GUIDS "no-generic-guids"
 /**
- * FU_DEVICE_INTERNAL_FLAG_ENFORCE_REQUIRES:
+ * FU_DEVICE_PRIVATE_FLAG_ENFORCE_REQUIRES:
  *
- * The device uses a generic instance ID and firmware requires a parent, child, sibling or CHID
- * requirement.
+ * The device uses a generic instance ID and firmware requires a parent, child, sibling or
+ * CHID requirement.
  *
  * Since: 1.9.8
  */
-#define FU_DEVICE_INTERNAL_FLAG_ENFORCE_REQUIRES (1ull << 37)
-
+#define FU_DEVICE_PRIVATE_FLAG_ENFORCE_REQUIRES "enforce-requires"
 /**
- * FU_DEVICE_INTERNAL_FLAG_HOST_FIRMWARE:
+ * FU_DEVICE_PRIVATE_FLAG_HOST_FIRMWARE:
  *
  * The device represents the main system host firmware.
  *
  * Since: 1.9.10
  */
-#define FU_DEVICE_INTERNAL_FLAG_HOST_FIRMWARE (1ull << 39)
-
+#define FU_DEVICE_PRIVATE_FLAG_HOST_FIRMWARE "host-firmware"
 /**
- * FU_DEVICE_INTERNAL_FLAG_HOST_FIRMWARE_CHILD:
+ * FU_DEVICE_PRIVATE_FLAG_HOST_FIRMWARE_CHILD:
  *
  * The device should be a child of the main system host firmware device.
  *
  * Since: 1.9.10
  */
-#define FU_DEVICE_INTERNAL_FLAG_HOST_FIRMWARE_CHILD (1ull << 40)
-
+#define FU_DEVICE_PRIVATE_FLAG_HOST_FIRMWARE_CHILD "host-firmware-child"
 /**
- * FU_DEVICE_INTERNAL_FLAG_HOST_CPU:
+ * FU_DEVICE_PRIVATE_FLAG_HOST_CPU:
  *
  * The device represents the main CPU device.
  *
  * Since: 1.9.10
  */
-#define FU_DEVICE_INTERNAL_FLAG_HOST_CPU (1ull << 41)
-
+#define FU_DEVICE_PRIVATE_FLAG_HOST_CPU "host-cpu"
 /**
- * FU_DEVICE_INTERNAL_FLAG_HOST_CPU_CHILD:
+ * FU_DEVICE_PRIVATE_FLAG_HOST_CPU_CHILD:
  *
  * The device should be a child of the main CPU device.
  *
  * Since: 1.9.10
  */
-#define FU_DEVICE_INTERNAL_FLAG_HOST_CPU_CHILD (1ull << 42)
-
+#define FU_DEVICE_PRIVATE_FLAG_HOST_CPU_CHILD "host-cpu-child"
 /**
- * FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER:
+ * FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER:
  *
  * Do not automatically set the device order, e.g. updating the child before the parent.
  *
  * Since: 1.9.13
  */
-#define FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER (1ull << 43)
-
+#define FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER "explicit-order"
 /**
- * FU_DEVICE_INTERNAL_FLAG_REFCOUNTED_PROXY:
+ * FU_DEVICE_PRIVATE_FLAG_REFCOUNTED_PROXY:
  *
  * Reference-count the proxy -- which is useful when using `ProxyGType`.
  *
  * Since: 1.9.15
  */
-#define FU_DEVICE_INTERNAL_FLAG_REFCOUNTED_PROXY (1ull << 44)
-
+#define FU_DEVICE_PRIVATE_FLAG_REFCOUNTED_PROXY "refcounted-proxy"
 /**
- * FU_DEVICE_INTERNAL_FLAG_USE_PROXY_FOR_OPEN:
+ * FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FOR_OPEN:
  *
  * Use proxy to open and close the device.
  *
  * Since: 1.9.16
  */
-#define FU_DEVICE_INTERNAL_FLAG_USE_PROXY_FOR_OPEN (1ull << 45)
+#define FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FOR_OPEN "use-proxy-for-open"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_INSTALL_PARENT_FIRST:
+ *
+ * The composite device requires installation of composite firmware on the parent before
+ * the child.
+ * Normally the child is installed before the parent.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_INSTALL_PARENT_FIRST "install-parent-first"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_REGISTERED:
+ *
+ * The device has been registered with other plugins.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_REGISTERED "registered"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_ADD_COUNTERPART_GUIDS:
+ *
+ * The device will add counterpart GUIDs from an alternate mode like bootloader.
+ * This flag is typically specified in a quirk.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_ADD_COUNTERPART_GUIDS "add-counterpart-guids"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_USE_RUNTIME_VERSION:
+ *
+ * The device will always display use the runtime version rather than the bootloader
+ * version.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_USE_RUNTIME_VERSION "use-runtime-version"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART:
+ *
+ * The device relies upon activation or power cycle to load firmware.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART "skips-restart"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_IS_FAKE:
+ *
+ * The device is fake, and constructed from a fake sysfs test directory.
+ *
+ * Since: 2.0.0
+ */
+#define FU_DEVICE_PRIVATE_FLAG_IS_FAKE "is-fake"
+
+/**
+ * FU_DEVICE_PRIVATE_FLAG_COUNTERPART_VISIBLE:
+ *
+ * Add the counterpart instance IDs as visible GUID that can be matched in firmware.
+ *
+ * This was how the daemon worked in 1.9.x, and some devices in recovery mode still expect this.
+ *
+ * Since: 2.0.4
+ */
+#define FU_DEVICE_PRIVATE_FLAG_COUNTERPART_VISIBLE "counterpart-visible"
 
 /* accessors */
 gchar *
@@ -668,32 +806,28 @@ fu_device_to_string(FuDevice *self) G_GNUC_NON_NULL(1);
 void
 fu_device_add_string(FuDevice *self, guint idt, GString *str) G_GNUC_NON_NULL(1, 3);
 const gchar *
-fu_device_get_alternate_id(FuDevice *self) G_GNUC_NON_NULL(1);
-void
-fu_device_set_alternate_id(FuDevice *self, const gchar *alternate_id) G_GNUC_NON_NULL(1, 2);
-const gchar *
 fu_device_get_equivalent_id(FuDevice *self) G_GNUC_NON_NULL(1);
 void
 fu_device_set_equivalent_id(FuDevice *self, const gchar *equivalent_id) G_GNUC_NON_NULL(1, 2);
-void
-fu_device_add_guid(FuDevice *self, const gchar *guid) G_GNUC_NON_NULL(1, 2);
-void
-fu_device_add_guid_full(FuDevice *self, const gchar *guid, FuDeviceInstanceFlags flags)
-    G_GNUC_NON_NULL(1, 2);
 gboolean
 fu_device_has_guid(FuDevice *self, const gchar *guid) G_GNUC_NON_NULL(1);
 void
-fu_device_add_instance_id(FuDevice *self, const gchar *instance_id) G_GNUC_NON_NULL(1);
+fu_device_add_instance_id(FuDevice *self, const gchar *instance_id) G_GNUC_NON_NULL(1, 2);
+gboolean
+fu_device_has_instance_id(FuDevice *self, const gchar *instance_id, FuDeviceInstanceFlag flags)
+    G_GNUC_NON_NULL(1, 2);
 void
-fu_device_add_instance_id_full(FuDevice *self,
-			       const gchar *instance_id,
-			       FuDeviceInstanceFlags flags) G_GNUC_NON_NULL(1, 2);
-FuDevice *
-fu_device_get_alternate(FuDevice *self) G_GNUC_NON_NULL(1);
+fu_device_add_instance_id_full(FuDevice *self, const gchar *instance_id, FuDeviceInstanceFlag flags)
+    G_GNUC_NON_NULL(1, 2);
 FuDevice *
 fu_device_get_root(FuDevice *self) G_GNUC_NON_NULL(1);
 FuDevice *
 fu_device_get_parent(FuDevice *self) G_GNUC_NON_NULL(1);
+FuDevice *
+fu_device_get_backend_parent(FuDevice *self, GError **error) G_GNUC_NON_NULL(1);
+FuDevice *
+fu_device_get_backend_parent_with_subsystem(FuDevice *self, const gchar *subsystem, GError **error)
+    G_GNUC_NON_NULL(1);
 GPtrArray *
 fu_device_get_children(FuDevice *self) G_GNUC_NON_NULL(1);
 void
@@ -706,8 +840,6 @@ void
 fu_device_add_parent_physical_id(FuDevice *self, const gchar *physical_id) G_GNUC_NON_NULL(1, 2);
 void
 fu_device_add_parent_backend_id(FuDevice *self, const gchar *backend_id) G_GNUC_NON_NULL(1, 2);
-void
-fu_device_add_counterpart_guid(FuDevice *self, const gchar *guid) G_GNUC_NON_NULL(1, 2);
 FuDevice *
 fu_device_get_proxy(FuDevice *self) G_GNUC_NON_NULL(1);
 void
@@ -742,16 +874,6 @@ fu_device_set_version_bootloader(FuDevice *self, const gchar *version) G_GNUC_NO
 void
 fu_device_set_version_raw(FuDevice *self, guint64 version_raw) G_GNUC_NON_NULL(1);
 void
-fu_device_set_version_u16(FuDevice *self, guint16 version_raw) G_GNUC_NON_NULL(1);
-void
-fu_device_set_version_u24(FuDevice *self, guint32 version_raw) G_GNUC_NON_NULL(1);
-void
-fu_device_set_version_u32(FuDevice *self, guint32 version_raw) G_GNUC_NON_NULL(1);
-void
-fu_device_set_version_u64(FuDevice *self, guint64 version_raw) G_GNUC_NON_NULL(1);
-void
-fu_device_set_version_lowest_raw(FuDevice *self, guint64 version_raw) G_GNUC_NON_NULL(1);
-void
 fu_device_inhibit(FuDevice *self, const gchar *inhibit_id, const gchar *reason)
     G_GNUC_NON_NULL(1, 2);
 void
@@ -788,10 +910,6 @@ void
 fu_device_add_flag(FuDevice *self, FwupdDeviceFlags flag) G_GNUC_NON_NULL(1);
 void
 fu_device_remove_flag(FuDevice *self, FwupdDeviceFlags flag) G_GNUC_NON_NULL(1);
-const gchar *
-fu_device_get_custom_flags(FuDevice *self) G_GNUC_NON_NULL(1);
-void
-fu_device_set_custom_flags(FuDevice *self, const gchar *custom_flags) G_GNUC_NON_NULL(1);
 void
 fu_device_set_name(FuDevice *self, const gchar *value) G_GNUC_NON_NULL(1);
 void
@@ -822,6 +940,33 @@ guint
 fu_device_get_battery_threshold(FuDevice *self) G_GNUC_NON_NULL(1);
 void
 fu_device_set_battery_threshold(FuDevice *self, guint battery_threshold) G_GNUC_NON_NULL(1);
+const gchar *
+fu_device_get_update_message(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_update_message(FuDevice *self, const gchar *update_message) G_GNUC_NON_NULL(1);
+const gchar *
+fu_device_get_update_image(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_update_image(FuDevice *self, const gchar *update_image) G_GNUC_NON_NULL(1);
+
+gint64
+fu_device_get_created_usec(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_created_usec(FuDevice *self, gint64 created_usec) G_GNUC_NON_NULL(1);
+gint64
+fu_device_get_modified_usec(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_modified_usec(FuDevice *self, gint64 modified_usec) G_GNUC_NON_NULL(1);
+
+guint16
+fu_device_get_vid(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_vid(FuDevice *self, guint16 vid);
+guint16
+fu_device_get_pid(FuDevice *self) G_GNUC_NON_NULL(1);
+void
+fu_device_set_pid(FuDevice *self, guint16 pid);
+
 void
 fu_device_set_update_state(FuDevice *self, FwupdUpdateState update_state) G_GNUC_NON_NULL(1);
 void
@@ -836,23 +981,20 @@ GType
 fu_device_get_firmware_gtype(FuDevice *self) G_GNUC_NON_NULL(1);
 void
 fu_device_set_firmware_gtype(FuDevice *self, GType firmware_gtype) G_GNUC_NON_NULL(1);
-void
-fu_device_add_internal_flag(FuDevice *self, FuDeviceInternalFlags flag) G_GNUC_NON_NULL(1);
-void
-fu_device_remove_internal_flag(FuDevice *self, FuDeviceInternalFlags flag) G_GNUC_NON_NULL(1);
-gboolean
-fu_device_has_internal_flag(FuDevice *self, FuDeviceInternalFlags flag) G_GNUC_NON_NULL(1);
 gboolean
 fu_device_get_results(FuDevice *self, GError **error) G_GNUC_NON_NULL(1);
 gboolean
 fu_device_write_firmware(FuDevice *self,
-			 GBytes *fw,
+			 GInputStream *stream,
 			 FuProgress *progress,
 			 FwupdInstallFlags flags,
 			 GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2, 3);
 FuFirmware *
-fu_device_prepare_firmware(FuDevice *self, GBytes *fw, FwupdInstallFlags flags, GError **error)
-    G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1);
+fu_device_prepare_firmware(FuDevice *self,
+			   GInputStream *stream,
+			   FuProgress *progress,
+			   FwupdInstallFlags flags,
+			   GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2, 3);
 FuFirmware *
 fu_device_read_firmware(FuDevice *self,
 			FuProgress *progress,
@@ -882,7 +1024,8 @@ gboolean
 fu_device_cleanup(FuDevice *self, FuProgress *progress, FwupdInstallFlags flags, GError **error)
     G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2);
 void
-fu_device_incorporate(FuDevice *self, FuDevice *donor) G_GNUC_NON_NULL(1);
+fu_device_incorporate(FuDevice *self, FuDevice *donor, FuDeviceIncorporateFlags flag)
+    G_GNUC_NON_NULL(1, 2);
 void
 fu_device_incorporate_flag(FuDevice *self, FuDevice *donor, FwupdDeviceFlags flag)
     G_GNUC_NON_NULL(1);
@@ -942,20 +1085,40 @@ fu_device_report_metadata_post(FuDevice *self) G_GNUC_NON_NULL(1);
 void
 fu_device_add_security_attrs(FuDevice *self, FuSecurityAttrs *attrs) G_GNUC_NON_NULL(1, 2);
 void
-fu_device_register_private_flag(FuDevice *self, guint64 value, const gchar *value_str)
-    G_GNUC_NON_NULL(1, 3);
+fu_device_register_private_flag(FuDevice *self, const gchar *flag) G_GNUC_NON_NULL(1, 2);
 void
-fu_device_add_private_flag(FuDevice *self, guint64 flag) G_GNUC_NON_NULL(1);
+fu_device_add_private_flag(FuDevice *self, const gchar *flag) G_GNUC_NON_NULL(1, 2);
 void
-fu_device_remove_private_flag(FuDevice *self, guint64 flag) G_GNUC_NON_NULL(1);
+fu_device_remove_private_flag(FuDevice *self, const gchar *flag) G_GNUC_NON_NULL(1, 2);
 gboolean
-fu_device_has_private_flag(FuDevice *self, guint64 flag) G_GNUC_NON_NULL(1);
+fu_device_has_private_flag(FuDevice *self, const gchar *flag) G_GNUC_NON_NULL(1, 2);
 gboolean
 fu_device_emit_request(FuDevice *self, FwupdRequest *request, FuProgress *progress, GError **error)
     G_GNUC_NON_NULL(1, 2);
 FwupdSecurityAttr *
 fu_device_security_attr_new(FuDevice *self, const gchar *appstream_id) G_GNUC_NON_NULL(1, 2);
 
+gboolean
+fu_device_set_contents(FuDevice *self,
+		       const gchar *filename,
+		       GInputStream *stream,
+		       FuProgress *progress,
+		       GError **error) G_GNUC_NON_NULL(1, 2, 3);
+gboolean
+fu_device_set_contents_bytes(FuDevice *self,
+			     const gchar *filename,
+			     GBytes *blob,
+			     FuProgress *progress,
+			     GError **error) G_GNUC_NON_NULL(1, 2, 3);
+GBytes *
+fu_device_get_contents_bytes(FuDevice *self,
+			     const gchar *filename,
+			     FuProgress *progress,
+			     GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2);
+gboolean
+fu_device_query_file_exists(FuDevice *self, const gchar *filename, gboolean *exists, GError **error)
+    G_GNUC_NON_NULL(1, 2, 3);
+
 const gchar *
 fu_device_get_instance_str(FuDevice *self, const gchar *key) G_GNUC_NON_NULL(1, 2);
 void
@@ -980,9 +1143,18 @@ fu_device_build_instance_id(FuDevice *self, GError **error, const gchar *subsyst
     G_GNUC_NULL_TERMINATED G_GNUC_NON_NULL(1, 3);
 gboolean
 fu_device_build_instance_id_full(FuDevice *self,
-				 FuDeviceInstanceFlags flags,
+				 FuDeviceInstanceFlag flags,
 				 GError **error,
 				 const gchar *subsystem,
 				 ...) G_GNUC_NULL_TERMINATED G_GNUC_NON_NULL(1, 4);
+void
+fu_device_build_vendor_id(FuDevice *self, const gchar *prefix, const gchar *value);
+void
+fu_device_build_vendor_id_u16(FuDevice *self, const gchar *prefix, guint16 value);
 FuDeviceLocker *
 fu_device_poll_locker_new(FuDevice *self, GError **error) G_GNUC_NON_NULL(1);
+
+FuDeviceEvent *
+fu_device_save_event(FuDevice *self, const gchar *id);
+FuDeviceEvent *
+fu_device_load_event(FuDevice *self, const gchar *id, GError **error);
