@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuCommon"
@@ -9,7 +9,10 @@
 #include "config.h"
 
 #include "fu-byte-array.h"
+#include "fu-chunk-array.h"
+#include "fu-input-stream.h"
 #include "fu-mem.h"
+#include "fu-partial-input-stream.h"
 #include "fu-string.h"
 
 /**
@@ -18,59 +21,78 @@
  * @value: (out) (nullable): parsed value
  * @min: minimum acceptable value, typically 0
  * @max: maximum acceptable value, typically G_MAXUINT64
+ * @base: default log base, usually %FU_INTEGER_BASE_AUTO
  * @error: (nullable): optional return location for an error
  *
- * Converts a string value to an integer. Values are assumed base 10, unless
- * prefixed with "0x" where they are parsed as base 16.
+ * Converts a string value to an integer. If the @value is prefixed with `0x` then the base is
+ * set to 16 automatically.
  *
  * Returns: %TRUE if the value was parsed correctly, or %FALSE for error
  *
- * Since: 1.8.2
+ * Since: 2.0.0
  **/
 gboolean
-fu_strtoull(const gchar *str, guint64 *value, guint64 min, guint64 max, GError **error)
+fu_strtoull(const gchar *str,
+	    guint64 *value,
+	    guint64 min,
+	    guint64 max,
+	    FuIntegerBase base,
+	    GError **error)
 {
 	gchar *endptr = NULL;
 	guint64 value_tmp;
-	guint base = 10;
 
 	/* sanity check */
 	if (str == NULL) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "cannot parse NULL");
 		return FALSE;
 	}
 
 	/* detect hex */
-	if (g_str_has_prefix(str, "0x")) {
+	if (base == FU_INTEGER_BASE_AUTO) {
+		if (g_str_has_prefix(str, "0x")) {
+			str += 2;
+			base = FU_INTEGER_BASE_16;
+		} else {
+			base = FU_INTEGER_BASE_10;
+		}
+	} else if (base == FU_INTEGER_BASE_16 && g_str_has_prefix(str, "0x")) {
 		str += 2;
-		base = 16;
+	} else if (base == FU_INTEGER_BASE_10 && g_str_has_prefix(str, "0x")) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "cannot parse 0x-prefixed base-10 string");
+		return FALSE;
 	}
 
 	/* convert */
-	value_tmp = g_ascii_strtoull(str, &endptr, base);
+	value_tmp = g_ascii_strtoull(str, &endptr, base); /* nocheck:blocked */
 	if ((gsize)(endptr - str) != strlen(str) && *endptr != '\n') {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "cannot parse %s", str);
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "cannot parse datastream");
 		return FALSE;
 	}
 
 	/* overflow check */
 	if (value_tmp == G_MAXUINT64) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "cannot parse %s as caused overflow",
-			    str);
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "parsing datastream caused overflow");
 		return FALSE;
 	}
 
 	/* range check */
 	if (value_tmp < min) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "value %" G_GUINT64_FORMAT " was below minimum %" G_GUINT64_FORMAT,
 			    value_tmp,
 			    min);
@@ -78,8 +100,8 @@ fu_strtoull(const gchar *str, guint64 *value, guint64 min, guint64 max, GError *
 	}
 	if (value_tmp > max) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "value %" G_GUINT64_FORMAT " was above maximum %" G_GUINT64_FORMAT,
 			    value_tmp,
 			    max);
@@ -98,6 +120,7 @@ fu_strtoull(const gchar *str, guint64 *value, guint64 min, guint64 max, GError *
  * @value: (out) (nullable): parsed value
  * @min: minimum acceptable value, typically 0
  * @max: maximum acceptable value, typically G_MAXINT64
+ * @base: default log base, usually %FU_INTEGER_BASE_AUTO
  * @error: (nullable): optional return location for an error
  *
  * Converts a string value to an integer. Values are assumed base 10, unless
@@ -105,42 +128,58 @@ fu_strtoull(const gchar *str, guint64 *value, guint64 min, guint64 max, GError *
  *
  * Returns: %TRUE if the value was parsed correctly, or %FALSE for error
  *
- * Since: 1.9.7
+ * Since: 2.0.0
  **/
 gboolean
-fu_strtoll(const gchar *str, gint64 *value, gint64 min, gint64 max, GError **error)
+fu_strtoll(const gchar *str,
+	   gint64 *value,
+	   gint64 min,
+	   gint64 max,
+	   FuIntegerBase base,
+	   GError **error)
 {
 	gchar *endptr = NULL;
 	gint64 value_tmp;
-	guint base = 10;
 
 	/* sanity check */
 	if (str == NULL) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "cannot parse NULL");
 		return FALSE;
 	}
 
 	/* detect hex */
-	if (g_str_has_prefix(str, "0x")) {
+	if (base == FU_INTEGER_BASE_AUTO) {
+		if (g_str_has_prefix(str, "0x")) {
+			str += 2;
+			base = FU_INTEGER_BASE_16;
+		} else {
+			base = FU_INTEGER_BASE_10;
+		}
+	} else if (base == FU_INTEGER_BASE_16 && g_str_has_prefix(str, "0x")) {
 		str += 2;
-		base = 16;
+	} else if (base == FU_INTEGER_BASE_10 && g_str_has_prefix(str, "0x")) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "cannot parse 0x-prefixed base-10 string");
+		return FALSE;
 	}
 
 	/* convert */
-	value_tmp = g_ascii_strtoll(str, &endptr, base);
+	value_tmp = g_ascii_strtoll(str, &endptr, base); /* nocheck:blocked */
 	if ((gsize)(endptr - str) != strlen(str) && *endptr != '\n') {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "cannot parse %s", str);
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA, "cannot parse %s", str);
 		return FALSE;
 	}
 
 	/* overflow check */
 	if (value_tmp == G_MAXINT64) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "cannot parse %s as caused overflow",
 			    str);
 		return FALSE;
@@ -149,8 +188,8 @@ fu_strtoll(const gchar *str, gint64 *value, gint64 min, gint64 max, GError **err
 	/* range check */
 	if (value_tmp < min) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "value %" G_GINT64_FORMAT " was below minimum %" G_GINT64_FORMAT,
 			    value_tmp,
 			    min);
@@ -158,8 +197,8 @@ fu_strtoll(const gchar *str, gint64 *value, gint64 min, gint64 max, GError **err
 	}
 	if (value_tmp > max) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "value %" G_GINT64_FORMAT " was above maximum %" G_GINT64_FORMAT,
 			    value_tmp,
 			    max);
@@ -190,8 +229,8 @@ fu_strtobool(const gchar *str, gboolean *value, GError **error)
 	/* sanity check */
 	if (str == NULL) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "cannot parse NULL");
 		return FALSE;
 	}
@@ -210,8 +249,8 @@ fu_strtobool(const gchar *str, gboolean *value, GError **error)
 
 	/* invalid */
 	g_set_error(error,
-		    G_IO_ERROR,
-		    G_IO_ERROR_INVALID_DATA,
+		    FWUPD_ERROR,
+		    FWUPD_ERROR_INVALID_DATA,
 		    "cannot parse %s as boolean, expected true|false",
 		    str);
 	return FALSE;
@@ -253,36 +292,6 @@ fu_strstrip(const gchar *str)
 	return g_strndup(str + head, tail - head + 1);
 }
 
-/**
- * fu_string_strip:
- * @str: a #GString, e.g. ` test `
- *
- * Removes leading and trailing whitespace from a mutable string.
- *
- * Since: 2.0.17
- **/
-void
-fu_string_strip(GString *str)
-{
-	guint i;
-
-	/* leading whitespace */
-	for (i = 0; i < str->len; i++) {
-		if (!g_ascii_isspace(str->str[i]))
-			break;
-	}
-	if (i > 0)
-		g_string_erase(str, 0, i);
-
-	/* trailing whitespace */
-	for (i = 0; i < str->len; i++) {
-		if (!g_ascii_isspace(str->str[str->len - (i + 1)]))
-			break;
-	}
-	if (i < str->len)
-		g_string_truncate(str, str->len - i);
-}
-
 /**
  * fu_strdup:
  * @str: a string, e.g. ` test `
@@ -341,107 +350,6 @@ fu_strwidth(const gchar *text)
 	return width;
 }
 
-/**
- * fu_string_append:
- * @str: a #GString
- * @idt: the indent
- * @key: a string to append
- * @value: a string to append
- *
- * Appends a key and string value to a string
- *
- * Since: 1.8.2
- */
-void
-fu_string_append(GString *str, guint idt, const gchar *key, const gchar *value)
-{
-	const guint align = 24;
-	gsize keysz;
-
-	g_return_if_fail(idt * 2 < align);
-
-	/* ignore */
-	if (key == NULL)
-		return;
-	for (gsize i = 0; i < idt; i++)
-		g_string_append(str, "  ");
-	if (key[0] != '\0') {
-		g_string_append_printf(str, "%s:", key);
-		keysz = (idt * 2) + fu_strwidth(key) + 1;
-	} else {
-		keysz = idt * 2;
-	}
-	if (value != NULL) {
-		g_auto(GStrv) split = NULL;
-		split = g_strsplit(value, "\n", -1);
-		for (guint i = 0; split[i] != NULL; i++) {
-			if (i == 0) {
-				for (gsize j = keysz; j < align; j++)
-					g_string_append(str, " ");
-			} else {
-				g_string_append(str, "\n");
-				for (gsize j = 0; j < idt; j++)
-					g_string_append(str, "  ");
-			}
-			g_string_append(str, split[i]);
-		}
-	}
-	g_string_append(str, "\n");
-}
-
-/**
- * fu_string_append_ku:
- * @str: a #GString
- * @idt: the indent
- * @key: a string to append
- * @value: guint64
- *
- * Appends a key and unsigned integer to a string
- *
- * Since: 1.8.2
- */
-void
-fu_string_append_ku(GString *str, guint idt, const gchar *key, guint64 value)
-{
-	g_autofree gchar *tmp = g_strdup_printf("%" G_GUINT64_FORMAT, value);
-	fu_string_append(str, idt, key, tmp);
-}
-
-/**
- * fu_string_append_kx:
- * @str: a #GString
- * @idt: the indent
- * @key: a string to append
- * @value: guint64
- *
- * Appends a key and hex integer to a string
- *
- * Since: 1.8.2
- */
-void
-fu_string_append_kx(GString *str, guint idt, const gchar *key, guint64 value)
-{
-	g_autofree gchar *tmp = g_strdup_printf("0x%x", (guint)value);
-	fu_string_append(str, idt, key, tmp);
-}
-
-/**
- * fu_string_append_kb:
- * @str: a #GString
- * @idt: the indent
- * @key: a string to append
- * @value: Boolean
- *
- * Appends a key and boolean value to a string
- *
- * Since: 1.8.2
- */
-void
-fu_string_append_kb(GString *str, guint idt, const gchar *key, gboolean value)
-{
-	fu_string_append(str, idt, key, value ? "true" : "false");
-}
-
 /**
  * fu_strsplit:
  * @str: (not nullable): a string to split
@@ -469,6 +377,131 @@ fu_strsplit(const gchar *str, gsize sz, const gchar *delimiter, gint max_tokens)
 	return g_strsplit(str, delimiter, max_tokens);
 }
 
+typedef struct {
+	FuStrsplitFunc callback;
+	gpointer user_data;
+	guint token_idx;
+	const gchar *delimiter;
+	gsize delimiter_sz;
+	gboolean detected_nul;
+	gboolean more_chunks;
+} FuStrsplitHelper;
+
+static gboolean
+fu_strsplit_buffer_drain(GByteArray *buf, FuStrsplitHelper *helper, GError **error)
+{
+	gsize buf_offset = 0;
+	while (buf_offset <= buf->len) {
+		gsize offset;
+		g_autoptr(GString) token = g_string_new(NULL);
+
+		/* find first match in buffer, starting at the buffer offset */
+		for (offset = buf_offset; offset < buf->len; offset++) {
+			if (buf->data[offset] == 0x0) {
+				helper->detected_nul = TRUE;
+				break;
+			}
+			if (strncmp((const gchar *)buf->data + offset,
+				    helper->delimiter,
+				    helper->delimiter_sz) == 0)
+				break;
+		}
+
+		/* no token found, keep going */
+		if (helper->more_chunks && offset == buf->len)
+			break;
+
+		/* sanity check is valid UTF-8 */
+		g_string_append_len(token,
+				    (const gchar *)buf->data + buf_offset,
+				    offset - buf_offset);
+		if (!g_utf8_validate_len(token->str, token->len, NULL)) {
+			g_debug("ignoring invalid UTF-8, got: %s", token->str);
+		} else {
+			if (!helper->callback(token, helper->token_idx++, helper->user_data, error))
+				return FALSE;
+		}
+		if (helper->detected_nul) {
+			buf_offset = buf->len;
+			break;
+		}
+		buf_offset = offset + helper->delimiter_sz;
+	}
+	g_byte_array_remove_range(buf, 0, MIN(buf_offset, buf->len));
+	return TRUE;
+}
+
+/**
+ * fu_strsplit_stream:
+ * @stream: a #GInputStream to split
+ * @offset: offset into @stream
+ * @delimiter: a string which specifies the places at which to split the string
+ * @callback: (scope call) (closure user_data): a #FuStrsplitFunc.
+ * @user_data: user data
+ * @error: (nullable): optional return location for an error
+ *
+ * Splits the string, calling the given function for each
+ * of the tokens found. If any @callback returns %FALSE scanning is aborted.
+ *
+ * Use this function in preference to fu_strsplit() when the input file is untrusted,
+ * and you don't want to allocate a GStrv with billions of one byte items.
+ *
+ * Returns: %TRUE if no @callback returned FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+fu_strsplit_stream(GInputStream *stream,
+		   gsize offset,
+		   const gchar *delimiter,
+		   FuStrsplitFunc callback,
+		   gpointer user_data,
+		   GError **error)
+{
+	g_autoptr(FuChunkArray) chunks = NULL;
+	g_autoptr(GByteArray) buf = g_byte_array_new();
+	g_autoptr(GInputStream) stream_partial = NULL;
+	FuStrsplitHelper helper = {
+	    .callback = callback,
+	    .user_data = user_data,
+	    .delimiter = delimiter,
+	    .token_idx = 0,
+	};
+
+	g_return_val_if_fail(G_IS_INPUT_STREAM(stream), FALSE);
+	g_return_val_if_fail(delimiter != NULL && delimiter[0] != '\0', FALSE);
+	g_return_val_if_fail(callback != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	helper.delimiter_sz = strlen(delimiter);
+	if (offset > 0) {
+		stream_partial = fu_partial_input_stream_new(stream, offset, G_MAXSIZE, error);
+		if (stream_partial == NULL)
+			return FALSE;
+	} else {
+		stream_partial = g_object_ref(stream);
+	}
+	chunks = fu_chunk_array_new_from_stream(stream_partial,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						0x8000,
+						error);
+	if (chunks == NULL)
+		return FALSE;
+	for (gsize i = 0; i < fu_chunk_array_length(chunks); i++) {
+		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i, error);
+		if (chk == NULL)
+			return FALSE;
+		g_byte_array_append(buf, fu_chunk_get_data(chk), fu_chunk_get_data_sz(chk));
+		helper.more_chunks = i != fu_chunk_array_length(chunks) - 1;
+		if (!fu_strsplit_buffer_drain(buf, &helper, error))
+			return FALSE;
+		if (helper.detected_nul)
+			break;
+	}
+	return TRUE;
+}
+
 /**
  * fu_strsplit_full:
  * @str: a string to split
@@ -497,8 +530,8 @@ fu_strsplit_full(const gchar *str,
 		 GError **error)
 {
 	gsize delimiter_sz;
+	gsize offset_old = 0;
 	gsize str_sz;
-	guint found_idx = 0;
 	guint token_idx = 0;
 
 	g_return_val_if_fail(str != NULL, FALSE);
@@ -517,25 +550,18 @@ fu_strsplit_full(const gchar *str,
 	}
 
 	/* start splittin' */
-	for (gsize i = 0; i < (str_sz - delimiter_sz) + 1;) {
-		if (strncmp(str + i, delimiter, delimiter_sz) == 0) {
-			g_autoptr(GString) token = g_string_new(NULL);
-			g_string_append_len(token, str + found_idx, i - found_idx);
-			if (!callback(token, token_idx++, user_data, error))
-				return FALSE;
-			i += delimiter_sz;
-			found_idx = i;
-		} else {
-			i++;
-		}
-	}
-
-	/* any bits left over? */
-	if (found_idx != str_sz) {
+	while (offset_old <= str_sz) {
+		gsize offset;
 		g_autoptr(GString) token = g_string_new(NULL);
-		g_string_append_len(token, str + found_idx, str_sz - found_idx);
-		if (!callback(token, token_idx, user_data, error))
+
+		for (offset = offset_old; offset < str_sz; offset++) {
+			if (strncmp(str + offset, delimiter, delimiter_sz) == 0)
+				break;
+		}
+		g_string_append_len(token, str + offset_old, offset - offset_old);
+		if (!callback(token, token_idx++, user_data, error))
 			return FALSE;
+		offset_old = offset + delimiter_sz;
 	}
 
 	/* success */
@@ -581,6 +607,25 @@ fu_strsafe(const gchar *str, gsize maxsz)
 	return g_string_free(g_steal_pointer(&tmp), FALSE);
 }
 
+/**
+ * fu_strsafe_bytes:
+ * @blob: (not nullable): a #GBytes
+ * @maxsz: maximum size of returned string
+ *
+ * Converts a #GBytes into something that can be safely printed.
+ *
+ * Returns: (transfer full): safe string, or %NULL if there was nothing valid
+ *
+ * Since: 2.0.2
+ **/
+gchar *
+fu_strsafe_bytes(GBytes *blob, gsize maxsz)
+{
+	g_return_val_if_fail(blob != NULL, NULL);
+	return fu_strsafe((const gchar *)g_bytes_get_data(blob, NULL),
+			  MIN(g_bytes_get_size(blob), maxsz));
+}
+
 /**
  * fu_strjoin:
  * @separator: (nullable): string to insert between each of the strings
@@ -680,8 +725,8 @@ fu_utf16_to_utf8_byte_array(GByteArray *array, FuEndianType endian, GError **err
 
 	if (array->len % 2 != 0) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "invalid UTF-16 buffer length");
 		return NULL;
 	}
