@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Dell Inc.
+ * Copyright 2018 Dell Inc.
  * All rights reserved.
  *
  * This software and associated documentation (if any) is furnished
@@ -10,13 +10,17 @@
  * redistributing this file, you may do so under either license.
  * Dell Chooses the MIT license part of Dual MIT/LGPLv2 license agreement.
  *
- * SPDX-License-Identifier: LGPL-2.1+ OR MIT
+ * SPDX-License-Identifier: LGPL-2.1-or-later OR MIT
  */
 
 #include "config.h"
 
 #include "fu-dell-dock-common.h"
+#include "fu-dell-dock-ec.h"
+#include "fu-dell-dock-mst.h"
 #include "fu-dell-dock-plugin.h"
+#include "fu-dell-dock-status.h"
+#include "fu-dell-dock-tbt.h"
 
 struct _FuDellDockPlugin {
 	FuPlugin parent_instance;
@@ -41,10 +45,6 @@ fu_dell_dock_plugin_create_node(FuPlugin *plugin, FuDevice *device, GError **err
 static gboolean
 fu_dell_dock_plugin_probe(FuPlugin *plugin, FuDevice *proxy, GError **error)
 {
-	const gchar *instance_id_mst;
-	const gchar *instance_id_status;
-	g_autofree const gchar *instance_guid_mst = NULL;
-	g_autofree const gchar *instance_guid_status = NULL;
 	g_autoptr(FuDellDockEc) ec_device = NULL;
 	g_autoptr(FuDellDockMst) mst_device = NULL;
 	g_autoptr(FuDellDockStatus) status_device = NULL;
@@ -61,13 +61,10 @@ fu_dell_dock_plugin_probe(FuPlugin *plugin, FuDevice *proxy, GError **error)
 
 	/* create mst endpoint */
 	mst_device = fu_dell_dock_mst_new(ctx);
-	if (fu_dell_dock_get_dock_type(FU_DEVICE(ec_device)) == DOCK_BASE_TYPE_ATOMIC)
-		instance_id_mst = DELL_DOCK_VMM6210_INSTANCE_ID;
+	if (fu_dell_dock_ec_get_dock_type(FU_DEVICE(ec_device)) == DOCK_BASE_TYPE_ATOMIC)
+		fu_device_add_instance_id(FU_DEVICE(mst_device), DELL_DOCK_VMM6210_INSTANCE_ID);
 	else
-		instance_id_mst = DELL_DOCK_VM5331_INSTANCE_ID;
-	fu_device_add_instance_id(FU_DEVICE(mst_device), instance_id_mst);
-	instance_guid_mst = fwupd_guid_hash_string(instance_id_mst);
-	fu_device_add_guid(FU_DEVICE(mst_device), instance_guid_mst);
+		fu_device_add_instance_id(FU_DEVICE(mst_device), DELL_DOCK_VM5331_INSTANCE_ID);
 	if (!fu_device_probe(FU_DEVICE(mst_device), error))
 		return FALSE;
 	fu_device_add_child(FU_DEVICE(ec_device), FU_DEVICE(mst_device));
@@ -76,25 +73,22 @@ fu_dell_dock_plugin_probe(FuPlugin *plugin, FuDevice *proxy, GError **error)
 
 	/* create package version endpoint */
 	status_device = fu_dell_dock_status_new(ctx);
-	if (fu_dell_dock_get_dock_type(FU_DEVICE(ec_device)) == DOCK_BASE_TYPE_ATOMIC)
-		instance_id_status = DELL_DOCK_ATOMIC_STATUS_INSTANCE_ID;
-	else if (fu_dell_dock_module_is_usb4(FU_DEVICE(ec_device)))
-		instance_id_status = DELL_DOCK_DOCK2_INSTANCE_ID;
-	else
-		instance_id_status = DELL_DOCK_DOCK1_INSTANCE_ID;
-	instance_guid_status = fwupd_guid_hash_string(instance_id_status);
-	fu_device_add_guid(FU_DEVICE(status_device), fwupd_guid_hash_string(instance_guid_status));
+	if (fu_dell_dock_ec_get_dock_type(FU_DEVICE(ec_device)) == DOCK_BASE_TYPE_ATOMIC) {
+		fu_device_add_instance_id(FU_DEVICE(status_device),
+					  DELL_DOCK_ATOMIC_STATUS_INSTANCE_ID);
+	} else if (fu_dell_dock_ec_module_is_usb4(FU_DEVICE(ec_device))) {
+		fu_device_add_instance_id(FU_DEVICE(status_device), DELL_DOCK_DOCK2_INSTANCE_ID);
+	} else {
+		fu_device_add_instance_id(FU_DEVICE(status_device), DELL_DOCK_DOCK1_INSTANCE_ID);
+	}
 	fu_device_add_child(FU_DEVICE(ec_device), FU_DEVICE(status_device));
-	fu_device_add_instance_id(FU_DEVICE(status_device), instance_id_status);
 	if (!fu_dell_dock_plugin_create_node(plugin, FU_DEVICE(status_device), error))
 		return FALSE;
 
 	/* create TBT endpoint if Thunderbolt SKU and Thunderbolt link inactive */
 	if (fu_dell_dock_ec_needs_tbt(FU_DEVICE(ec_device))) {
 		g_autoptr(FuDellDockTbt) tbt_device = fu_dell_dock_tbt_new(proxy);
-		g_autofree const gchar *instance_guid_tbt =
-		    fwupd_guid_hash_string(DELL_DOCK_TBT_INSTANCE_ID);
-		fu_device_add_guid(FU_DEVICE(tbt_device), instance_guid_tbt);
+		fu_device_add_instance_id(FU_DEVICE(tbt_device), DELL_DOCK_TBT_INSTANCE_ID);
 		fu_device_add_child(FU_DEVICE(ec_device), FU_DEVICE(tbt_device));
 		if (!fu_dell_dock_plugin_create_node(plugin, FU_DEVICE(tbt_device), error))
 			return FALSE;
@@ -159,7 +153,7 @@ fu_dell_dock_plugin_backend_device_added(FuPlugin *plugin,
 	}
 
 	/* determine dock type by ec */
-	dock_type = fu_dell_dock_get_dock_type(ec_device);
+	dock_type = fu_dell_dock_ec_get_dock_type(ec_device);
 	if (dock_type == DOCK_BASE_TYPE_UNKNOWN) {
 		g_set_error(error,
 			    FWUPD_ERROR,
@@ -204,11 +198,11 @@ fu_dell_dock_plugin_device_registered(FuPlugin *plugin, FuDevice *device)
 {
 	/* dell dock delays the activation so skips device restart */
 	if (fu_device_has_guid(device, DELL_DOCK_TBT_INSTANCE_ID)) {
-		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
 		fu_plugin_cache_add(plugin, "tbt", device);
 	}
 	if (fu_device_has_guid(device, DELL_DOCK_USB4_INSTANCE_ID)) {
-		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
 		fu_plugin_cache_add(plugin, "usb4", device);
 	}
 	if (FU_IS_DELL_DOCK_EC(device))
@@ -231,24 +225,15 @@ fu_dell_dock_plugin_device_registered(FuPlugin *plugin, FuDevice *device)
 static gboolean
 fu_dell_dock_plugin_backend_device_removed(FuPlugin *plugin, FuDevice *device, GError **error)
 {
-	/* device will be activated on disconnected */
-	if (fu_device_has_flag(device, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)) {
-		fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION);
-		g_debug("dropped needs-activation flag: %s", fu_device_get_name(device));
-	}
+	FuDevice *parent;
 
-	if (FU_IS_DELL_DOCK_EC(device)) {
-		GPtrArray *children = fu_device_get_children(device);
+	if (!FU_IS_USB_DEVICE(device))
+		return TRUE;
 
-		for (guint i = 0; i < children->len; i++) {
-			FuDevice *dev = g_ptr_array_index(children, i);
-
-			if (fu_device_has_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)) {
-				fu_device_remove_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION);
-				g_debug("dropped needs-activation flag: %s",
-					fu_device_get_name(dev));
-			}
-		}
+	parent = fu_device_get_parent(device);
+	if (parent != NULL && FU_IS_DELL_DOCK_EC(parent)) {
+		g_debug("Removing %s (%s)", fu_device_get_name(parent), fu_device_get_id(parent));
+		fu_plugin_device_remove(plugin, parent);
 	}
 
 	return TRUE;
@@ -343,6 +328,10 @@ fu_dell_dock_plugin_constructed(GObject *obj)
 	/* allow these to be built by quirks */
 	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_STATUS);
 	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_MST);
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_EC);     /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_MST);    /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_STATUS); /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_TBT);    /* coverage */
 
 #ifndef _WIN32
 	/* currently slower performance, but more reliable in corner cases */
