@@ -73,18 +73,6 @@ fu_dell_kestrel_ec_is_dev_present(FuDellKestrelEc *self,
 	return dev_entry != NULL;
 }
 
-gboolean
-fu_dell_kestrel_ec_is_chunk_supported(FuDellKestrelEc *self, FuDellKestrelEcDevType dev_type)
-{
-	if (dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_PD) {
-		guint8 chunk_support = 0;
-
-		chunk_support = fu_struct_dell_kestrel_dock_data_get_chunk_support(self->dock_data);
-		return (chunk_support & FU_DELL_KESTREL_DOCK_DATA_CHUNK_SUPPORT_BITMAP_PD);
-	}
-	return TRUE;
-}
-
 const gchar *
 fu_dell_kestrel_ec_devicetype_to_str(FuDellKestrelEcDevType dev_type,
 				     FuDellKestrelEcDevSubtype subtype,
@@ -154,14 +142,14 @@ fu_dell_kestrel_ec_get_dock_sku(FuDellKestrelEc *self)
 
 static gboolean
 fu_dell_kestrel_ec_read(FuDellKestrelEc *self,
-			FuDellKestrelEcCmd cmd,
+			FuDellKestrelEcHidCmd cmd,
 			GByteArray *res,
 			GError **error)
 {
 	if (!fu_dell_kestrel_hid_device_i2c_read(FU_DELL_KESTREL_HID_DEVICE(self),
 						 cmd,
 						 res,
-						 100,
+						 800,
 						 error)) {
 		g_prefix_error(error, "read over HID-I2C failed: ");
 		return FALSE;
@@ -327,7 +315,7 @@ fu_dell_kestrel_ec_dock_type_extract(FuDellKestrelEc *self, GError **error)
 static gboolean
 fu_dell_kestrel_ec_dock_type_cmd(FuDellKestrelEc *self, GError **error)
 {
-	FuDellKestrelEcCmd cmd = FU_DELL_KESTREL_EC_CMD_GET_DOCK_TYPE;
+	FuDellKestrelEcHidCmd cmd = FU_DELL_KESTREL_EC_HID_CMD_GET_DOCK_TYPE;
 	gsize length = 1;
 	g_autoptr(GByteArray) res = g_byte_array_new_take(g_malloc0(length), length);
 
@@ -346,7 +334,7 @@ fu_dell_kestrel_ec_dock_type_cmd(FuDellKestrelEc *self, GError **error)
 static gboolean
 fu_dell_kestrel_ec_dock_info_cmd(FuDellKestrelEc *self, GError **error)
 {
-	FuDellKestrelEcCmd cmd = FU_DELL_KESTREL_EC_CMD_GET_DOCK_INFO;
+	FuDellKestrelEcHidCmd cmd = FU_DELL_KESTREL_EC_HID_CMD_GET_DOCK_INFO;
 	g_autoptr(GByteArray) res = fu_struct_dell_kestrel_dock_info_new();
 
 	/* get dock info over HID */
@@ -383,7 +371,7 @@ fu_dell_kestrel_ec_dock_data_extract(FuDellKestrelEc *self, GError **error)
 static gboolean
 fu_dell_kestrel_ec_dock_data_cmd(FuDellKestrelEc *self, GError **error)
 {
-	FuDellKestrelEcCmd cmd = FU_DELL_KESTREL_EC_CMD_GET_DOCK_DATA;
+	FuDellKestrelEcHidCmd cmd = FU_DELL_KESTREL_EC_HID_CMD_GET_DOCK_DATA;
 	g_autoptr(GByteArray) res = fu_struct_dell_kestrel_dock_data_new();
 
 	/* get dock data over HID */
@@ -416,7 +404,7 @@ fu_dell_kestrel_ec_is_dock_ready4update(FuDevice *device, GError **error)
 	if ((dock_status & bitmask_fw_update_pending) != 0) {
 		g_set_error(error,
 			    FWUPD_ERROR,
-			    FWUPD_ERROR_INTERNAL,
+			    FWUPD_ERROR_BUSY,
 			    "dock status (%x) has pending updates, unavailable for now.",
 			    dock_status);
 		return FALSE;
@@ -425,54 +413,31 @@ fu_dell_kestrel_ec_is_dock_ready4update(FuDevice *device, GError **error)
 	return TRUE;
 }
 
-static gboolean
-fu_dell_kestrel_ec_is_new_ownership_cmd(FuDellKestrelEc *self)
-{
-	FuDevice *device = FU_DEVICE(self);
-	const gchar *version = fu_device_get_version(device);
-	FwupdVersionFormat fmt = fu_device_get_version_format(device);
-
-	if (fu_version_compare(version, "01.00.00.00", fmt) >= 0) {
-		if (fu_version_compare(version, "01.00.05.02", fmt) >= 0)
-			return TRUE;
-
-		return FALSE;
-	}
-	return fu_version_compare(version, "00.00.34.00", fmt) >= 0;
-}
-
 gboolean
 fu_dell_kestrel_ec_own_dock(FuDellKestrelEc *self, gboolean lock, GError **error)
 {
-	guint16 bitmask = 0x0;
-	g_autoptr(GByteArray) st_req = fu_struct_dell_kestrel_ec_databytes_new();
+	g_autoptr(GByteArray) req = g_byte_array_new();
 	g_autoptr(GError) error_local = NULL;
 	g_autofree gchar *msg = NULL;
 
-	fu_struct_dell_kestrel_ec_databytes_set_cmd(st_req, FU_DELL_KESTREL_EC_CMD_SET_MODIFY_LOCK);
-	fu_struct_dell_kestrel_ec_databytes_set_data_sz(st_req, 2);
+	fu_byte_array_append_uint8(req, FU_DELL_KESTREL_EC_HID_CMD_SET_MODIFY_LOCK);
+	fu_byte_array_append_uint8(req, 2); // length of data
 
 	if (lock) {
 		msg = g_strdup("own the dock");
-		bitmask = fu_dell_kestrel_ec_is_new_ownership_cmd(self) ? 0x10CC : 0xFFFF;
+		fu_byte_array_append_uint16(req, 0xFFFF, G_LITTLE_ENDIAN);
 	} else {
 		msg = g_strdup("relesae the dock");
-		bitmask = fu_dell_kestrel_ec_is_new_ownership_cmd(self) ? 0xC001 : 0x0000;
+		fu_byte_array_append_uint16(req, 0x0000, G_LITTLE_ENDIAN);
 	}
-	if (!fu_struct_dell_kestrel_ec_databytes_set_data(st_req,
-							  (const guint8 *)&bitmask,
-							  sizeof(bitmask),
-							  error))
-		return FALSE;
 
 	fu_device_sleep(FU_DEVICE(self), 1000);
-	if (!fu_dell_kestrel_ec_write(self, st_req, &error_local)) {
-		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND) ||
-		    g_error_matches(error_local, G_USB_DEVICE_ERROR, G_USB_DEVICE_ERROR_NO_DEVICE)) {
+	if (!fu_dell_kestrel_ec_write(self, req, &error_local)) {
+		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND))
 			g_debug("ignoring: %s", error_local->message);
-		} else {
+		else {
 			g_propagate_error(error, g_steal_pointer(&error_local));
-			g_prefix_error(error, "failed to %s: ", msg);
+			g_prefix_error(error, "failed to %s", msg);
 			return FALSE;
 		}
 	}
@@ -485,21 +450,16 @@ gboolean
 fu_dell_kestrel_ec_run_passive_update(FuDellKestrelEc *self, GError **error)
 {
 	guint max_tries = 2;
-	g_autoptr(GByteArray) st_req = fu_struct_dell_kestrel_ec_databytes_new();
-	const guint8 bitmap = 0x07;
+	g_autoptr(GByteArray) req = g_byte_array_new();
 
 	/* ec included in cmd, set bit2 in data for tbt */
-	fu_struct_dell_kestrel_ec_databytes_set_cmd(st_req, FU_DELL_KESTREL_EC_CMD_SET_PASSIVE);
-	fu_struct_dell_kestrel_ec_databytes_set_data_sz(st_req, 1);
-	if (!fu_struct_dell_kestrel_ec_databytes_set_data(st_req,
-							  (const guint8 *)&bitmap,
-							  sizeof(bitmap),
-							  error))
-		return FALSE;
+	fu_byte_array_append_uint8(req, FU_DELL_KESTREL_EC_HID_CMD_SET_PASSIVE);
+	fu_byte_array_append_uint8(req, 1); // length of data
+	fu_byte_array_append_uint8(req, 0x02);
 
 	for (guint i = 1; i <= max_tries; i++) {
 		g_debug("register passive update (uod) flow (%u/%u)", i, max_tries);
-		if (!fu_dell_kestrel_ec_write(self, st_req, error)) {
+		if (!fu_dell_kestrel_ec_write(self, req, error)) {
 			g_prefix_error(error, "failed to register uod flow: ");
 			return FALSE;
 		}
@@ -623,34 +583,38 @@ fu_dell_kestrel_ec_get_package_version(FuDellKestrelEc *self)
 }
 
 gboolean
-fu_dell_kestrel_ec_commit_package(FuDellKestrelEc *self, GBytes *blob_fw, GError **error)
+fu_dell_kestrel_ec_commit_package(FuDellKestrelEc *self, GInputStream *stream, GError **error)
 {
-	gsize bufsz = 0;
-	const guint8 *buf = NULL;
-	g_autoptr(GByteArray) st_req = fu_struct_dell_kestrel_ec_databytes_new();
-
-	g_return_val_if_fail(blob_fw != NULL, FALSE);
-
-	buf = g_bytes_get_data(blob_fw, &bufsz);
+	g_autoptr(GByteArray) req = g_byte_array_new();
+	g_autoptr(GBytes) bytes = NULL;
+	gsize streamsz = 0;
 
 	/* verify package length */
-	if (bufsz != FU_STRUCT_DELL_KESTREL_PACKAGE_FW_VERSIONS_SIZE) {
+	if (!fu_input_stream_size(stream, &streamsz, error))
+		return FALSE;
+
+	if (streamsz != FU_STRUCT_DELL_KESTREL_PACKAGE_FW_VERSIONS_SIZE) {
 		g_set_error(error,
 			    FWUPD_ERROR,
-			    FWUPD_ERROR_INTERNAL,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "Invalid package size %" G_GSIZE_FORMAT,
-			    bufsz);
+			    streamsz);
 		return FALSE;
 	}
 
-	fu_struct_dell_kestrel_ec_databytes_set_cmd(st_req, FU_DELL_KESTREL_EC_CMD_SET_DOCK_PKG);
-	fu_struct_dell_kestrel_ec_databytes_set_data_sz(st_req, bufsz);
-	if (!fu_struct_dell_kestrel_ec_databytes_set_data(st_req, buf, bufsz, error))
-		return FALSE;
+	/* get the data bytes */
+	bytes = fu_input_stream_read_bytes(stream,
+					   0,
+					   FU_STRUCT_DELL_KESTREL_PACKAGE_FW_VERSIONS_SIZE,
+					   NULL,
+					   error);
 
-	fu_dump_raw(G_LOG_DOMAIN, "->PACKAGE", st_req->data, st_req->len);
+	fu_byte_array_append_uint8(req, FU_DELL_KESTREL_EC_HID_CMD_SET_DOCK_PKG);
+	fu_byte_array_append_uint8(req, streamsz); // length of data
+	fu_byte_array_append_bytes(req, bytes);
+	fu_dump_raw(G_LOG_DOMAIN, "->PACKAGE", req->data, req->len);
 
-	if (!fu_dell_kestrel_ec_write(self, st_req, error)) {
+	if (!fu_dell_kestrel_ec_write(self, req, error)) {
 		g_prefix_error(error, "Failed to commit package: ");
 		return FALSE;
 	}
@@ -701,8 +665,8 @@ fu_dell_kestrel_ec_reload(FuDevice *device, GError **error)
 	/* if query looks bad, wait a few seconds and retry */
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_dell_kestrel_ec_query_cb,
-				  DELL_KESTREL_MAX_RETRIES,
-				  500,
+				  10,
+				  2000,
 				  NULL,
 				  error)) {
 		g_prefix_error(error, "failed to query dock ec: ");
@@ -726,12 +690,7 @@ fu_dell_kestrel_ec_setup(FuDevice *device, GError **error)
 		return FALSE;
 
 	/* if query looks bad, wait a few seconds and retry */
-	if (!fu_device_retry_full(device,
-				  fu_dell_kestrel_ec_query_cb,
-				  DELL_KESTREL_MAX_RETRIES,
-				  500,
-				  NULL,
-				  error)) {
+	if (!fu_device_retry_full(device, fu_dell_kestrel_ec_query_cb, 10, 2000, NULL, error)) {
 		g_prefix_error(error, "failed to query dock ec: ");
 		return FALSE;
 	}
@@ -794,10 +753,10 @@ fu_dell_kestrel_ec_init(FuDellKestrelEc *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_DUAL_IMAGE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SELF_RECOVERY);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_INSTALL_SKIP_VERSION_CHECK);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SKIPS_RESTART);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_RETRY_OPEN);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_FLAGS);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_RETRY_OPEN);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_FLAGS);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_QUAD);
 }
 
@@ -822,7 +781,7 @@ fu_dell_kestrel_ec_new(FuDevice *device, gboolean uod)
 	FuContext *ctx = fu_device_get_context(device);
 
 	self = g_object_new(FU_TYPE_DELL_KESTREL_EC, "context", ctx, NULL);
-	fu_device_incorporate(FU_DEVICE(self), device);
+	fu_device_incorporate(FU_DEVICE(self), device, FU_DEVICE_INCORPORATE_FLAG_ALL);
 	fu_device_set_logical_id(FU_DEVICE(self), "ec");
 	if (uod)
 		fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
