@@ -7,37 +7,36 @@
 
 #include "config.h"
 
-#include "fu-dell-kestrel-ec.h"
 #include "fu-dell-kestrel-hid-device.h"
 #include "fu-dell-kestrel-hid-struct.h"
 
 G_DEFINE_TYPE(FuDellKestrelHidDevice, fu_dell_kestrel_hid_device, FU_TYPE_HID_DEVICE)
 
 /* Used for EC HID communication */
-#define FU_DELL_KESTREL_HID_TIMEOUT	     3000
+#define FU_DELL_KESTREL_HID_TIMEOUT	     2000
 #define FU_DELL_KESTREL_HID_CMD_FWUPDATE     0xAB
 #define FU_DELL_KESTREL_HID_EXT_FWUPDATE     0x80
 #define FU_DELL_KESTREL_HID_SUBCMD_FWUPDATE  0x00
-#define FU_DELL_KESTREL_HID_DEV_ANY_CHUNK_SZ 120000
+#define FU_DELL_KESTREL_HID_DEV_EC_CHUNK_SZ  160000
+#define FU_DELL_KESTREL_HID_DEV_PD_CHUNK_SZ  190000
+#define FU_DELL_KESTREL_HID_DEV_ANY_CHUNK_SZ 180000
 #define FU_DELL_KESTREL_HID_DEV_NO_CHUNK_SZ  G_MAXSIZE
 #define FU_DELL_KESTREL_HID_DATA_PAGE_SZ     192
 #define FU_DELL_KESTREL_HID_RESPONSE_LENGTH  0x03
 #define FU_DELL_KESTREL_HID_I2C_ADDRESS	     0xec
+#define FU_DELL_KESTREL_HID_MAX_RETRIES	     8
 
 #define FU_DELL_KESTREL_HID_I2C_MAX_READ  192
 #define FU_DELL_KESTREL_HID_I2C_MAX_WRITE 128
 
 static gboolean
-fu_dell_kestrel_hid_device_write(FuDellKestrelHidDevice *self,
-				 GByteArray *buf,
-				 guint delay_ms,
-				 GError **error)
+fu_dell_kestrel_hid_device_write(FuDellKestrelHidDevice *self, GByteArray *buf, GError **error)
 {
 	return fu_hid_device_set_report(FU_HID_DEVICE(self),
 					0x0,
 					buf->data,
 					buf->len,
-					delay_ms,
+					FU_DELL_KESTREL_HID_TIMEOUT,
 					FU_HID_DEVICE_FLAG_RETRY_FAILURE,
 					error);
 }
@@ -75,25 +74,25 @@ fu_dell_kestrel_hid_device_fwup_pkg_new(FuChunk *chk,
 static gboolean
 fu_dell_kestrel_hid_device_hid_set_report_cb(FuDevice *self, gpointer user_data, GError **error)
 {
-	GByteArray *buf = (GByteArray *)user_data;
+	guint8 *outbuffer = (guint8 *)user_data;
 	return fu_hid_device_set_report(FU_HID_DEVICE(self),
 					0x0,
-					buf->data,
-					buf->len,
-					FU_DELL_KESTREL_HID_TIMEOUT,
+					outbuffer,
+					192,
+					FU_DELL_KESTREL_HID_TIMEOUT * 3,
 					FU_HID_DEVICE_FLAG_NONE,
 					error);
 }
 
 static gboolean
 fu_dell_kestrel_hid_device_hid_set_report(FuDellKestrelHidDevice *self,
-					  GByteArray *buf,
+					  guint8 *outbuffer,
 					  GError **error)
 {
 	return fu_device_retry(FU_DEVICE(self),
 			       fu_dell_kestrel_hid_device_hid_set_report_cb,
-			       DELL_KESTREL_MAX_RETRIES,
-			       buf,
+			       FU_DELL_KESTREL_HID_MAX_RETRIES,
+			       outbuffer,
 			       error);
 }
 
@@ -117,7 +116,7 @@ fu_dell_kestrel_hid_device_get_report(FuDellKestrelHidDevice *self,
 {
 	return fu_device_retry_full(FU_DEVICE(self),
 				    fu_dell_kestrel_hid_device_get_report_cb,
-				    DELL_KESTREL_MAX_RETRIES,
+				    FU_DELL_KESTREL_HID_MAX_RETRIES,
 				    2000,
 				    inbuffer,
 				    error);
@@ -136,18 +135,18 @@ fu_dell_kestrel_hid_device_i2c_write(FuDellKestrelHidDevice *self,
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_cmd(buf, FU_DELL_KESTREL_HID_CMD_WRITE_DATA);
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_ext(buf, FU_DELL_KESTREL_HID_CMD_EXT_I2C_WRITE);
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_dwregaddr(buf, 0x00);
-	fu_struct_dell_kestrel_hid_cmd_buffer_set_bufferlen(buf, GUINT16_TO_LE(cmd_buf->len));
+	fu_struct_dell_kestrel_hid_cmd_buffer_set_bufferlen(buf, cmd_buf->len);
 	if (!fu_struct_dell_kestrel_hid_cmd_buffer_set_databytes(buf,
 								 cmd_buf->data,
 								 cmd_buf->len,
 								 error))
 		return FALSE;
-	return fu_dell_kestrel_hid_device_hid_set_report(self, buf, error);
+	return fu_dell_kestrel_hid_device_hid_set_report(self, buf->data, error);
 }
 
 gboolean
 fu_dell_kestrel_hid_device_i2c_read(FuDellKestrelHidDevice *self,
-				    FuDellKestrelEcCmd cmd,
+				    FuDellKestrelEcHidCmd cmd,
 				    GByteArray *res,
 				    guint delayms,
 				    GError **error)
@@ -160,7 +159,7 @@ fu_dell_kestrel_hid_device_i2c_read(FuDellKestrelHidDevice *self,
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_ext(buf, FU_DELL_KESTREL_HID_CMD_EXT_I2C_READ);
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_dwregaddr(buf, cmd);
 	fu_struct_dell_kestrel_hid_cmd_buffer_set_bufferlen(buf, res->len + 1);
-	if (!fu_dell_kestrel_hid_device_hid_set_report(self, buf, error))
+	if (!fu_dell_kestrel_hid_device_hid_set_report(self, buf->data, error))
 		return FALSE;
 
 	if (delayms > 0)
@@ -190,17 +189,25 @@ fu_dell_kestrel_hid_device_get_chunk_delaytime(FuDellKestrelEcDevType dev_type)
 }
 
 static gsize
-fu_dell_kestrel_hid_device_get_chunk_size(FuDellKestrelHidDevice *self,
-					  FuDellKestrelEcDevType dev_type)
+fu_dell_kestrel_hid_device_get_chunk_size(FuDellKestrelEcDevType dev_type)
 {
-	if (dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_RMM)
+	/* return the max chunk size in bytes */
+	switch (dev_type) {
+	case FU_DELL_KESTREL_EC_DEV_TYPE_MAIN_EC:
+		return FU_DELL_KESTREL_HID_DEV_EC_CHUNK_SZ;
+	case FU_DELL_KESTREL_EC_DEV_TYPE_PD:
+		return FU_DELL_KESTREL_HID_DEV_PD_CHUNK_SZ;
+	case FU_DELL_KESTREL_EC_DEV_TYPE_RMM:
 		return FU_DELL_KESTREL_HID_DEV_NO_CHUNK_SZ;
-
-	if (FU_IS_DELL_KESTREL_EC(self) &&
-	    fu_dell_kestrel_ec_is_chunk_supported(FU_DELL_KESTREL_EC(self), dev_type))
+	default:
 		return FU_DELL_KESTREL_HID_DEV_ANY_CHUNK_SZ;
+	}
+}
 
-	return FU_DELL_KESTREL_HID_DEV_NO_CHUNK_SZ;
+static guint
+fu_dell_kestrel_hid_device_get_first_page_delaytime(FuDellKestrelEcDevType dev_type)
+{
+	return (dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_RMM) ? 75 * 1000 : 0;
 }
 
 static gboolean
@@ -209,9 +216,10 @@ fu_dell_kestrel_hid_device_write_firmware_pages(FuDellKestrelHidDevice *self,
 						FuProgress *progress,
 						FuDellKestrelEcDevType dev_type,
 						guint chunk_idx,
-						guint chunks_num,
 						GError **error)
 {
+	guint first_page_delay = fu_dell_kestrel_hid_device_get_first_page_delaytime(dev_type);
+
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_set_steps(progress, fu_chunk_array_length(pages));
@@ -219,17 +227,14 @@ fu_dell_kestrel_hid_device_write_firmware_pages(FuDellKestrelHidDevice *self,
 	for (guint j = 0; j < fu_chunk_array_length(pages); j++) {
 		g_autoptr(GByteArray) page_aligned = g_byte_array_new();
 		g_autoptr(FuChunk) page = NULL;
-		g_autoptr(GBytes) page_bytes = NULL;
 		g_autoptr(GError) error_local = NULL;
-		guint page_ack_time = FU_DELL_KESTREL_HID_TIMEOUT;
 
-		page = fu_chunk_array_index(pages, j);
+		page = fu_chunk_array_index(pages, j, error);
 		if (page == NULL)
 			return FALSE;
 
-		g_debug("sending chunk: %u/%u, page: %u/%u.",
+		g_debug("sending chunk: %u, page: %u/%u.",
 			chunk_idx,
-			chunks_num - 1,
 			j,
 			fu_chunk_array_length(pages) - 1);
 
@@ -239,56 +244,28 @@ fu_dell_kestrel_hid_device_write_firmware_pages(FuDellKestrelHidDevice *self,
 				    fu_chunk_get_data_sz(page));
 		fu_byte_array_set_size(page_aligned, FU_DELL_KESTREL_HID_DATA_PAGE_SZ, 0xFF);
 
-		/* rmm needs extra time to ack the first page */
-		if (j == 0 && dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_RMM)
-			page_ack_time = 75 * 1000;
-
 		/* send to ec */
-		if (!fu_dell_kestrel_hid_device_write(self,
-						      page_aligned,
-						      page_ack_time,
-						      &error_local)) {
+		if (!fu_dell_kestrel_hid_device_write(self, page_aligned, &error_local)) {
 			/* A buggy device may fail to send an acknowledgment receipt
-			   after the last page write, resulting in a timeout error or a pipe
-			   error on reading. This is a known issue so waive it for now.
+			   after the last page write, resulting in a timeout error.
+
+			   This is a known issue so waive it for now.
 			*/
 			if (dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_LAN &&
-			    j == fu_chunk_array_length(pages) - 1) {
-				if (g_error_matches(error_local,
-						    G_USB_DEVICE_ERROR,
-						    G_USB_DEVICE_ERROR_NOT_SUPPORTED) ||
-#if G_USB_CHECK_VERSION(0, 4, 8)
-				    g_error_matches(error_local,
-						    G_USB_DEVICE_ERROR,
-						    G_USB_DEVICE_ERROR_BUSY) ||
-#endif
-				    g_error_matches(error_local,
-						    G_USB_DEVICE_ERROR,
-						    G_USB_DEVICE_ERROR_TIMED_OUT)) {
-					g_debug("ignored error: %s", error_local->message);
-					fu_progress_step_done(progress);
-					continue;
-				}
+			    j == fu_chunk_array_length(pages) - 1 &&
+			    g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT)) {
+				g_debug("ignored error: %s", error_local->message);
+				fu_progress_step_done(progress);
+				continue;
 			}
-
-			/* propagate error as usual */
-			g_propagate_prefixed_error(error,
-						   g_steal_pointer(&error_local),
-						   "%s failed to write page: ",
-						   fu_device_get_name(FU_DEVICE(self)));
+			g_propagate_error(error, g_steal_pointer(&error_local));
 			return FALSE;
 		}
 
-		/* rmm needs extra time to accept incoming pages */
-		if (j == 0 && dev_type == FU_DELL_KESTREL_EC_DEV_TYPE_RMM) {
-			FuDevice *dev = FU_DEVICE(self);
-			if (fu_version_compare(fu_device_get_version(dev),
-					       "1.8.6.0",
-					       fu_device_get_version_format(dev)) < 0) {
-				guint delay_ms = 75 * 1000;
-				g_debug("wait %u ms before the next page", delay_ms);
-				fu_device_sleep(FU_DEVICE(self), delay_ms);
-			}
+		/* device needs time to process incoming pages */
+		if (j == 0) {
+			g_debug("wait %u ms before the next page", first_page_delay);
+			fu_device_sleep(FU_DEVICE(self), first_page_delay);
 		}
 		fu_progress_step_done(progress);
 	}
@@ -323,7 +300,7 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 					  GError **error)
 {
 	gsize fw_sz = 0;
-	gsize chunk_sz = fu_dell_kestrel_hid_device_get_chunk_size(self, dev_type);
+	gsize chunk_sz = fu_dell_kestrel_hid_device_get_chunk_size(dev_type);
 	guint chunk_delay = fu_dell_kestrel_hid_device_get_chunk_delaytime(dev_type);
 	g_autoptr(GBytes) fw = NULL;
 	g_autoptr(FuChunkArray) chunks = NULL;
@@ -344,7 +321,10 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 	}
 
 	/* maximum buffer size */
-	chunks = fu_chunk_array_new_from_bytes(fw, 0, chunk_sz);
+	chunks = fu_chunk_array_new_from_bytes(fw,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       chunk_sz);
 
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
@@ -358,7 +338,7 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 		g_autoptr(FuChunkArray) pages = NULL;
 		g_autoptr(GBytes) buf = NULL;
 
-		chk = fu_chunk_array_index(chunks, i);
+		chk = fu_chunk_array_index(chunks, i, error);
 		if (chk == NULL)
 			return FALSE;
 
@@ -366,7 +346,10 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 		buf = fu_dell_kestrel_hid_device_fwup_pkg_new(chk, fw_sz, dev_type, dev_identifier);
 
 		/* slice the chunk into pages */
-		pages = fu_chunk_array_new_from_bytes(buf, 0, FU_DELL_KESTREL_HID_DATA_PAGE_SZ);
+		pages = fu_chunk_array_new_from_bytes(buf,
+						      FU_CHUNK_ADDR_OFFSET_NONE,
+						      FU_CHUNK_PAGESZ_NONE,
+						      FU_DELL_KESTREL_HID_DATA_PAGE_SZ);
 
 		/* write pages */
 		if (!fu_dell_kestrel_hid_device_write_firmware_pages(
@@ -375,7 +358,6 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 			fu_progress_get_child(progress),
 			dev_type,
 			i,
-			fu_chunk_array_length(chunks),
 			error))
 			return FALSE;
 
@@ -404,8 +386,7 @@ fu_dell_kestrel_hid_device_write_firmware(FuDellKestrelHidDevice *self,
 			g_set_error(error,
 				    FWUPD_ERROR,
 				    FWUPD_ERROR_WRITE,
-				    "%s failed to write chunk[%u]: %s",
-				    fu_device_get_name(FU_DEVICE(self)),
+				    "failed to write chunk[%u]: %s",
 				    i,
 				    fu_dell_kestrel_hid_ec_chunk_response_to_string(resp));
 			return FALSE;
