@@ -39,8 +39,8 @@ fu_dell_kestrel_plugin_device_add(FuPlugin *plugin, FuDevice *device, GError **e
 {
 	FuDellDockBaseType dock_type;
 	FuDevice *ec_device = fu_plugin_cache_lookup(plugin, "ec");
-	guint16 vid = fu_usb_device_get_vid(FU_USB_DEVICE(device));
-	guint16 pid = fu_usb_device_get_pid(FU_USB_DEVICE(device));
+	guint16 vid = fu_device_get_vid(device);
+	guint16 pid = fu_device_get_pid(device);
 
 	/* cache this device until dock type is seen */
 	if (ec_device == NULL) {
@@ -88,11 +88,7 @@ fu_dell_kestrel_plugin_device_add(FuPlugin *plugin, FuDevice *device, GError **e
 			return FALSE;
 
 		fu_device_add_child(ec_device, FU_DEVICE(rmm_device));
-
-		if (!fu_dell_kestrel_rmm_fix_version(rmm_device, error)) {
-			g_prefix_error(error, "failed to fix rmm version: ");
-			return FALSE;
-		}
+		fu_dell_kestrel_rmm_fix_version(rmm_device);
 
 		return TRUE;
 	}
@@ -100,14 +96,10 @@ fu_dell_kestrel_plugin_device_add(FuPlugin *plugin, FuDevice *device, GError **e
 	/* RTS usb hub devices */
 	if (pid == DELL_KESTREL_USB_RTS0_G1_PID || pid == DELL_KESTREL_USB_RTS0_G2_PID ||
 	    pid == DELL_KESTREL_USB_RTS5_G2_PID) {
-		g_autoptr(FuDellKestrelRtshub) hub_device = NULL;
+		g_autoptr(FuDellKestrelRtsHub) hub_device = NULL;
 		g_autoptr(FuDeviceLocker) locker = NULL;
-		gboolean uod = FALSE;
 
-		uod = fu_plugin_get_config_value_boolean(plugin,
-							 FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD);
-
-		hub_device = fu_dell_kestrel_rtshub_new(FU_USB_DEVICE(device), dock_type, uod);
+		hub_device = fu_dell_kestrel_rtshub_new(FU_USB_DEVICE(device), dock_type);
 		if (hub_device == NULL) {
 			g_set_error(error,
 				    FWUPD_ERROR,
@@ -145,7 +137,6 @@ fu_dell_kestrel_plugin_ec_add_cached_devices(FuPlugin *plugin, FuDevice *ec_devi
 	    {DELL_VID, DELL_KESTREL_USB_RTS0_G1_PID},
 	    {DELL_VID, DELL_KESTREL_USB_RTS0_G2_PID},
 	    {DELL_VID, DELL_KESTREL_USB_RTS5_G2_PID},
-	    {DELL_VID, DELL_KESTREL_USB_RMM_PID},
 	    {0},
 	};
 
@@ -179,8 +170,8 @@ fu_dell_kestrel_plugin_backend_device_added(FuPlugin *plugin,
 		return TRUE;
 
 	/* VID and PID */
-	vid = fu_usb_device_get_vid(FU_USB_DEVICE(device));
-	pid = fu_usb_device_get_pid(FU_USB_DEVICE(device));
+	vid = fu_device_get_vid(device);
+	pid = fu_device_get_pid(device);
 
 	/* USB HUB HID bridge device */
 	if ((vid == DELL_VID && pid == DELL_KESTREL_HID_PID)) {
@@ -236,7 +227,7 @@ fu_dell_kestrel_plugin_config_mst_dev(FuPlugin *plugin)
 		return;
 
 	/* run only once */
-	if (fu_device_has_internal_flag(device_mst, FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER))
+	if (fu_device_has_private_flag(device_mst, FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER))
 		return;
 
 	/* vmm8 */
@@ -263,8 +254,8 @@ fu_dell_kestrel_plugin_config_mst_dev(FuPlugin *plugin)
 	fu_device_set_name(device_mst, devname);
 
 	/* flags */
-	fu_device_add_internal_flag(device_mst, FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER);
-	fu_device_add_flag(device_mst, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
+	fu_device_add_private_flag(device_mst, FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER);
+	fu_device_add_private_flag(device_mst, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
 	return;
 }
 
@@ -289,6 +280,10 @@ fu_dell_kestrel_plugin_config_parentship(FuPlugin *plugin)
 static void
 fu_dell_kestrel_plugin_device_registered(FuPlugin *plugin, FuDevice *device)
 {
+	/* usb device of interset */
+	if (!FU_IS_USB_DEVICE(device))
+		return;
+
 	/* leverage intel_usb4 for usb4 devices */
 	if (fu_device_has_guid(device, DELL_KESTREL_T4_DEVID) ||
 	    fu_device_has_guid(device, DELL_KESTREL_T5_DEVID)) {
@@ -300,20 +295,13 @@ fu_dell_kestrel_plugin_device_registered(FuPlugin *plugin, FuDevice *device)
 			fu_device_inhibit(device, "hidden", msg);
 			return;
 		}
-		/* activation should already done when device is added */
-		fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION);
-		fu_device_add_internal_flag(device, FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER);
-		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER);
 		fu_plugin_cache_add(plugin, "usb4", device);
 	}
 
-	/* usb device of interset */
-	if (!FU_IS_USB_DEVICE(device))
-		return;
-
 	/* leverage synaptics_vmm9 plugin for the mst device */
-	if (fu_usb_device_get_vid(FU_USB_DEVICE(device)) == MST_VMM89_USB_VID &&
-	    fu_usb_device_get_pid(FU_USB_DEVICE(device)) == MST_VMM89_USB_PID)
+	if (fu_device_get_vid(device) == MST_VMM89_USB_VID &&
+	    fu_device_get_pid(device) == MST_VMM89_USB_PID)
 		fu_plugin_cache_add(plugin, "mst", device);
 
 	/* add ec to cache */
@@ -359,23 +347,6 @@ fu_dell_kestrel_plugin_composite_cleanup(FuPlugin *plugin, GPtrArray *devices, G
 	if (locker == NULL)
 		return FALSE;
 
-	/* update on connected, i.e., uod is false */
-	if (!fu_plugin_get_config_value_boolean(plugin, FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD)) {
-		for (guint i = 0; i < devices->len; i++) {
-			FuDevice *dev = g_ptr_array_index(devices, i);
-
-			/* release the dock will activate devices immediately */
-			if (fu_device_has_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)) {
-				fu_device_remove_flag(dev, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION);
-				g_debug("uoc dropped needs-activation flag for %s",
-					fu_device_get_name(dev));
-			}
-
-			/* don't expect user manually replug */
-			fu_device_set_remove_delay(dev, FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE);
-		}
-	}
-
 	/* release the dock */
 	if (!fu_dell_kestrel_ec_own_dock(FU_DELL_KESTREL_EC(ec_dev), FALSE, error))
 		return FALSE;
@@ -419,40 +390,60 @@ fu_dell_kestrel_plugin_composite_prepare(FuPlugin *plugin, GPtrArray *devices, G
 	return TRUE;
 }
 
+static gboolean
+fu_dell_kestrel_plugin_modify_config(FuPlugin *plugin,
+				     const gchar *key,
+				     const gchar *value,
+				     GError **error)
+{
+	const gchar *keys[] = {FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD, NULL};
+	if (!g_strv_contains(keys, key)) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "config key %s not supported",
+			    key);
+		return FALSE;
+	}
+	return fu_plugin_set_config_value(plugin, key, value, error);
+}
+
 static gboolean
 fu_dell_kestrel_plugin_backend_device_removed(FuPlugin *plugin, FuDevice *device, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
 	const gchar *cache_keys[] = {"ec", "mst", "usb4"};
+	FuDevice *parent = fu_device_get_parent(device);
 
 	if (parent == NULL)
 		return TRUE;
-
-	/* uod: device is managed to activate when disconnected */
-	if (fu_plugin_get_config_value_boolean(plugin, FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD) &&
-	    fu_device_has_flag(device, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)) {
-		fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION);
-		g_debug("uod dropped needs-activation flag for %s", fu_device_get_name(device));
-	}
-
 	if (!FU_IS_DELL_KESTREL_EC(parent))
 		return TRUE;
 
 	if (FU_IS_USB_DEVICE(device)) {
-		guint16 vid = fu_usb_device_get_vid(FU_USB_DEVICE(device));
-		guint16 pid = fu_usb_device_get_pid(FU_USB_DEVICE(device));
-		g_autofree gchar *key = g_strdup_printf("USB\\VID_%04X&PID_%04X", vid, pid);
+		g_autofree gchar *key = g_strdup_printf("USB\\VID_%04X&PID_%04X",
+							fu_device_get_vid(device),
+							fu_device_get_pid(device));
+		fu_plugin_cache_remove(plugin, key);
+	}
+	for (gsize i = 0; i < G_N_ELEMENTS(cache_keys); i++)
+		fu_plugin_cache_remove(plugin, cache_keys[i]);
 
-		if (fu_plugin_cache_lookup(plugin, key))
-			fu_plugin_cache_remove(plugin, key);
+	return TRUE;
+}
+
+static gboolean
+fu_dell_kestrel_plugin_prepare(FuPlugin *plugin,
+			       FuDevice *device,
+			       FuProgress *progress,
+			       FwupdInstallFlags flags,
+			       GError **error)
+{
+	/* usb4 device reboot is suppressed, let ec handle it in passive update */
+	if (fu_device_has_guid(device, DELL_KESTREL_T4_DEVID) ||
+	    fu_device_has_guid(device, DELL_KESTREL_T5_DEVID)) {
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
 	}
 
-	for (gsize i = 0; i < G_N_ELEMENTS(cache_keys); i++) {
-		if (fu_plugin_cache_lookup(plugin, cache_keys[i]))
-			fu_plugin_cache_remove(plugin, cache_keys[i]);
-	}
-
-	fu_plugin_device_remove(plugin, parent);
 	return TRUE;
 }
 
@@ -480,7 +471,7 @@ fu_dell_kestrel_plugin_constructed(GObject *obj)
 	fu_plugin_add_firmware_gtype(plugin, NULL, FU_TYPE_DELL_KESTREL_RTSHUB_FIRMWARE);
 
 	/* defaults changed here will also be reflected in the fwupd.conf man page */
-	fu_plugin_set_config_default(plugin, FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD, "false");
+	fu_plugin_set_config_default(plugin, FWUPD_DELL_KESTREL_PLUGIN_CONFIG_UOD, "true");
 }
 
 static void
@@ -493,4 +484,6 @@ fu_dell_kestrel_plugin_class_init(FuDellKestrelPluginClass *klass)
 	plugin_class->backend_device_removed = fu_dell_kestrel_plugin_backend_device_removed;
 	plugin_class->composite_prepare = fu_dell_kestrel_plugin_composite_prepare;
 	plugin_class->composite_cleanup = fu_dell_kestrel_plugin_composite_cleanup;
+	plugin_class->modify_config = fu_dell_kestrel_plugin_modify_config;
+	plugin_class->prepare = fu_dell_kestrel_plugin_prepare;
 }
