@@ -20,29 +20,18 @@ fu_dell_kestrel_rmm_convert_version(FuDevice *device, guint64 version_raw)
 	return fu_version_from_uint32_hex(version_raw, fu_device_get_version_format(device));
 }
 
-gboolean
-fu_dell_kestrel_rmm_fix_version(FuDellKestrelRmm *self, GError **error)
+void
+fu_dell_kestrel_rmm_fix_version(FuDellKestrelRmm *self)
 {
-	FuDevice *parent = fu_device_get_parent(FU_DEVICE(self));
+	FuDevice *parent = NULL;
 
-	/* RMM version is given by the parent EC device */
-	if (parent != NULL && FU_IS_DELL_KESTREL_EC(parent)) {
+	/* use version given by parent */
+	parent = fu_device_get_parent(FU_DEVICE(self));
+	if (parent != NULL) {
 		guint32 rmm_version;
-		g_autoptr(FuDeviceLocker) locker = NULL;
-
-		locker = fu_device_locker_new(parent, error);
-		if (locker == NULL)
-			return FALSE;
-
-		/* RMM might be added after EC, reload for the latest dock info */
-		if (!fu_device_reload(parent, error))
-			return FALSE;
-
 		rmm_version = fu_dell_kestrel_ec_get_rmm_version(FU_DELL_KESTREL_EC(parent));
 		fu_device_set_version_raw(FU_DEVICE(self), rmm_version);
 	}
-
-	return TRUE;
 }
 
 static gboolean
@@ -54,11 +43,7 @@ fu_dell_kestrel_rmm_setup(FuDevice *device, GError **error)
 	if (!FU_DEVICE_CLASS(fu_dell_kestrel_rmm_parent_class)->setup(device, error))
 		return FALSE;
 
-	if (!fu_dell_kestrel_rmm_fix_version(self, error)) {
-		g_prefix_error(error, "failed to fix RMM version: ");
-		return FALSE;
-	}
-
+	fu_dell_kestrel_rmm_fix_version(self);
 	return TRUE;
 }
 
@@ -95,8 +80,7 @@ fu_dell_kestrel_rmm_init(FuDellKestrelRmm *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_INSTALL_SKIP_VERSION_CHECK);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SKIPS_RESTART);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_EXPLICIT_ORDER);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_EXPLICIT_ORDER);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_QUAD);
 }
 
@@ -115,6 +99,6 @@ fu_dell_kestrel_rmm_new(FuUsbDevice *device)
 {
 	FuDellKestrelRmm *self = g_object_new(FU_TYPE_DELL_KESTREL_RMM, NULL);
 
-	fu_device_incorporate(FU_DEVICE(self), FU_DEVICE(device));
+	fu_device_incorporate(FU_DEVICE(self), FU_DEVICE(device), FU_DEVICE_INCORPORATE_FLAG_ALL);
 	return self;
 }
