@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -23,7 +23,6 @@ typedef struct {
 	guint8 device_idx;
 	guint16 hidpp_pid;
 	guint8 hidpp_version;
-	FuIOChannel *io_channel;
 	gchar *model_id;
 	GPtrArray *feature_index; /* of FuLogitechHidppHidppMap */
 } FuLogitechHidppDevicePrivate;
@@ -133,7 +132,6 @@ static gboolean
 fu_logitech_hidpp_device_ping(FuLogitechHidppDevice *self, GError **error)
 {
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
-	gdouble version;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(FuLogitechHidppHidppMsg) msg = fu_logitech_hidpp_msg_new();
 	GPtrArray *children = NULL;
@@ -147,12 +145,12 @@ fu_logitech_hidpp_device_ping(FuLogitechHidppDevice *self, GError **error)
 	msg->data[1] = 0x00;
 	msg->data[2] = 0xaa; /* user-selected value */
 	msg->hidpp_version = priv->hidpp_version;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, &error_local)) {
-		if (g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, &error_local)) {
+		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
 			priv->hidpp_version = 1;
 			return TRUE;
 		}
-		if (g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_HOST_UNREACHABLE)) {
+		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND)) {
 			fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNREACHABLE);
 			return TRUE;
 		}
@@ -177,28 +175,14 @@ fu_logitech_hidpp_device_ping(FuLogitechHidppDevice *self, GError **error)
 
 	/* format version in BCD format */
 	if (priv->hidpp_version != FU_HIDPP_VERSION_BLE) {
-		version = (gdouble)msg->data[0] + ((gdouble)msg->data[1]) / 100.f;
-		priv->hidpp_version = (guint)version;
+		/* minor version is in msg->data[1] */;
+		priv->hidpp_version = msg->data[0];
 	}
 
 	/* success */
 	return TRUE;
 }
 
-static gboolean
-fu_logitech_hidpp_device_close(FuDevice *device, GError **error)
-{
-	FuLogitechHidppDevice *self = FU_HIDPP_DEVICE(device);
-	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
-
-	if (priv->io_channel != NULL) {
-		if (!fu_io_channel_shutdown(priv->io_channel, error))
-			return FALSE;
-		g_clear_object(&priv->io_channel);
-	}
-	return TRUE;
-}
-
 static gboolean
 fu_logitech_hidpp_device_poll(FuDevice *device, GError **error)
 {
@@ -217,8 +201,8 @@ fu_logitech_hidpp_device_poll(FuDevice *device, GError **error)
 	/* flush pending data */
 	msg->device_id = priv->device_idx;
 	msg->hidpp_version = priv->hidpp_version;
-	if (!fu_logitech_hidpp_receive(priv->io_channel, msg, timeout, &error_local)) {
-		if (!g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_TIMED_OUT)) {
+	if (!fu_logitech_hidpp_receive(FU_UDEV_DEVICE(self), msg, timeout, &error_local)) {
+		if (!g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT)) {
 			g_warning("failed to get pending read: %s", error_local->message);
 			return TRUE;
 		}
@@ -247,29 +231,14 @@ fu_logitech_hidpp_device_poll(FuDevice *device, GError **error)
 	return TRUE;
 }
 
-static gboolean
-fu_logitech_hidpp_device_open(FuDevice *device, GError **error)
-{
-	FuLogitechHidppDevice *self = FU_HIDPP_DEVICE(device);
-	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
-	const gchar *devpath = fu_udev_device_get_device_file(FU_UDEV_DEVICE(device));
-
-	/* open */
-	priv->io_channel = fu_io_channel_new_file(devpath, error);
-	if (priv->io_channel == NULL)
-		return FALSE;
-
-	return TRUE;
-}
-
 static void
-fu_logitech_hidpp_map_to_string(FuLogitechHidppHidppMap *map, guint idt, GString *str)
+fu_logitech_hidpp_device_map_to_string(FuLogitechHidppHidppMap *map, guint idt, GString *str)
 {
 	g_autofree gchar *title = g_strdup_printf("Feature%02x", map->idx);
 	g_autofree gchar *tmp = g_strdup_printf("%s [0x%04x]",
 						fu_logitech_hidpp_feature_to_string(map->feature),
 						map->feature);
-	fu_string_append(str, idt, title, tmp);
+	fwupd_codec_string_append(str, idt, title, tmp);
 }
 
 static void
@@ -277,17 +246,13 @@ fu_logitech_hidpp_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuLogitechHidppDevice *self = FU_HIDPP_DEVICE(device);
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
-
-	/* FuUdevDevice->to_string */
-	FU_DEVICE_CLASS(fu_logitech_hidpp_device_parent_class)->to_string(device, idt, str);
-
-	fu_string_append_ku(str, idt, "HidppVersion", priv->hidpp_version);
-	fu_string_append_ku(str, idt, "HidppPid", priv->hidpp_pid);
-	fu_string_append_kx(str, idt, "DeviceIdx", priv->device_idx);
-	fu_string_append(str, idt, "ModelId", priv->model_id);
+	fwupd_codec_string_append_int(str, idt, "HidppVersion", priv->hidpp_version);
+	fwupd_codec_string_append_int(str, idt, "HidppPid", priv->hidpp_pid);
+	fwupd_codec_string_append_hex(str, idt, "DeviceIdx", priv->device_idx);
+	fwupd_codec_string_append(str, idt, "ModelId", priv->model_id);
 	for (guint i = 0; i < priv->feature_index->len; i++) {
 		FuLogitechHidppHidppMap *map = g_ptr_array_index(priv->feature_index, i);
-		fu_logitech_hidpp_map_to_string(map, idt, str);
+		fu_logitech_hidpp_device_map_to_string(map, idt, str);
 	}
 }
 
@@ -321,15 +286,17 @@ fu_logitech_hidpp_device_create_radio_child(FuLogitechHidppDevice *self,
 	/* sanity check */
 	if (priv->model_id == NULL) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_SUPPORTED,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_SUPPORTED,
 				    "model ID not set");
 		return FALSE;
 	}
 
 	radio_version = g_strdup_printf("0x%.4x", build);
 	radio = fu_logitech_hidpp_radio_new(ctx, entity);
-	fu_device_set_physical_id(FU_DEVICE(radio), fu_device_get_physical_id(FU_DEVICE(self)));
+	fu_device_incorporate(FU_DEVICE(radio),
+			      FU_DEVICE(self),
+			      FU_DEVICE_INCORPORATE_FLAG_PHYSICAL_ID);
 	/*
 	 * Use the parent logical id as well as the model id for the
 	 * logical id of the radio child device. This allows the radio
@@ -340,7 +307,7 @@ fu_logitech_hidpp_device_create_radio_child(FuLogitechHidppDevice *self,
 	    g_strdup_printf("%s-%s", fu_device_get_logical_id(FU_DEVICE(self)), priv->model_id);
 	fu_device_set_logical_id(FU_DEVICE(radio), logical_id);
 	instance_id = g_strdup_printf("HIDRAW\\VEN_%04X&MOD_%s&ENT_05",
-				      (guint)FU_LOGITECH_HIDPP_DEVICE_VID,
+				      fu_device_get_vid(FU_DEVICE(self)),
 				      priv->model_id);
 	fu_device_add_instance_id(FU_DEVICE(radio), instance_id);
 	fu_device_set_version(FU_DEVICE(radio), radio_version);
@@ -370,6 +337,7 @@ fu_logitech_hidpp_device_fetch_firmware_info(FuLogitechHidppDevice *self, GError
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
 	g_autoptr(FuLogitechHidppHidppMsg) msg_count = fu_logitech_hidpp_msg_new();
 	gboolean radio_ok = FALSE;
+	gboolean app_ok = FALSE;
 
 	/* get the feature index */
 	idx = fu_logitech_hidpp_device_feature_get_idx(self,
@@ -383,7 +351,7 @@ fu_logitech_hidpp_device_fetch_firmware_info(FuLogitechHidppDevice *self, GError
 	msg_count->sub_id = idx;
 	msg_count->function_id = 0x00 << 4; /* getCount */
 	msg_count->hidpp_version = priv->hidpp_version;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg_count, error)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg_count, error)) {
 		g_prefix_error(error, "failed to get firmware count: ");
 		return FALSE;
 	}
@@ -403,10 +371,16 @@ fu_logitech_hidpp_device_fetch_firmware_info(FuLogitechHidppDevice *self, GError
 		msg->function_id = 0x01 << 4; /* getInfo */
 		msg->hidpp_version = priv->hidpp_version;
 		msg->data[0] = i;
-		if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+		if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 			g_prefix_error(error, "failed to get firmware info: ");
 			return FALSE;
 		}
+		/* use the single available slot, otherwise -- the slot which is not active */
+		if (msg->data[0] == 0) {
+			if (!app_ok || FU_BIT_IS_CLEAR(msg->data[8], 0))
+				priv->cached_fw_entity = i;
+			app_ok = TRUE;
+		}
 		if (msg->data[1] == 0x00 && msg->data[2] == 0x00 && msg->data[3] == 0x00 &&
 		    msg->data[4] == 0x00 && msg->data[5] == 0x00 && msg->data[6] == 0x00 &&
 		    msg->data[7] == 0x00) {
@@ -418,8 +392,9 @@ fu_logitech_hidpp_device_fetch_firmware_info(FuLogitechHidppDevice *self, GError
 		version = fu_logitech_hidpp_format_version(name, msg->data[4], msg->data[5], build);
 		g_debug("firmware entity 0x%02x version is %s", i, version);
 		if (msg->data[0] == 0) {
-			fu_device_set_version(FU_DEVICE(self), version);
-			priv->cached_fw_entity = i;
+			/* set version from the active entity */
+			if (FU_BIT_IS_SET(msg->data[8], 0))
+				fu_device_set_version(FU_DEVICE(self), version);
 		} else if (msg->data[0] == 1) {
 			fu_device_set_version_bootloader(FU_DEVICE(self), version);
 		} else if (msg->data[0] == 2) {
@@ -454,6 +429,7 @@ fu_logitech_hidpp_device_fetch_model_id(FuLogitechHidppDevice *self, GError **er
 {
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
 	guint8 idx;
+	guint64 pid_tmp = 0;
 	g_autoptr(FuLogitechHidppHidppMsg) msg = fu_logitech_hidpp_msg_new();
 	g_autoptr(GString) str = g_string_new(NULL);
 
@@ -468,7 +444,7 @@ fu_logitech_hidpp_device_fetch_model_id(FuLogitechHidppDevice *self, GError **er
 	msg->sub_id = idx;
 	msg->function_id = 0x00 << 4; /* getDeviceInfo */
 	msg->hidpp_version = priv->hidpp_version;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 		g_prefix_error(error, "failed to get the model ID: ");
 		return FALSE;
 	}
@@ -478,8 +454,16 @@ fu_logitech_hidpp_device_fetch_model_id(FuLogitechHidppDevice *self, GError **er
 		g_string_append_printf(str, "%02X", msg->data[i]);
 	fu_logitech_hidpp_device_set_model_id(self, str->str);
 
+	/* truncate to 4 chars and convert to a PID */
+	g_string_set_size(str, 4);
+	if (!fu_strtoull(str->str, &pid_tmp, 0, G_MAXUINT32, FU_INTEGER_BASE_16, error)) {
+		g_prefix_error(error, "failed to parse the model ID: ");
+		return FALSE;
+	}
+	fu_device_set_pid(FU_DEVICE(self), (guint32)pid_tmp);
+
 	/* add one more instance ID */
-	fu_device_add_instance_u16(FU_DEVICE(self), "VEN", FU_LOGITECH_HIDPP_DEVICE_VID);
+	fu_device_add_instance_u16(FU_DEVICE(self), "VEN", fu_device_get_vid(FU_DEVICE(self)));
 	fu_device_add_instance_str(FU_DEVICE(self), "MOD", priv->model_id);
 	return fu_device_build_instance_id(FU_DEVICE(self), error, "HIDRAW", "VEN", "MOD", NULL);
 }
@@ -505,14 +489,14 @@ fu_logitech_hidpp_device_fetch_battery_level(FuLogitechHidppDevice *self, GError
 			msg->sub_id = idx;
 			msg->function_id = 0x00 << 4; /* get_capabilities */
 			msg->hidpp_version = priv->hidpp_version;
-			if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+			if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 				g_prefix_error(error, "failed to get battery info: ");
 				return FALSE;
 			}
 			if (msg->data[1] & 0x02)
 				socc = TRUE;
 			msg->function_id = 0x01 << 4; /* get_status */
-			if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+			if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 				g_prefix_error(error, "failed to get battery info: ");
 				return FALSE;
 			}
@@ -551,7 +535,7 @@ fu_logitech_hidpp_device_fetch_battery_level(FuLogitechHidppDevice *self, GError
 			msg->sub_id = idx;
 			msg->function_id = 0x00 << 4; /* GetBatteryLevelStatus */
 			msg->hidpp_version = priv->hidpp_version;
-			if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+			if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 				g_prefix_error(error, "failed to get battery info: ");
 				return FALSE;
 			}
@@ -562,14 +546,14 @@ fu_logitech_hidpp_device_fetch_battery_level(FuLogitechHidppDevice *self, GError
 	}
 
 	/* try HID++1.0 battery mileage */
-	if (priv->hidpp_version == 1.f) {
+	if (priv->hidpp_version == 1) {
 		g_autoptr(FuLogitechHidppHidppMsg) msg = fu_logitech_hidpp_msg_new();
 		msg->report_id = FU_LOGITECH_HIDPP_REPORT_ID_SHORT;
 		msg->device_id = priv->device_idx;
 		msg->sub_id = FU_LOGITECH_HIDPP_SUBID_GET_REGISTER;
 		msg->function_id = FU_LOGITECH_HIDPP_REGISTER_BATTERY_MILEAGE << 4;
 		msg->hidpp_version = priv->hidpp_version;
-		if (fu_logitech_hidpp_transfer(priv->io_channel, msg, NULL)) {
+		if (fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, NULL)) {
 			if (msg->data[0] != 0x7F)
 				fu_device_set_battery_level(FU_DEVICE(self), msg->data[0]);
 			else
@@ -579,7 +563,7 @@ fu_logitech_hidpp_device_fetch_battery_level(FuLogitechHidppDevice *self, GError
 
 		/* try HID++1.0 battery status instead */
 		msg->function_id = FU_LOGITECH_HIDPP_REGISTER_BATTERY_STATUS << 4;
-		if (fu_logitech_hidpp_transfer(priv->io_channel, msg, NULL)) {
+		if (fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, NULL)) {
 			switch (msg->data[0]) {
 			case 1: /* 0 - 10 */
 				fu_device_set_battery_level(FU_DEVICE(self), 5);
@@ -606,9 +590,10 @@ fu_logitech_hidpp_device_fetch_battery_level(FuLogitechHidppDevice *self, GError
 }
 
 static gboolean
-fu_logitech_hidpp_feature_search(FuDevice *device, guint16 feature, GError **error)
+fu_logitech_hidpp_device_feature_search(FuLogitechHidppDevice *self,
+					guint16 feature,
+					GError **error)
 {
-	FuLogitechHidppDevice *self = FU_HIDPP_DEVICE(device);
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
 	FuLogitechHidppHidppMap *map;
 	g_autoptr(FuLogitechHidppHidppMsg) msg = fu_logitech_hidpp_msg_new();
@@ -622,7 +607,7 @@ fu_logitech_hidpp_feature_search(FuDevice *device, guint16 feature, GError **err
 	msg->data[1] = feature;
 	msg->data[2] = 0x00;
 	msg->hidpp_version = priv->hidpp_version;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 		g_prefix_error(error,
 			       "failed to get idx for feature %s [0x%04x]: ",
 			       fu_logitech_hidpp_feature_to_string(feature),
@@ -633,8 +618,8 @@ fu_logitech_hidpp_feature_search(FuDevice *device, guint16 feature, GError **err
 	/* zero index */
 	if (msg->data[0] == 0x00) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "feature %s [0x%04x] not found",
 			    fu_logitech_hidpp_feature_to_string(feature),
 			    feature);
@@ -659,21 +644,8 @@ fu_logitech_hidpp_device_probe(FuDevice *device, GError **error)
 	FuLogitechHidppDevice *self = FU_HIDPP_DEVICE(device);
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
 
-	/* check the kernel has CONFIG_HIDRAW */
-	if (!g_file_test("/sys/class/hidraw", G_FILE_TEST_IS_DIR)) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_NOT_SUPPORTED,
-				    "no kernel support for CONFIG_HIDRAW");
-		return FALSE;
-	}
-
-	/* set the physical ID */
-	if (!fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "hid", error))
-		return FALSE;
-
 	/* nearly... */
-	fu_device_add_vendor_id(device, "USB:0x046D");
+	fu_device_build_vendor_id_u16(device, "USB", 0x046D);
 
 	/*
 	 * All devices connected to a Bolt receiver share the same
@@ -687,14 +659,20 @@ fu_logitech_hidpp_device_probe(FuDevice *device, GError **error)
 		fu_device_set_logical_id(device, id_str->str);
 	}
 
-	/* this is a non-standard extension */
-	fu_device_add_instance_u16(FU_DEVICE(self),
-				   "VID",
-				   fu_udev_device_get_vendor(FU_UDEV_DEVICE(device)));
-	fu_device_add_instance_u16(FU_DEVICE(self),
-				   "PID",
-				   fu_udev_device_get_model(FU_UDEV_DEVICE(device)));
-	return fu_device_build_instance_id(FU_DEVICE(self), error, "UFY", "VID", "PID", NULL);
+	/* success */
+	return TRUE;
+}
+
+static void
+fu_logitech_hidpp_device_vid_pid_notify_cb(FuDevice *device, GParamSpec *pspec, gpointer user_data)
+{
+	if (fu_device_get_pid(device) == 0x0)
+		return;
+
+	/* this is a non-standard extension, but essentially API */
+	fu_device_add_instance_u16(device, "VID", fu_device_get_vid(device));
+	fu_device_add_instance_u16(device, "PID", fu_device_get_pid(device));
+	fu_device_build_instance_id(device, NULL, "UFY", "VID", "PID", NULL);
 }
 
 static gboolean
@@ -716,17 +694,6 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 	if (fu_device_has_private_flag(device, FU_LOGITECH_HIDPP_DEVICE_FLAG_BLE)) {
 		priv->hidpp_version = FU_HIDPP_VERSION_BLE;
 		priv->device_idx = FU_LOGITECH_HIDPP_DEVICE_IDX_RECEIVER;
-		/*
-		 * Set the logical ID for BLE devices. Note that for BLE
-		 * devices, physical_id = HID_PHYS = MAC of the BT adapter,
-		 * logical_id = HID_UNIQ = MAC of the device. The physical id is
-		 * not enough to differentiate two BLE devices connected to the
-		 * same adapter. This is done here because private flags
-		 * are not loaded when the probe method runs, so we
-		 * can't tell the device is in BLE mode.
-		 */
-		if (!fu_udev_device_set_logical_id(FU_UDEV_DEVICE(device), "hid", error))
-			return FALSE;
 		/*
 		 * BLE devices might not be ready for ping right after
 		 * they come up -> wait a bit before pinging.
@@ -758,10 +725,10 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 	/* map some *optional* HID++2.0 features we might use */
 	for (guint i = 0; map_features[i] != FU_LOGITECH_HIDPP_FEATURE_ROOT; i++) {
 		g_autoptr(GError) error_local = NULL;
-		if (!fu_logitech_hidpp_feature_search(device, map_features[i], &error_local)) {
+		if (!fu_logitech_hidpp_device_feature_search(self, map_features[i], &error_local)) {
 			g_debug("%s", error_local->message);
-			if (g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) ||
-			    g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_HOST_UNREACHABLE)) {
+			if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT) ||
+			    g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND)) {
 				/* timed out, so not trying any more */
 				break;
 			}
@@ -784,7 +751,7 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 		msg->sub_id = idx;
 		msg->function_id = 0x02 << 4; /* getDeviceType */
 		msg->hidpp_version = priv->hidpp_version;
-		if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+		if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 			g_prefix_error(error, "failed to get device type: ");
 			return FALSE;
 		}
@@ -800,6 +767,7 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 	idx = fu_logitech_hidpp_device_feature_get_idx(self, FU_LOGITECH_HIDPP_FEATURE_DFU_CONTROL);
 	if (idx != 0x00) {
 		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
+		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_UPDATABLE);
 		fu_device_remove_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_IS_BOOTLOADER);
 		fu_device_add_protocol(FU_DEVICE(self), "com.logitech.unifying");
 	}
@@ -817,7 +785,7 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 		msg->sub_id = idx;
 		msg->function_id = 0x00 << 4; /* getDfuStatus */
 		msg->hidpp_version = priv->hidpp_version;
-		if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+		if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 			g_prefix_error(error, "failed to get DFU status: ");
 			return FALSE;
 		}
@@ -828,10 +796,13 @@ fu_logitech_hidpp_device_setup(FuDevice *device, GError **error)
 		}
 		fu_device_add_protocol(FU_DEVICE(device), "com.logitech.unifyingsigned");
 		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
+		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_UPDATABLE);
 	}
 	idx = fu_logitech_hidpp_device_feature_get_idx(self, FU_LOGITECH_HIDPP_FEATURE_DFU);
 	if (idx != 0x00) {
 		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_IS_BOOTLOADER);
+		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
+		fu_device_add_flag(FU_DEVICE(device), FWUPD_DEVICE_FLAG_UPDATABLE);
 		if (fu_device_get_version(device) == NULL) {
 			g_info("repairing device in bootloader mode");
 			fu_device_set_version(FU_DEVICE(device), "MPK00.00_B0000");
@@ -894,7 +865,7 @@ fu_logitech_hidpp_device_detach(FuDevice *device, FuProgress *progress, GError *
 		msg->hidpp_version = priv->hidpp_version;
 		msg->flags = FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_IGNORE_SUB_ID |
 			     FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_LONGER_TIMEOUT;
-		if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, &error_local)) {
+		if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, &error_local)) {
 			if (fu_device_has_private_flag(
 				device,
 				FU_LOGITECH_HIDPP_DEVICE_FLAG_NO_REQUEST_REQUIRED)) {
@@ -953,7 +924,7 @@ fu_logitech_hidpp_device_detach(FuDevice *device, FuProgress *progress, GError *
 		msg->data[5] = 'F';
 		msg->data[6] = 'U';
 		msg->flags = FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_IGNORE_SUB_ID;
-		if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+		if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 			g_prefix_error(error, "failed to put device into DFU mode: ");
 			return FALSE;
 		}
@@ -962,110 +933,61 @@ fu_logitech_hidpp_device_detach(FuDevice *device, FuProgress *progress, GError *
 	}
 
 	/* we don't know how */
-	g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "no method to detach");
+	g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no method to detach");
 	return FALSE;
 }
 
 static gboolean
-fu_logitech_hidpp_device_check_status(guint8 status, GError **error)
+fu_logitech_hidpp_device_check_status(guint8 status_raw, GError **error)
 {
-	switch (status & 0x7f) {
-	case 0x00:
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
-			    "invalid status value 0x%02x",
-			    status);
-		break;
-	case 0x01: /* packet success */
-	case 0x02: /* DFU success */
-	case 0x05: /* DFU success: entity restart required */
-	case 0x06: /* DFU success: system restart required */
+	FuLogitechHidppStatus status = status_raw & 0x7f;
+	const gchar *status_str = fu_logitech_hidpp_status_to_string(status);
+	switch (status) {
+	case FU_LOGITECH_HIDPP_STATUS_PACKET_SUCCESS:
+	case FU_LOGITECH_HIDPP_STATUS_DFU_SUCCESS:
+	case FU_LOGITECH_HIDPP_STATUS_DFU_SUCCESS_ENTITY_RESTART_REQUIRED:
+	case FU_LOGITECH_HIDPP_STATUS_DFU_SUCCESS_SYSTEM_RESTART_REQUIRED:
 		/* success */
+		g_debug("ignoring: %s", status_str);
 		return TRUE;
 		break;
-	case 0x03:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_PENDING,
-				    "wait for event (command in progress)");
+	case FU_LOGITECH_HIDPP_STATUS_COMMAND_IN_PROGRESS:
+	case FU_LOGITECH_HIDPP_STATUS_WAIT_FOR_EVENT:
+	case FU_LOGITECH_HIDPP_STATUS_BLOCKED_COMMAND:
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_BUSY, status_str);
 		break;
-	case 0x04:
-	case 0x10: /* unknown */
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "generic error");
+	case FU_LOGITECH_HIDPP_STATUS_GENERIC_ERROR04:
+	case FU_LOGITECH_HIDPP_STATUS_GENERIC_ERROR10:
+	case FU_LOGITECH_HIDPP_STATUS_BAD_VOLTAGE:
+	case FU_LOGITECH_HIDPP_STATUS_DFU_NOT_STARTED:
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, status_str);
 		break;
-	case 0x11:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
-				    "bad voltage (power too low?)");
+	case FU_LOGITECH_HIDPP_STATUS_UNKNOWN12:
+	case FU_LOGITECH_HIDPP_STATUS_BAD_MAGIC_STRING:
+	case FU_LOGITECH_HIDPP_STATUS_BAD_FIRMWARE:
+	case FU_LOGITECH_HIDPP_STATUS_UNSUPPORTED_ENCRYPTION_MODE:
+	case FU_LOGITECH_HIDPP_STATUS_UNSUPPORTED_COMMAND:
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, status_str);
 		break;
-	case 0x12:
-	case 0x14: /* bad magic string */
-	case 0x21: /* bad firmware */
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "unsupported firmware");
+	case FU_LOGITECH_HIDPP_STATUS_INVALID:
+	case FU_LOGITECH_HIDPP_STATUS_BAD_SEQUENCE_NUMBER:
+	case FU_LOGITECH_HIDPP_STATUS_ADDRESS_OUT_OF_RANGE:
+	case FU_LOGITECH_HIDPP_STATUS_UNALIGNED_ADDRESS:
+	case FU_LOGITECH_HIDPP_STATUS_BAD_SIZE:
+	case FU_LOGITECH_HIDPP_STATUS_MISSING_PROGRAM_DATA:
+	case FU_LOGITECH_HIDPP_STATUS_MISSING_CHECK_DATA:
+	case FU_LOGITECH_HIDPP_STATUS_FIRMWARE_CHECK_FAILURE:
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA, status_str);
 		break;
-	case 0x13:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
-				    "unsupported encryption mode");
-		break;
-	case 0x15:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "erase failure");
-		break;
-	case 0x16:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "DFU not started");
-		break;
-	case 0x17:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "bad sequence number");
-		break;
-	case 0x18:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "unsupported command");
-		break;
-	case 0x19:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "command in progress");
-		break;
-	case 0x1a:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "address out of range");
-		break;
-	case 0x1b:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "unaligned address");
-		break;
-	case 0x1c:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "bad size");
-		break;
-	case 0x1d:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "missing program data");
-		break;
-	case 0x1e:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "missing check data");
-		break;
-	case 0x1f:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
-				    "program failed to write");
-		break;
-	case 0x20:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
-				    "program failed to verify");
-		break;
-	case 0x22:
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, "firmware check failure");
-		break;
-	case 0x23:
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
-				    "blocked command (restart required)");
+	case FU_LOGITECH_HIDPP_STATUS_PROGRAM_FAILED_TO_WRITE:
+	case FU_LOGITECH_HIDPP_STATUS_PROGRAM_FAILED_TO_VERIFY:
+	case FU_LOGITECH_HIDPP_STATUS_ERASE_FAILURE:
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_WRITE, status_str);
 		break;
 	default:
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "unhandled status value 0x%02x",
 			    status);
 		break;
@@ -1104,7 +1026,7 @@ fu_logitech_hidpp_device_write_firmware_pkt(FuLogitechHidppDevice *self,
 			    16,
 			    error))
 		return FALSE;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, error)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, error)) {
 		g_prefix_error(error, "failed to supply program data: ");
 		return FALSE;
 	}
@@ -1122,8 +1044,8 @@ fu_logitech_hidpp_device_write_firmware_pkt(FuLogitechHidppDevice *self,
 		return TRUE;
 
 	/* fatal error */
-	if (!g_error_matches(error_local, G_IO_ERROR, G_IO_ERROR_PENDING)) {
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_FAILED, error_local->message);
+	if (!g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_BUSY)) {
+		g_propagate_error(error, g_steal_pointer(&error_local));
 		return FALSE;
 	}
 
@@ -1132,7 +1054,7 @@ fu_logitech_hidpp_device_write_firmware_pkt(FuLogitechHidppDevice *self,
 	for (guint retry = 0; retry < 10; retry++) {
 		g_autoptr(FuLogitechHidppHidppMsg) msg2 = fu_logitech_hidpp_msg_new();
 		msg2->flags = FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_IGNORE_FNCT_ID;
-		if (!fu_logitech_hidpp_receive(priv->io_channel, msg2, 15000, error))
+		if (!fu_logitech_hidpp_receive(FU_UDEV_DEVICE(self), msg2, 15000, error))
 			return FALSE;
 		if (fu_logitech_hidpp_msg_is_reply(msg, msg2)) {
 			g_autoptr(GError) error2 = NULL;
@@ -1147,8 +1069,8 @@ fu_logitech_hidpp_device_write_firmware_pkt(FuLogitechHidppDevice *self,
 
 	/* nothing in the queue */
 	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_TIMED_OUT,
 			    "failed to get event after timeout");
 	return FALSE;
 }
@@ -1171,7 +1093,10 @@ fu_logitech_hidpp_device_write_firmware(FuDevice *device,
 	/* if we're in bootloader mode, we should be able to get this feature */
 	idx = fu_logitech_hidpp_device_feature_get_idx(self, FU_LOGITECH_HIDPP_FEATURE_DFU);
 	if (idx == 0x00) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "no DFU feature available");
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "no DFU feature available");
 		return FALSE;
 	}
 
@@ -1236,7 +1161,10 @@ fu_logitech_hidpp_device_attach(FuLogitechHidppDevice *self,
 	/* if we're in bootloader mode, we should be able to get this feature */
 	idx = fu_logitech_hidpp_device_feature_get_idx(self, FU_LOGITECH_HIDPP_FEATURE_DFU);
 	if (idx == 0x00) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "no DFU feature available");
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "no DFU feature available");
 		return FALSE;
 	}
 
@@ -1250,7 +1178,7 @@ fu_logitech_hidpp_device_attach(FuLogitechHidppDevice *self,
 	msg->flags = FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_IGNORE_SUB_ID |
 		     FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_IGNORE_SWID | // inferred?
 		     FU_LOGITECH_HIDPP_HIDPP_MSG_FLAG_LONGER_TIMEOUT;
-	if (!fu_logitech_hidpp_transfer(priv->io_channel, msg, &error_local)) {
+	if (!fu_logitech_hidpp_transfer(FU_UDEV_DEVICE(self), msg, &error_local)) {
 		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_READ) ||
 		    g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND)) {
 			g_debug("ignoring '%s' on reset", error_local->message);
@@ -1263,12 +1191,7 @@ fu_logitech_hidpp_device_attach(FuLogitechHidppDevice *self,
 
 	if (fu_device_has_private_flag(device, FU_LOGITECH_HIDPP_DEVICE_FLAG_REBIND_ATTACH)) {
 		fu_device_set_poll_interval(device, 0);
-		/*
-		 * Wait for device to become ready after flashing.
-		 * Possible race condition: after the device is reset, Linux might enumerate it as
-		 * a different hidraw device depending on timing.
-		 */
-		fu_device_sleep_full(FU_DEVICE(self), 1000, progress); /* ms */
+		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
 	} else {
 		/* device file hasn't been unbound/re-bound, just probe again */
 		if (!fu_device_retry(device, fu_logitech_hidpp_device_reprobe_cb, 10, NULL, error))
@@ -1301,7 +1224,10 @@ fu_logitech_hidpp_device_set_quirk_kv(FuDevice *device,
 		fu_logitech_hidpp_device_set_model_id(self, value);
 		return TRUE;
 	}
-	g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "quirk key not supported");
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "quirk key not supported");
 	return FALSE;
 }
 
@@ -1341,22 +1267,20 @@ fu_logitech_hidpp_device_cleanup(FuDevice *device,
 static void
 fu_logitech_hidpp_device_class_init(FuLogitechHidppDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
 	object_class->finalize = fu_logitech_hidpp_device_finalize;
-	klass_device->setup = fu_logitech_hidpp_device_setup;
-	klass_device->open = fu_logitech_hidpp_device_open;
-	klass_device->close = fu_logitech_hidpp_device_close;
-	klass_device->write_firmware = fu_logitech_hidpp_device_write_firmware;
-	klass_device->attach = fu_logitech_hidpp_device_attach_cached;
-	klass_device->detach = fu_logitech_hidpp_device_detach;
-	klass_device->poll = fu_logitech_hidpp_device_poll;
-	klass_device->to_string = fu_logitech_hidpp_device_to_string;
-	klass_device->probe = fu_logitech_hidpp_device_probe;
-	klass_device->set_quirk_kv = fu_logitech_hidpp_device_set_quirk_kv;
-	klass_device->cleanup = fu_logitech_hidpp_device_cleanup;
-	klass_device->set_progress = fu_logitech_hidpp_device_set_progress;
+	device_class->setup = fu_logitech_hidpp_device_setup;
+	device_class->write_firmware = fu_logitech_hidpp_device_write_firmware;
+	device_class->attach = fu_logitech_hidpp_device_attach_cached;
+	device_class->detach = fu_logitech_hidpp_device_detach;
+	device_class->poll = fu_logitech_hidpp_device_poll;
+	device_class->to_string = fu_logitech_hidpp_device_to_string;
+	device_class->probe = fu_logitech_hidpp_device_probe;
+	device_class->set_quirk_kv = fu_logitech_hidpp_device_set_quirk_kv;
+	device_class->cleanup = fu_logitech_hidpp_device_cleanup;
+	device_class->set_progress = fu_logitech_hidpp_device_set_progress;
 }
 
 static void
@@ -1365,43 +1289,41 @@ fu_logitech_hidpp_device_init(FuLogitechHidppDevice *self)
 	FuLogitechHidppDevicePrivate *priv = GET_PRIVATE(self);
 	priv->device_idx = FU_LOGITECH_HIDPP_DEVICE_IDX_WIRED;
 	priv->feature_index = g_ptr_array_new_with_free_func(g_free);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_request_flag(FU_DEVICE(self), FWUPD_REQUEST_FLAG_ALLOW_GENERIC_MESSAGE);
+	fu_device_set_vid(FU_DEVICE(self), FU_LOGITECH_HIDPP_DEVICE_VID);
 	fu_device_set_remove_delay(FU_DEVICE(self), FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PLAIN);
 	fu_device_set_vendor(FU_DEVICE(self), "Logitech");
 	fu_device_retry_set_delay(FU_DEVICE(self), 1000);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_READ);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_WRITE);
 	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_LOGITECH_HIDPP_DEVICE_FLAG_FORCE_RECEIVER_ID,
-					"force-receiver-id");
-	fu_device_register_private_flag(FU_DEVICE(self), FU_LOGITECH_HIDPP_DEVICE_FLAG_BLE, "ble");
+					FU_LOGITECH_HIDPP_DEVICE_FLAG_FORCE_RECEIVER_ID);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_LOGITECH_HIDPP_DEVICE_FLAG_BLE);
 	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_LOGITECH_HIDPP_DEVICE_FLAG_REBIND_ATTACH,
-					"rebind-attach");
+					FU_LOGITECH_HIDPP_DEVICE_FLAG_REBIND_ATTACH);
 	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_LOGITECH_HIDPP_DEVICE_FLAG_NO_REQUEST_REQUIRED,
-					"no-request-required");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_LOGITECH_HIDPP_DEVICE_FLAG_ADD_RADIO,
-					"add-radio");
+					FU_LOGITECH_HIDPP_DEVICE_FLAG_NO_REQUEST_REQUIRED);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_LOGITECH_HIDPP_DEVICE_FLAG_ADD_RADIO);
 	fu_device_set_remove_delay(FU_DEVICE(self), FU_DEVICE_REMOVE_DELAY_USER_REPLUG);
 	fu_device_set_battery_threshold(FU_DEVICE(self), 20);
+	g_signal_connect(FU_DEVICE(self),
+			 "notify::vid",
+			 G_CALLBACK(fu_logitech_hidpp_device_vid_pid_notify_cb),
+			 NULL);
+	g_signal_connect(FU_DEVICE(self),
+			 "notify::pid",
+			 G_CALLBACK(fu_logitech_hidpp_device_vid_pid_notify_cb),
+			 NULL);
 }
 
 FuLogitechHidppDevice *
 fu_logitech_hidpp_device_new(FuUdevDevice *parent)
 {
-	FuLogitechHidppDevice *self = NULL;
-	FuLogitechHidppDevicePrivate *priv;
-	self = g_object_new(FU_TYPE_HIDPP_DEVICE,
-			    "context",
-			    fu_device_get_context(FU_DEVICE(parent)),
-			    "physical-id",
-			    fu_device_get_physical_id(FU_DEVICE(parent)),
-			    "udev-device",
-			    fu_udev_device_get_dev(parent),
+	return g_object_new(FU_TYPE_HIDPP_DEVICE,
+			    "proxy",
+			    parent,
+			    "device-file",
+			    fu_udev_device_get_device_file(parent),
 			    NULL);
-	priv = GET_PRIVATE(self);
-	priv->io_channel = fu_logitech_hidpp_runtime_get_io_channel(FU_HIDPP_RUNTIME(parent));
-	return self;
 }
