@@ -1,93 +1,55 @@
 /*
- * Copyright (C) 2020 Jimmy Yu <Jimmy_yu@pixart.com>
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 Jimmy Yu <Jimmy_yu@pixart.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "config.h"
 
-#ifdef HAVE_HIDRAW_H
-#include <linux/hidraw.h>
-#include <linux/input.h>
-#endif
-
 #include "fu-pxi-firmware.h"
 #include "fu-pxi-receiver-device.h"
 #include "fu-pxi-struct.h"
 #include "fu-pxi-wireless-device.h"
 
 struct _FuPxiReceiverDevice {
-	FuUdevDevice parent_instance;
+	FuHidrawDevice parent_instance;
 	struct ota_fw_state fwstate;
 	guint8 sn;
-	guint vendor;
-	guint product;
 };
 
-G_DEFINE_TYPE(FuPxiReceiverDevice, fu_pxi_receiver_device, FU_TYPE_UDEV_DEVICE)
-
-#ifdef HAVE_HIDRAW_H
-static gboolean
-fu_pxi_receiver_device_get_raw_info(FuPxiReceiverDevice *self,
-				    struct hidraw_devinfo *info,
-				    GError **error)
-{
-	if (!fu_udev_device_ioctl(FU_UDEV_DEVICE(self),
-				  HIDIOCGRAWINFO,
-				  (guint8 *)info,
-				  NULL,
-				  FU_PXI_DEVICE_IOCTL_TIMEOUT,
-				  error)) {
-		return FALSE;
-	}
-	return TRUE;
-}
-#endif
+G_DEFINE_TYPE(FuPxiReceiverDevice, fu_pxi_receiver_device, FU_TYPE_HIDRAW_DEVICE)
 
 static void
 fu_pxi_receiver_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuPxiReceiverDevice *self = FU_PXI_RECEIVER_DEVICE(device);
 	fu_pxi_ota_fw_state_to_string(&self->fwstate, idt, str);
-	fu_string_append_kx(str, idt, "Vendor", self->vendor);
-	fu_string_append_kx(str, idt, "Product", self->product);
 }
 
 static FuFirmware *
 fu_pxi_receiver_device_prepare_firmware(FuDevice *device,
-					GBytes *fw,
+					GInputStream *stream,
+					FuProgress *progress,
 					FwupdInstallFlags flags,
 					GError **error)
 {
 	g_autoptr(FuFirmware) firmware = fu_pxi_firmware_new();
 
-	if (!fu_firmware_parse(firmware, fw, flags, error))
+	if (!fu_firmware_parse_stream(firmware, stream, 0x0, flags, error))
 		return NULL;
 
 	if (fu_device_has_private_flag(device, FU_PXI_DEVICE_FLAG_IS_HPAC) &&
 	    fu_pxi_firmware_is_hpac(FU_PXI_FIRMWARE(firmware))) {
-		g_autoptr(GBytes) fw_tmp = NULL;
 		guint32 hpac_fw_size = 0;
-		const guint8 *fw_ptr = g_bytes_get_data(fw, NULL);
+		g_autoptr(GInputStream) stream_new = NULL;
 
-		if (!fu_memread_uint32_safe(fw_ptr,
-					    g_bytes_get_size(fw),
-					    9,
-					    &hpac_fw_size,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u32(stream, 9, &hpac_fw_size, G_LITTLE_ENDIAN, error))
 			return NULL;
-		hpac_fw_size += 264;
-		fw_tmp = fu_bytes_new_offset(fw, 9, hpac_fw_size, error);
-		if (fw_tmp == NULL) {
-			g_set_error(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_INVALID_FILE,
-				    "HPAC F/W preparation failed.");
+		stream_new = fu_partial_input_stream_new(stream, 9, hpac_fw_size + 264, error);
+		if (stream_new == NULL)
+			return NULL;
+		if (!fu_firmware_set_stream(firmware, stream_new, error))
 			return NULL;
-		}
-
-		fu_firmware_set_bytes(firmware, fw_tmp);
 	} else if (fu_device_has_private_flag(device, FU_PXI_DEVICE_FLAG_IS_HPAC) !=
 		   fu_pxi_firmware_is_hpac(FU_PXI_FIRMWARE(firmware))) {
 		g_set_error(error,
@@ -100,55 +62,6 @@ fu_pxi_receiver_device_prepare_firmware(FuDevice *device,
 	return g_steal_pointer(&firmware);
 }
 
-static gboolean
-fu_pxi_receiver_device_set_feature(FuPxiReceiverDevice *self,
-				   const guint8 *buf,
-				   guint bufsz,
-				   GError **error)
-{
-#ifdef HAVE_HIDRAW_H
-	fu_dump_raw(G_LOG_DOMAIN, "SetFeature", buf, bufsz);
-	return fu_udev_device_ioctl(FU_UDEV_DEVICE(self),
-				    HIDIOCSFEATURE(bufsz),
-				    (guint8 *)buf,
-				    NULL,
-				    FU_PXI_DEVICE_IOCTL_TIMEOUT,
-				    error);
-#else
-	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
-			    "<linux/hidraw.h> not available");
-	return FALSE;
-#endif
-}
-
-static gboolean
-fu_pxi_receiver_device_get_feature(FuPxiReceiverDevice *self,
-				   guint8 *buf,
-				   guint bufsz,
-				   GError **error)
-{
-#ifdef HAVE_HIDRAW_H
-	if (!fu_udev_device_ioctl(FU_UDEV_DEVICE(self),
-				  HIDIOCGFEATURE(bufsz),
-				  buf,
-				  NULL,
-				  FU_PXI_DEVICE_IOCTL_TIMEOUT,
-				  error)) {
-		return FALSE;
-	}
-	fu_dump_raw(G_LOG_DOMAIN, "GetFeature", buf, bufsz);
-	return TRUE;
-#else
-	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
-			    "<linux/hidraw.h> not available");
-	return FALSE;
-#endif
-}
-
 static gboolean
 fu_pxi_receiver_device_fw_ota_init_new(FuPxiReceiverDevice *device, gsize bufsz, GError **error)
 {
@@ -173,13 +86,11 @@ fu_pxi_receiver_device_fw_ota_init_new(FuPxiReceiverDevice *device, gsize bufsz,
 					   error))
 		return FALSE;
 
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
-		return FALSE;
-
-	return TRUE;
+	return fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					    receiver_device_cmd->data,
+					    receiver_device_cmd->len,
+					    FU_IOCTL_FLAG_NONE,
+					    error);
 }
 
 static gboolean
@@ -205,16 +116,21 @@ fu_pxi_receiver_device_fw_ota_ini_new_check(FuPxiReceiverDevice *device, GError
 					   error))
 		return FALSE;
 
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	/* delay for wireless module device read command */
 	fu_device_sleep(FU_DEVICE(device), 5); /* ms */
 	buf[0] = PXI_HID_WIRELESS_DEV_OTA_REPORT_ID;
-	if (!fu_pxi_receiver_device_get_feature(self, buf, sizeof(buf), error))
+	if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(self),
+					  buf,
+					  sizeof(buf),
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	/* shared state */
@@ -222,7 +138,7 @@ fu_pxi_receiver_device_fw_ota_ini_new_check(FuPxiReceiverDevice *device, GError
 }
 
 static gboolean
-fu_pxi_receiver_device_get_cmd_response(FuPxiReceiverDevice *device,
+fu_pxi_receiver_device_get_cmd_response(FuPxiReceiverDevice *self,
 					guint8 *buf,
 					guint bufsz,
 					GError **error)
@@ -233,15 +149,19 @@ fu_pxi_receiver_device_get_cmd_response(FuPxiReceiverDevice *device,
 		memset(buf, 0, bufsz);
 		buf[0] = PXI_HID_WIRELESS_DEV_OTA_REPORT_ID;
 
-		fu_device_sleep(FU_DEVICE(device), 5); /* ms */
+		fu_device_sleep(FU_DEVICE(self), 5); /* ms */
 
-		if (!fu_pxi_receiver_device_get_feature(device, buf, bufsz, error))
+		if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(self),
+						  buf,
+						  bufsz,
+						  FU_IOCTL_FLAG_NONE,
+						  error))
 			return FALSE;
 
 		if (!fu_memread_uint8_safe(buf, bufsz, 0x4, &sn, error))
 			return FALSE;
 
-		if (device->sn != sn)
+		if (self->sn != sn)
 			retry++;
 		else
 			break;
@@ -253,7 +173,7 @@ fu_pxi_receiver_device_get_cmd_response(FuPxiReceiverDevice *device,
 				    "reach retry maximum, hid sn fail, "
 				    "got 0x%04x, expected 0x%04x",
 				    sn,
-				    device->sn);
+				    self->sn);
 			return FALSE;
 		}
 	}
@@ -286,10 +206,11 @@ fu_pxi_receiver_device_check_crc(FuDevice *device, guint16 checksum, GError **er
 					   error))
 		return FALSE;
 
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	if (!fu_pxi_receiver_device_get_cmd_response(self, buf, sizeof(buf), error))
@@ -337,10 +258,11 @@ fu_pxi_receiver_device_fw_object_create(FuDevice *device, FuChunk *chk, GError *
 					   error))
 		return FALSE;
 
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	if (!fu_pxi_receiver_device_get_cmd_response(self, buf, sizeof(buf), error))
@@ -388,10 +310,11 @@ fu_pxi_receiver_device_write_payload(FuDevice *device, FuChunk *chk, GError **er
 					   error))
 		return FALSE;
 
-	return fu_pxi_receiver_device_set_feature(self,
-						  receiver_device_cmd->data,
-						  receiver_device_cmd->len,
-						  error);
+	return fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					    receiver_device_cmd->data,
+					    receiver_device_cmd->len,
+					    FU_IOCTL_FLAG_NONE,
+					    error);
 }
 
 static gboolean
@@ -409,10 +332,17 @@ fu_pxi_receiver_device_write_chunk(FuDevice *device, FuChunk *chk, GError **erro
 	/* write payload */
 	chunks = fu_chunk_array_new_from_bytes(chk_bytes,
 					       fu_chunk_get_address(chk),
+					       FU_CHUNK_PAGESZ_NONE,
 					       self->fwstate.mtu_size);
 
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
-		g_autoptr(FuChunk) chk2 = fu_chunk_array_index(chunks, i);
+		g_autoptr(FuChunk) chk2 = NULL;
+
+		/* prepare chunk */
+		chk2 = fu_chunk_array_index(chunks, i, error);
+		if (chk2 == NULL)
+			return FALSE;
+
 		/* calculate checksum of each payload packet */
 		self->fwstate.checksum +=
 		    fu_sum16(fu_chunk_get_data(chk2), fu_chunk_get_data_sz(chk2));
@@ -497,10 +427,11 @@ fu_pxi_receiver_device_fw_upgrade(FuDevice *device,
 	fu_progress_step_done(progress);
 
 	/* send ota fw upgrade command */
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	/* delay for wireless module device read command */
@@ -535,10 +466,8 @@ fu_pxi_receiver_device_reset(FuDevice *device, GError **error)
 
 	/* ota mcu reset command */
 	fu_byte_array_append_uint8(ota_cmd, 0x1); /* ota mcu reset command */
-	fu_byte_array_append_uint8(
-	    ota_cmd,
-	    FU_PXI_DEVICE_CMD_FW_MCU_RESET);		/* ota mcu reset command op code */
-	fu_byte_array_append_uint8(ota_cmd, OTA_RESET); /* ota mcu reset command reason */
+	fu_byte_array_append_uint8(ota_cmd, FU_PXI_DEVICE_CMD_FW_MCU_RESET); /* op code */
+	fu_byte_array_append_uint8(ota_cmd, FU_PXI_OTA_DISCONNECT_REASON_RESET);
 
 	self->sn++;
 	/* get pixart wireless module ota command */
@@ -551,10 +480,11 @@ fu_pxi_receiver_device_reset(FuDevice *device, GError **error)
 		return FALSE;
 
 	/* send ota mcu reset command */
-	return fu_pxi_receiver_device_set_feature(self,
-						  receiver_device_cmd->data,
-						  receiver_device_cmd->len,
-						  error);
+	return fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					    receiver_device_cmd->data,
+					    receiver_device_cmd->len,
+					    FU_IOCTL_FLAG_NONE,
+					    error);
 }
 
 static gboolean
@@ -588,14 +518,22 @@ fu_pxi_receiver_device_write_firmware(FuDevice *device,
 		return FALSE;
 	fu_progress_step_done(progress);
 
-	chunks = fu_chunk_array_new_from_bytes(fw, 0x0, FU_PXI_DEVICE_OBJECT_SIZE_MAX);
+	chunks = fu_chunk_array_new_from_bytes(fw,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       FU_PXI_DEVICE_OBJECT_SIZE_MAX);
 	/* prepare write fw into device */
 	self->fwstate.offset = 0;
 	self->fwstate.checksum = 0;
 
 	/* write fw into device */
 	for (guint i = self->fwstate.offset; i < fu_chunk_array_length(chunks); i++) {
-		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i);
+		g_autoptr(FuChunk) chk = NULL;
+
+		/* prepare chunk */
+		chk = fu_chunk_array_index(chunks, i, error);
+		if (chk == NULL)
+			return FALSE;
 		if (!fu_pxi_receiver_device_write_chunk(device, chk, error))
 			return FALSE;
 		fu_progress_set_percentage_full(fu_progress_get_child(progress),
@@ -626,7 +564,7 @@ fu_pxi_receiver_device_write_firmware(FuDevice *device,
 }
 
 static gboolean
-fu_pxi_receiver_device_get_peripheral_info(FuPxiReceiverDevice *device,
+fu_pxi_receiver_device_get_peripheral_info(FuPxiReceiverDevice *self,
 					   struct ota_fw_dev_model *model,
 					   guint idx,
 					   GError **error)
@@ -641,25 +579,30 @@ fu_pxi_receiver_device_get_peripheral_info(FuPxiReceiverDevice *device,
 	fu_byte_array_append_uint8(ota_cmd, 0x1); /* ota init new command length */
 	fu_byte_array_append_uint8(ota_cmd, FU_PXI_DEVICE_CMD_FW_OTA_GET_MODEL);
 	fu_byte_array_append_uint8(ota_cmd, idx);
-	device->sn++;
+	self->sn++;
 
 	if (!fu_pxi_composite_receiver_cmd(FU_PXI_DEVICE_CMD_FW_OTA_GET_MODEL,
-					   device->sn,
+					   self->sn,
 					   FU_PXI_WIRELESS_DEVICE_TARGET_RECEIVER,
 					   receiver_device_cmd,
 					   ota_cmd,
 					   error))
 		return FALSE;
-	if (!fu_pxi_receiver_device_set_feature(device,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
-	fu_device_sleep(FU_DEVICE(device), 5); /* ms */
+	fu_device_sleep(FU_DEVICE(self), 5); /* ms */
 	buf[0] = PXI_HID_WIRELESS_DEV_OTA_REPORT_ID;
 
-	if (!fu_pxi_receiver_device_get_feature(device, buf, sizeof(buf), error))
+	if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(self),
+					  buf,
+					  sizeof(buf),
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	fu_dump_raw(G_LOG_DOMAIN, "model_info", buf, sizeof(buf));
@@ -698,7 +641,7 @@ fu_pxi_receiver_device_get_peripheral_info(FuPxiReceiverDevice *device,
 	/* set current version and model name */
 	model->checksum = checksum;
 	g_debug("checksum %x", model->checksum);
-	if (!fu_device_has_private_flag(FU_DEVICE(device), FU_PXI_DEVICE_FLAG_IS_HPAC)) {
+	if (!fu_device_has_private_flag(FU_DEVICE(self), FU_PXI_DEVICE_FLAG_IS_HPAC)) {
 		version_str = g_strndup((gchar *)model->version, 5);
 	} else {
 		if (!fu_memread_uint16_safe(model->version, 5, 3, &hpac_ver, G_BIG_ENDIAN, error))
@@ -733,16 +676,21 @@ fu_pxi_receiver_device_get_peripheral_num(FuPxiReceiverDevice *device,
 					   ota_cmd,
 					   error))
 		return FALSE;
-	if (!fu_pxi_receiver_device_set_feature(self,
-						receiver_device_cmd->data,
-						receiver_device_cmd->len,
-						error))
+	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
+					  receiver_device_cmd->data,
+					  receiver_device_cmd->len,
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 
 	fu_device_sleep(FU_DEVICE(device), 5); /* ms */
 
 	buf[0] = PXI_HID_WIRELESS_DEV_OTA_REPORT_ID;
-	if (!fu_pxi_receiver_device_get_feature(device, buf, sizeof(buf), error))
+	if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(self),
+					  buf,
+					  sizeof(buf),
+					  FU_IOCTL_FLAG_NONE,
+					  error))
 		return FALSE;
 	fu_dump_raw(G_LOG_DOMAIN, "buf from get model num", buf, sizeof(buf));
 	if (!fu_memread_uint8_safe(buf, sizeof(buf), 0xA, num_of_models, error))
@@ -755,7 +703,6 @@ fu_pxi_receiver_device_get_peripheral_num(FuPxiReceiverDevice *device,
 static gboolean
 fu_pxi_receiver_device_add_peripherals(FuPxiReceiverDevice *device, guint idx, GError **error)
 {
-#ifdef HAVE_HIDRAW_H
 	struct ota_fw_dev_model model = {0x0};
 	guint16 hpac_ver = 0;
 	g_autofree gchar *model_name = NULL;
@@ -776,8 +723,12 @@ fu_pxi_receiver_device_add_peripherals(FuPxiReceiverDevice *device, guint idx, G
 	/* idx 0 is for local_device */
 	if (idx == 0) {
 		fu_device_set_version(FU_DEVICE(device), model_version);
-		fu_device_add_instance_u16(FU_DEVICE(device), "VEN", device->vendor);
-		fu_device_add_instance_u16(FU_DEVICE(device), "DEV", device->product);
+		fu_device_add_instance_u16(FU_DEVICE(device),
+					   "VEN",
+					   fu_device_get_vid(FU_DEVICE(device)));
+		fu_device_add_instance_u16(FU_DEVICE(device),
+					   "DEV",
+					   fu_device_get_pid(FU_DEVICE(device)));
 		fu_device_add_instance_str(FU_DEVICE(device), "MODEL", model_name);
 		if (!fu_device_build_instance_id(FU_DEVICE(device),
 						 error,
@@ -791,8 +742,12 @@ fu_pxi_receiver_device_add_peripherals(FuPxiReceiverDevice *device, guint idx, G
 		FuContext *ctx = fu_device_get_context(FU_DEVICE(device));
 		g_autoptr(FuPxiWirelessDevice) child = fu_pxi_wireless_device_new(ctx, &model);
 		g_autofree gchar *logical_id = g_strdup_printf("IDX:0x%02x", idx);
-		fu_device_add_instance_u16(FU_DEVICE(child), "VEN", device->vendor);
-		fu_device_add_instance_u16(FU_DEVICE(child), "DEV", device->product);
+		fu_device_add_instance_u16(FU_DEVICE(child),
+					   "VEN",
+					   fu_device_get_vid(FU_DEVICE(device)));
+		fu_device_add_instance_u16(FU_DEVICE(child),
+					   "DEV",
+					   fu_device_get_pid(FU_DEVICE(device)));
 		fu_device_add_instance_str(FU_DEVICE(child), "MODEL", model_name);
 		if (!fu_device_build_instance_id(FU_DEVICE(child),
 						 error,
@@ -808,46 +763,21 @@ fu_pxi_receiver_device_add_peripherals(FuPxiReceiverDevice *device, guint idx, G
 		fu_device_add_child(FU_DEVICE(device), FU_DEVICE(child));
 	}
 	return TRUE;
-#else
-	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
-			    "<linux/hidraw.h> not available");
-	return FALSE;
-#endif
 }
 
 static gboolean
-fu_pxi_receiver_device_setup_guid(FuPxiReceiverDevice *device, GError **error)
+fu_pxi_receiver_device_setup_guid(FuPxiReceiverDevice *self, GError **error)
 {
-#ifdef HAVE_HIDRAW_H
-	struct hidraw_devinfo hid_raw_info = {0x0};
-	g_autofree gchar *devid = NULL;
-	g_autoptr(GString) dev_name = NULL;
-
-	/* extra GUID with device name */
-	if (!fu_pxi_receiver_device_get_raw_info(device, &hid_raw_info, error))
-		return FALSE;
-
-	device->vendor = (guint)hid_raw_info.vendor;
-	device->product = (guint)hid_raw_info.product;
-
-	dev_name = g_string_new(fu_device_get_name(FU_DEVICE(device)));
-	g_string_ascii_up(dev_name);
+	g_autoptr(GString) dev_name = g_string_new(fu_device_get_name(FU_DEVICE(self)));
 	g_string_replace(dev_name, " ", "_", 0);
-	devid = g_strdup_printf("HIDRAW\\VEN_%04X&DEV_%04X&NAME_%s",
-				(guint)hid_raw_info.vendor,
-				(guint)hid_raw_info.product,
-				dev_name->str);
-	fu_device_add_instance_id(FU_DEVICE(device), devid);
-	return TRUE;
-#else
-	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
-			    "<linux/hidraw.h> not available");
-	return FALSE;
-#endif
+	fu_device_add_instance_strup(FU_DEVICE(self), "NAME", dev_name->str);
+	return fu_device_build_instance_id(FU_DEVICE(self),
+					   error,
+					   "HIDRAW",
+					   "VEN",
+					   "DEV",
+					   "NAME",
+					   NULL);
 }
 
 static gboolean
@@ -894,32 +824,29 @@ fu_pxi_receiver_device_setup(FuDevice *device, GError **error)
 static gboolean
 fu_pxi_receiver_device_probe(FuDevice *device, GError **error)
 {
-	guint64 iface_nr = 0;
-	g_autoptr(FuUdevDevice) usb_parent = NULL;
+	g_autofree gchar *iface_nr = NULL;
+	g_autoptr(FuDevice) usb_parent = NULL;
 
 	/* check USB interface number */
-	usb_parent = fu_udev_device_get_parent_with_subsystem(FU_UDEV_DEVICE(device), "usb");
-	if (usb_parent == NULL) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_NOT_SUPPORTED,
-				    "no parent with USB subsystem");
+	usb_parent = fu_device_get_backend_parent_with_subsystem(device, "usb", error);
+	if (usb_parent == NULL)
 		return FALSE;
-	}
-	if (!fu_udev_device_get_sysfs_attr_uint64(usb_parent, "bInterfaceNumber", &iface_nr, error))
+	iface_nr = fu_udev_device_read_sysfs(FU_UDEV_DEVICE(usb_parent),
+					     "bInterfaceNumber",
+					     FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+					     error);
+	if (iface_nr == NULL)
 		return FALSE;
-	if (iface_nr != 0) {
+	if (g_strcmp0(iface_nr, "01") != 0) {
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "only USB interface 0 supported");
+			    "only USB interface 1 supported");
 		return FALSE;
 	}
 
-	/* set the logical and physical ID */
-	if (!fu_udev_device_set_logical_id(FU_UDEV_DEVICE(device), "hid", error))
-		return FALSE;
-	return fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "hid", error);
+	/* success */
+	return TRUE;
 }
 
 static void
@@ -939,21 +866,23 @@ fu_pxi_receiver_device_init(FuPxiReceiverDevice *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
 	fu_device_add_icon(FU_DEVICE(self), "usb-receiver");
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_TRIPLET);
-	fu_device_add_vendor_id(FU_DEVICE(self), "USB:0x093A");
+	fu_device_build_vendor_id_u16(FU_DEVICE(self), "USB", 0x093A);
 	fu_device_add_protocol(FU_DEVICE(self), "com.pixart.rf");
 	fu_device_set_firmware_gtype(FU_DEVICE(self), FU_TYPE_PXI_FIRMWARE);
-	fu_device_register_private_flag(FU_DEVICE(self), FU_PXI_DEVICE_FLAG_IS_HPAC, "is-hpac");
+	fu_device_register_private_flag(FU_DEVICE(self), FU_PXI_DEVICE_FLAG_IS_HPAC);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_READ);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_WRITE);
 	fu_device_set_remove_delay(FU_DEVICE(self), 10000);
 }
 
 static void
 fu_pxi_receiver_device_class_init(FuPxiReceiverDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->to_string = fu_pxi_receiver_device_to_string;
-	klass_device->setup = fu_pxi_receiver_device_setup;
-	klass_device->probe = fu_pxi_receiver_device_probe;
-	klass_device->write_firmware = fu_pxi_receiver_device_write_firmware;
-	klass_device->prepare_firmware = fu_pxi_receiver_device_prepare_firmware;
-	klass_device->set_progress = fu_pxi_receiver_device_set_progress;
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->to_string = fu_pxi_receiver_device_to_string;
+	device_class->setup = fu_pxi_receiver_device_setup;
+	device_class->probe = fu_pxi_receiver_device_probe;
+	device_class->write_firmware = fu_pxi_receiver_device_write_firmware;
+	device_class->prepare_firmware = fu_pxi_receiver_device_prepare_firmware;
+	device_class->set_progress = fu_pxi_receiver_device_set_progress;
 }
