@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -30,12 +30,10 @@ fu_redfish_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuRedfishDevice *self = FU_REDFISH_DEVICE(device);
 	FuRedfishDevicePrivate *priv = GET_PRIVATE(self);
-	if (priv->milestone > 0x0)
-		fu_string_append_kx(str, idt, "Milestone", priv->milestone);
-	if (priv->build != NULL)
-		fu_string_append(str, idt, "Build", priv->build);
-	fu_string_append_ku(str, idt, "ResetPretDelay", priv->reset_pre_delay);
-	fu_string_append_ku(str, idt, "ResetPostDelay", priv->reset_post_delay);
+	fwupd_codec_string_append_hex(str, idt, "Milestone", priv->milestone);
+	fwupd_codec_string_append(str, idt, "Build", priv->build);
+	fwupd_codec_string_append_int(str, idt, "ResetPretDelay", priv->reset_pre_delay);
+	fwupd_codec_string_append_int(str, idt, "ResetPostDelay", priv->reset_post_delay);
 }
 
 static void
@@ -93,38 +91,54 @@ fu_redfish_device_probe_related_pcie_item(FuRedfishDevice *self, const gchar *ur
 	if (json_object_has_member(json_obj, "VendorId")) {
 		const gchar *tmp = json_object_get_string_member(json_obj, "VendorId");
 		if (tmp != NULL && tmp[0] != '\0') {
-			if (!fu_strtoull(tmp, &vendor_id, 0, G_MAXUINT16, error))
+			if (!fu_strtoull(tmp,
+					 &vendor_id,
+					 0,
+					 G_MAXUINT16,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 	}
 	if (json_object_has_member(json_obj, "DeviceId")) {
 		const gchar *tmp = json_object_get_string_member(json_obj, "DeviceId");
 		if (tmp != NULL && tmp[0] != '\0') {
-			if (!fu_strtoull(tmp, &model_id, 0, G_MAXUINT16, error))
+			if (!fu_strtoull(tmp,
+					 &model_id,
+					 0,
+					 G_MAXUINT16,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 	}
 	if (json_object_has_member(json_obj, "SubsystemVendorId")) {
 		const gchar *tmp = json_object_get_string_member(json_obj, "SubsystemVendorId");
 		if (tmp != NULL && tmp[0] != '\0') {
-			if (!fu_strtoull(tmp, &subsystem_vendor_id, 0, G_MAXUINT16, error))
+			if (!fu_strtoull(tmp,
+					 &subsystem_vendor_id,
+					 0,
+					 G_MAXUINT16,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 	}
 	if (json_object_has_member(json_obj, "SubsystemId")) {
 		const gchar *tmp = json_object_get_string_member(json_obj, "SubsystemId");
 		if (tmp != NULL && tmp[0] != '\0') {
-			if (!fu_strtoull(tmp, &subsystem_model_id, 0, G_MAXUINT16, error))
+			if (!fu_strtoull(tmp,
+					 &subsystem_model_id,
+					 0,
+					 G_MAXUINT16,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 	}
 
 	/* add vendor ID */
-	if (vendor_id != 0x0) {
-		g_autofree gchar *vendor_id_str = NULL;
-		vendor_id_str = g_strdup_printf("PCI:0x%04X", (guint)vendor_id);
-		fu_device_add_vendor_id(FU_DEVICE(self), vendor_id_str);
-	}
+	fu_device_build_vendor_id_u16(FU_DEVICE(self), "PCI", vendor_id);
 
 	/* add more instance IDs if possible */
 	if (vendor_id != 0x0)
@@ -241,14 +255,7 @@ fu_redfish_device_set_version_lenovo(FuRedfishDevice *self, const gchar *version
 		return FALSE;
 
 	/* split out milestone */
-	priv->milestone = g_ascii_strtoull(out_build, NULL, 10);
-	if (priv->milestone == 0) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "version milestone invalid");
-		return FALSE;
-	}
+	priv->milestone = g_ascii_strtoull(out_build, NULL, 10); /* nocheck:blocked */
 
 	/* odd numbered builds are unsigned */
 	if (priv->milestone % 2 != 0) {
@@ -257,7 +264,7 @@ fu_redfish_device_set_version_lenovo(FuRedfishDevice *self, const gchar *version
 
 	/* build is only one letter from A -> Z */
 	if (!g_ascii_isalpha(out_build[2])) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "build letter invalid");
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA, "build letter invalid");
 		return FALSE;
 	}
 	priv->build = g_strndup(out_build + 2, 1);
@@ -346,7 +353,6 @@ static void
 fu_redfish_device_set_vendor(FuRedfishDevice *self, const gchar *vendor)
 {
 	g_autofree gchar *vendor_upper = NULL;
-	g_autofree gchar *vendor_id = NULL;
 
 	/* fixup a common mistake */
 	if (g_strcmp0(vendor, "LEN") == 0 || g_strcmp0(vendor, "LNVO") == 0)
@@ -356,14 +362,12 @@ fu_redfish_device_set_vendor(FuRedfishDevice *self, const gchar *vendor)
 	/* add vendor-id */
 	vendor_upper = g_ascii_strup(vendor, -1);
 	g_strdelimit(vendor_upper, " ", '_');
-	vendor_id = g_strdup_printf("REDFISH:%s", vendor_upper);
-	fu_device_add_vendor_id(FU_DEVICE(self), vendor_id);
+	fu_device_build_vendor_id(FU_DEVICE(self), "REDFISH", vendor_upper);
 }
 
 static void
-fu_redfish_backend_smc_license_check(FuDevice *device)
+fu_redfish_device_smc_license_check(FuRedfishDevice *self)
 {
-	FuRedfishDevice *self = FU_REDFISH_DEVICE(device);
 	FuRedfishBackend *backend = fu_redfish_device_get_backend(self);
 	g_autoptr(FuRedfishRequest) request = fu_redfish_backend_request_new(backend);
 	g_autoptr(GError) error_local = NULL;
@@ -373,21 +377,47 @@ fu_redfish_backend_smc_license_check(FuDevice *device)
 					fu_redfish_backend_get_push_uri_path(backend),
 					FU_REDFISH_REQUEST_PERFORM_FLAG_LOAD_JSON,
 					&error_local)) {
-		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED))
-			fu_device_add_problem(device, FWUPD_DEVICE_PROBLEM_MISSING_LICENSE);
-		else
-			g_debug("supermicro license check returned %s\n", error_local->message);
+		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
+			fu_device_add_problem(FU_DEVICE(self),
+					      FWUPD_DEVICE_PROBLEM_MISSING_LICENSE);
+		} else {
+			g_debug("supermicro license check returned %s", error_local->message);
+		}
 	}
 }
 
+static gboolean
+fu_redfish_device_probe_oem_hpe(FuRedfishDevice *self, JsonObject *json_object, GError **error)
+{
+	if (json_object_has_member(json_object, "DeviceClass")) {
+		const gchar *guid = json_object_get_string_member(json_object, "DeviceClass");
+		if (guid != NULL)
+			fu_device_add_instance_id(FU_DEVICE(self), guid);
+	}
+	if (json_object_has_member(json_object, "Targets")) {
+		JsonArray *json_array = json_object_get_array_member(json_object, "Targets");
+		for (guint i = 0; i < json_array_get_length(json_array); i++) {
+			const gchar *guid = json_array_get_string_element(json_array, i);
+			if (guid != NULL)
+				fu_device_add_instance_id(FU_DEVICE(self), guid);
+		}
+	}
+	return TRUE;
+}
+
 static gboolean
 fu_redfish_device_probe(FuDevice *dev, GError **error)
 {
 	FuRedfishDevice *self = FU_REDFISH_DEVICE(dev);
 	FuRedfishDevicePrivate *priv = GET_PRIVATE(self);
 	JsonObject *member = priv->member;
-	const gchar *guid = NULL;
-	g_autofree gchar *guid_lower = NULL;
+	const gchar *software_id = NULL;
+
+	/* sanity check */
+	if (priv->member == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no member");
+		return FALSE;
+	}
 
 	/* required to POST later */
 	if (!json_object_has_member(member, "@odata.id")) {
@@ -404,27 +434,7 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 		if (tmp != NULL)
 			fu_device_set_backend_id(dev, tmp);
 	}
-
-	/* get SoftwareId, falling back to vendor-specific versions */
-	if (json_object_has_member(member, "SoftwareId")) {
-		guid = json_object_get_string_member(member, "SoftwareId");
-	} else if (json_object_has_member(member, "Oem")) {
-		JsonObject *oem = json_object_get_object_member(member, "Oem");
-		if (oem != NULL && json_object_has_member(oem, "Hpe")) {
-			JsonObject *hpe = json_object_get_object_member(oem, "Hpe");
-			if (hpe != NULL && json_object_has_member(hpe, "DeviceClass"))
-				guid = json_object_get_string_member(hpe, "DeviceClass");
-		}
-	}
-
-	/* GUID is required */
-	if (guid == NULL) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_NOT_FOUND,
-				    "no GUID for device");
-		return FALSE;
-	}
+	fu_device_add_instance_str(dev, "ID", fu_device_get_backend_id(dev));
 
 	/* device properties */
 	if (json_object_has_member(member, "Manufacturer")) {
@@ -432,6 +442,7 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 		if (tmp != NULL && tmp[0] != '\0')
 			fu_redfish_device_set_vendor(self, tmp);
 	}
+	fu_device_add_instance_strsafe(dev, "VENDOR", fu_device_get_vendor(dev));
 
 	/* the version can encode the instance ID suffix */
 	if (json_object_has_member(member, "Version")) {
@@ -443,7 +454,7 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 	/* ReleaseDate may or may not have a timezone */
 	if (json_object_has_member(member, "ReleaseDate")) {
 		const gchar *tmp = json_object_get_string_member(member, "ReleaseDate");
-		if (tmp != NULL && tmp[0] != '\0' && g_strcmp0(tmp, "00:00:00Z") != 0) {
+		if (tmp != NULL && tmp[0] != '\0') {
 			g_autoptr(GDateTime) dt = NULL;
 			g_autoptr(GTimeZone) tz = g_time_zone_new_utc();
 			dt = g_date_time_new_from_iso8601(tmp, tz);
@@ -456,37 +467,39 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 		}
 	}
 
-	/* add IDs */
-	fu_device_add_instance_strsafe(dev, "VENDOR", fu_device_get_vendor(dev));
-	fu_device_add_instance_str(dev, "SOFTWAREID", guid);
-	fu_device_add_instance_str(dev, "ID", fu_device_get_backend_id(dev));
-
 	/* some vendors use a GUID, others use an ID like BMC-AFBT-10 */
-	guid_lower = g_ascii_strdown(guid, -1);
-	if (fwupd_guid_is_valid(guid_lower)) {
-		fu_device_add_guid(dev, guid_lower);
-	} else {
-		if (fu_device_has_private_flag(dev, FU_REDFISH_DEVICE_FLAG_UNSIGNED_BUILD))
-			fu_device_add_instance_str(dev, "TYPE", "UNSIGNED");
+	if (json_object_has_member(member, "SoftwareId"))
+		software_id = json_object_get_string_member(member, "SoftwareId");
+	if (software_id != NULL) {
+		g_autofree gchar *software_id_lower = g_ascii_strdown(software_id, -1);
+		if (fwupd_guid_is_valid(software_id_lower)) {
+			fu_device_add_instance_id(dev, software_id_lower);
+		} else {
+			fu_device_add_instance_str(dev, "SOFTWAREID", software_id);
+			if (fu_device_has_private_flag(dev, FU_REDFISH_DEVICE_FLAG_UNSIGNED_BUILD))
+				fu_device_add_instance_str(dev, "TYPE", "UNSIGNED");
+			fu_device_build_instance_id(dev,
+						    NULL,
+						    "REDFISH",
+						    "VENDOR",
+						    "SOFTWAREID",
+						    "TYPE",
+						    NULL);
+		}
+	}
 
-		fu_device_build_instance_id(dev,
-					    NULL,
-					    "REDFISH",
-					    "VENDOR",
-					    "SOFTWAREID",
-					    "TYPE",
-					    NULL);
-		fu_device_build_instance_id(dev, NULL, "REDFISH", "VENDOR", "SOFTWAREID", NULL);
+	/* get vendor-specific properties too */
+	if (json_object_has_member(member, "Oem")) {
+		JsonObject *oem = json_object_get_object_member(member, "Oem");
+		if (oem != NULL && json_object_has_member(oem, "Hpe")) {
+			JsonObject *hpe = json_object_get_object_member(oem, "Hpe");
+			if (!fu_redfish_device_probe_oem_hpe(self, hpe, error))
+				return FALSE;
+		}
 	}
 
 	/* used for quirking and parenting */
-	fu_device_build_instance_id_full(dev,
-					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
-					 NULL,
-					 "REDFISH",
-					 "VENDOR",
-					 "ID",
-					 NULL);
+	fu_device_build_instance_id(dev, NULL, "REDFISH", "VENDOR", "ID", NULL);
 
 	if (json_object_has_member(member, "Name")) {
 		const gchar *tmp = json_object_get_string_member(member, "Name");
@@ -501,7 +514,7 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 	if (json_object_has_member(member, "Description")) {
 		const gchar *tmp = json_object_get_string_member(member, "Description");
 		if (tmp != NULL && tmp[0] != '\0')
-			fu_device_set_description(dev, tmp);
+			fu_device_set_summary(dev, tmp);
 	}
 
 	/* reasons why the device might not be updatable */
@@ -509,16 +522,10 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 		if (json_object_get_boolean_member(member, "Updateable"))
 			fu_device_add_flag(dev, FWUPD_DEVICE_FLAG_UPDATABLE);
 	}
-
-	/* not useful to export */
-	if (fu_device_has_private_flag(dev, FU_REDFISH_DEVICE_FLAG_IS_BACKUP)) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "%s is a backup partition",
-			    fu_device_get_backend_id(dev));
-		return FALSE;
-	}
+	if (fu_device_has_private_flag(dev, FU_REDFISH_DEVICE_FLAG_IS_BACKUP))
+		fu_device_inhibit(dev, "is-backup", "Is a backup partition");
+	else
+		fu_device_uninhibit(dev, "is-backup");
 
 	/* use related items to set extra instance IDs */
 	if (fu_device_has_flag(dev, FWUPD_DEVICE_FLAG_UPDATABLE) &&
@@ -538,7 +545,7 @@ fu_redfish_device_probe(FuDevice *dev, GError **error)
 
 	/* for Supermicro check whether we have a proper Redfish license installed */
 	if (g_strcmp0("SMCI", fu_device_get_vendor(dev)) == 0)
-		fu_redfish_backend_smc_license_check(dev);
+		fu_redfish_device_smc_license_check(self);
 
 	/* success */
 	return TRUE;
@@ -717,7 +724,7 @@ fu_redfish_device_poll_task_once(FuRedfishDevice *self, FuRedfishDevicePollCtx *
 		return TRUE;
 	}
 	if (g_strcmp0(state_tmp, "Cancelled") == 0) {
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Task was cancelled");
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "Task was cancelled");
 		return FALSE;
 	}
 	if (g_strcmp0(state_tmp, "Exception") == 0 ||
@@ -770,10 +777,8 @@ fu_redfish_device_poll_task(FuRedfishDevice *self,
 		fu_device_sleep(FU_DEVICE(self), 1000); /* ms */
 		if (!fu_redfish_device_poll_task_once(self, ctx, error))
 			return FALSE;
-		if (ctx->completed) {
-			fu_progress_finished(progress);
+		if (ctx->completed)
 			return TRUE;
-		}
 	} while (g_timer_elapsed(timer, NULL) < timeout);
 
 	/* success */
@@ -811,13 +816,13 @@ fu_redfish_device_set_quirk_kv(FuDevice *device,
 	guint64 tmp = 0;
 
 	if (g_strcmp0(key, "RedfishResetPreDelay") == 0) {
-		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT, FU_INTEGER_BASE_AUTO, error))
 			return FALSE;
 		priv->reset_pre_delay = tmp;
 		return TRUE;
 	}
 	if (g_strcmp0(key, "RedfishResetPostDelay") == 0) {
-		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT, FU_INTEGER_BASE_AUTO, error))
 			return FALSE;
 		priv->reset_post_delay = tmp;
 		return TRUE;
@@ -847,6 +852,18 @@ fu_redfish_device_get_property(GObject *object, guint prop_id, GValue *value, GP
 	}
 }
 
+static void
+fu_redfish_device_set_member(FuRedfishDevice *self, JsonObject *member)
+{
+	FuRedfishDevicePrivate *priv = GET_PRIVATE(self);
+	if (priv->member != NULL) {
+		json_object_unref(priv->member);
+		priv->member = NULL;
+	}
+	if (member != NULL)
+		priv->member = json_object_ref(member);
+}
+
 static void
 fu_redfish_device_set_property(GObject *object,
 			       guint prop_id,
@@ -860,7 +877,7 @@ fu_redfish_device_set_property(GObject *object,
 		g_set_object(&priv->backend, g_value_get_object(value));
 		break;
 	case PROP_MEMBER:
-		priv->member = json_object_ref(g_value_get_pointer(value));
+		fu_redfish_device_set_member(self, g_value_get_pointer(value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -874,26 +891,17 @@ fu_redfish_device_init(FuRedfishDevice *self)
 	fu_device_set_summary(FU_DEVICE(self), "Redfish device");
 	fu_device_add_protocol(FU_DEVICE(self), "org.dmtf.redfish");
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_NAME);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_VERFMT);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_ICON);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_VENDOR);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_SIGNED);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_NAME);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_VERFMT);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_ICON);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_VENDOR);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_SIGNED);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_REDFISH_DEVICE_FLAG_IS_BACKUP);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_REDFISH_DEVICE_FLAG_UNSIGNED_BUILD);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_REDFISH_DEVICE_FLAG_WILDCARD_TARGETS);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_REDFISH_DEVICE_FLAG_MANAGER_RESET);
 	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_REDFISH_DEVICE_FLAG_IS_BACKUP,
-					"is-backup");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_REDFISH_DEVICE_FLAG_UNSIGNED_BUILD,
-					"unsigned-build");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_REDFISH_DEVICE_FLAG_WILDCARD_TARGETS,
-					"wildcard-targets");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_REDFISH_DEVICE_FLAG_MANAGER_RESET,
-					"manager-reset");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_REDFISH_DEVICE_FLAG_NO_MANAGER_RESET_REQUEST,
-					"no-manager-reset-request");
+					FU_REDFISH_DEVICE_FLAG_NO_MANAGER_RESET_REQUEST);
 }
 
 static void
@@ -914,15 +922,15 @@ fu_redfish_device_class_init(FuRedfishDeviceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GParamSpec *pspec;
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 
 	object_class->get_property = fu_redfish_device_get_property;
 	object_class->set_property = fu_redfish_device_set_property;
 	object_class->finalize = fu_redfish_device_finalize;
 
-	klass_device->to_string = fu_redfish_device_to_string;
-	klass_device->probe = fu_redfish_device_probe;
-	klass_device->set_quirk_kv = fu_redfish_device_set_quirk_kv;
+	device_class->to_string = fu_redfish_device_to_string;
+	device_class->probe = fu_redfish_device_probe;
+	device_class->set_quirk_kv = fu_redfish_device_set_quirk_kv;
 
 	/**
 	 * FuRedfishDevice:backend:
