@@ -1,14 +1,17 @@
 /*
- * Copyright (C) 2022 Gaël PORTAY <gael.portay@collabora.com>
+ * Copyright 2022 Gaël PORTAY <gael.portay@collabora.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
 
 #include "fu-steelseries-firmware.h"
+#include "fu-steelseries-fizz-impl.h"
 #include "fu-steelseries-fizz-tunnel.h"
 
+#define FU_STEELSERIES_FIZZ_TUNNEL_POLL_INTERVAL 20000 /* ms*/
+
 struct _FuSteelseriesFizzTunnel {
 	FuDevice parent_instance;
 };
@@ -18,33 +21,39 @@ G_DEFINE_TYPE(FuSteelseriesFizzTunnel, fu_steelseries_fizz_tunnel, FU_TYPE_DEVIC
 static gboolean
 fu_steelseries_fizz_tunnel_ping(FuDevice *device, gboolean *reached, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
-	guint8 status;
+	FuDevice *proxy = fu_device_get_proxy(device);
+	FuSteelseriesFizzConnectionStatus status = FU_STEELSERIES_FIZZ_CONNECTION_STATUS_CONNECTED;
 	guint8 level;
 	g_autoptr(GError) error_local = NULL;
 	g_autofree gchar *version = NULL;
 
-	if (!fu_steelseries_fizz_get_connection_status(parent, &status, error)) {
+	if (proxy == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
+
+	if (!fu_steelseries_fizz_impl_get_connection_status(FU_STEELSERIES_FIZZ_IMPL(proxy),
+							    &status,
+							    error)) {
 		g_prefix_error(error, "failed to get connection status: ");
 		return FALSE;
 	}
-	g_debug("ConnectionStatus: %u", status);
-	*reached = status != STEELSERIES_FIZZ_CONNECTION_STATUS_NOT_CONNECTED;
+	g_debug("FuSteelseriesFizzConnection: %s",
+		fu_steelseries_fizz_connection_status_to_string(status));
+	*reached = status != FU_STEELSERIES_FIZZ_CONNECTION_STATUS_NOT_CONNECTED;
 	if (!*reached) {
 		/* success */
 		return TRUE;
 	}
 
 	/* ping device anyway */
-	if (!fu_steelseries_fizz_get_battery_level(fu_device_get_parent(device),
-						   TRUE,
-						   &level,
-						   &error_local)) {
+	if (!fu_steelseries_fizz_impl_get_battery_level(FU_STEELSERIES_FIZZ_IMPL(proxy),
+							TRUE,
+							&level,
+							&error_local)) {
 		*reached = FALSE;
 
-		if (!g_error_matches(error_local,
-				     G_USB_DEVICE_ERROR,
-				     G_USB_DEVICE_ERROR_TIMED_OUT)) {
+		if (!g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT)) {
 			/* failure */
 			*error = g_steal_pointer(&error_local);
 			return FALSE;
@@ -54,19 +63,11 @@ fu_steelseries_fizz_tunnel_ping(FuDevice *device, gboolean *reached, GError **er
 		return TRUE;
 	}
 	g_debug("BatteryLevel: 0x%02x", level);
-	/*
-	 * CHARGING: Most significant bit. When bit is set to 1 it means battery is currently
-	 * charging/plugged in
-	 *
-	 * LEVEL: 7 least significant bit value of the battery. Values are between 2-21, to get %
-	 * you can do (LEVEL - 1) * 5
-	 */
-	fu_device_set_battery_level(device,
-				    ((level & STEELSERIES_FIZZ_BATTERY_LEVEL_STATUS_BITS) - 1U) *
-					5U);
+	fu_device_set_battery_level(device, level);
 
 	/* re-read version after reconnect/update */
-	version = fu_steelseries_fizz_get_version(parent, TRUE, error);
+	version =
+	    fu_steelseries_fizz_impl_get_version(FU_STEELSERIES_FIZZ_IMPL(proxy), TRUE, error);
 	if (version == NULL) {
 		*reached = FALSE;
 		g_prefix_error(error,
@@ -74,7 +75,7 @@ fu_steelseries_fizz_tunnel_ping(FuDevice *device, gboolean *reached, GError **er
 			       fu_device_get_id(device));
 		return FALSE;
 	}
-	fu_device_set_version(device, version);
+	fu_device_set_version(device, version); /* nocheck:set-version */
 
 	/* success */
 	return TRUE;
@@ -86,15 +87,18 @@ fu_steelseries_fizz_tunnel_wait_for_reconnect_cb(FuDevice *device,
 						 GError **error)
 {
 	FuDevice *parent = fu_device_get_parent(device);
-	guint8 status;
+	FuSteelseriesFizzConnectionStatus status;
 	gboolean reached;
 
-	if (!fu_steelseries_fizz_get_connection_status(parent, &status, error)) {
+	if (!fu_steelseries_fizz_get_connection_status(FU_STEELSERIES_FIZZ(parent),
+						       &status,
+						       error)) {
 		g_prefix_error(error, "failed to get connection status: ");
 		return FALSE;
 	}
-	g_debug("ConnectionStatus: %u", status);
-	if (status == STEELSERIES_FIZZ_CONNECTION_STATUS_NOT_CONNECTED) {
+	g_debug("FuSteelseriesFizzConnection: %s",
+		fu_steelseries_fizz_connection_status_to_string(status));
+	if (status == FU_STEELSERIES_FIZZ_CONNECTION_STATUS_NOT_CONNECTED) {
 		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "device is unreachable");
 		return FALSE;
 	}
@@ -120,6 +124,35 @@ fu_steelseries_fizz_tunnel_wait_for_reconnect(FuDevice *device, guint delay, GEr
 				    error);
 }
 
+static gboolean
+fu_steelseries_fizz_tunnel_detach(FuDevice *device, FuProgress *progress, GError **error)
+{
+	FuDevice *proxy = fu_device_get_proxy(device);
+
+	if (proxy == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
+	if (!fu_steelseries_fizz_impl_is_updatable(FU_STEELSERIES_FIZZ_IMPL(proxy), device, error))
+		return FALSE;
+
+	if (!fu_device_has_private_flag(device, FU_STEELSERIES_DEVICE_FLAG_DETACH_BOOTLOADER))
+		return TRUE;
+
+	/* switch to bootloader mode only if device needs it */
+	if (!fu_steelseries_fizz_reset(FU_STEELSERIES_FIZZ(device),
+				       FALSE,
+				       FU_STEELSERIES_FIZZ_RESET_MODE_BOOTLOADER,
+				       error))
+		return FALSE;
+
+	fu_device_add_flag(device, FWUPD_DEVICE_FLAG_IS_BOOTLOADER);
+	fu_device_add_flag(device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
+
+	/* success */
+	return TRUE;
+}
+
 static gboolean
 fu_steelseries_fizz_tunnel_attach(FuDevice *device, FuProgress *progress, GError **error)
 {
@@ -132,9 +165,9 @@ fu_steelseries_fizz_tunnel_attach(FuDevice *device, FuProgress *progress, GError
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 67, "sleep");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 33, NULL);
 
-	if (!fu_steelseries_fizz_reset(parent,
+	if (!fu_steelseries_fizz_reset(FU_STEELSERIES_FIZZ(parent),
 				       TRUE,
-				       STEELSERIES_FIZZ_RESET_MODE_NORMAL,
+				       FU_STEELSERIES_FIZZ_RESET_MODE_NORMAL,
 				       &error_local))
 		g_warning("failed to reset: %s", error_local->message);
 	fu_progress_step_done(progress);
@@ -157,22 +190,26 @@ fu_steelseries_fizz_tunnel_attach(FuDevice *device, FuProgress *progress, GError
 static gboolean
 fu_steelseries_fizz_tunnel_probe(FuDevice *device, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(parent));
+	FuDevice *proxy = fu_device_get_proxy(device);
 	guint16 release;
 
+	if (proxy == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
+
 	/* set the version if the release has been set */
-	release = g_usb_device_get_release(usb_device);
+	release = fu_usb_device_get_release(FU_USB_DEVICE(proxy));
 	if (release != 0x0 &&
 	    fu_device_get_version_format(device) == FWUPD_VERSION_FORMAT_UNKNOWN) {
 		fu_device_set_version_format(device, FWUPD_VERSION_FORMAT_BCD);
-		fu_device_set_version_u16(device, release);
+		fu_device_set_version_raw(device, release);
 	}
 
 	/* add GUIDs in order of priority */
 	fu_device_add_instance_str(device, "PROTOCOL", "FIZZ_TUNNEL");
-	fu_device_add_instance_u16(device, "VID", g_usb_device_get_vid(usb_device));
-	fu_device_add_instance_u16(device, "PID", g_usb_device_get_pid(usb_device));
+	fu_device_add_instance_u16(device, "VID", fu_device_get_vid(proxy));
+	fu_device_add_instance_u16(device, "PID", fu_device_get_pid(proxy));
 	fu_device_add_instance_u16(device, "REV", release);
 	fu_device_build_instance_id_full(device,
 					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
@@ -182,7 +219,7 @@ fu_steelseries_fizz_tunnel_probe(FuDevice *device, GError **error)
 					 "PROTOCOL",
 					 NULL);
 	fu_device_build_instance_id(device, NULL, "STEELSERIES", "VID", "PID", "PROTOCOL", NULL);
-	if (fu_device_has_internal_flag(device, FU_DEVICE_INTERNAL_FLAG_ADD_INSTANCE_ID_REV)) {
+	if (fu_device_has_private_flag(device, FU_DEVICE_PRIVATE_FLAG_ADD_INSTANCE_ID_REV)) {
 		fu_device_build_instance_id(device,
 					    NULL,
 					    "STEELSERIES",
@@ -200,21 +237,35 @@ fu_steelseries_fizz_tunnel_probe(FuDevice *device, GError **error)
 static gboolean
 fu_steelseries_fizz_tunnel_setup(FuDevice *device, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
-	guint32 calculated_crc;
-	guint32 stored_crc;
+	FuDevice *proxy = fu_device_get_proxy(device);
 	gboolean reached;
-	guint8 fs = STEELSERIES_FIZZ_FILESYSTEM_MOUSE;
-	guint8 id = STEELSERIES_FIZZ_MOUSE_FILESYSTEM_BACKUP_APP_ID;
+	g_autofree gchar *serial = NULL;
 	g_autoptr(GError) error_local = NULL;
 
+	if (proxy == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
+
 	/* ping */
 	if (!fu_steelseries_fizz_tunnel_ping(device, &reached, &error_local)) {
-		g_debug("ignoring error on ping: %s", error_local->message);
+		g_debug("ignoring error for %s on ping: %s",
+			fu_device_get_id(device),
+			error_local->message);
 
 		/* success */
 		return TRUE;
 	}
+
+	serial = fu_steelseries_fizz_impl_get_serial(FU_STEELSERIES_FIZZ_IMPL(proxy),
+						     FALSE,
+						     &error_local);
+	if (serial != NULL) {
+		fu_device_set_serial(device, serial);
+	} else {
+		g_debug("ignoring error: %s", error_local->message);
+	}
+
 	if (!reached) {
 		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_UNREACHABLE);
 
@@ -222,27 +273,6 @@ fu_steelseries_fizz_tunnel_setup(FuDevice *device, GError **error)
 		return TRUE;
 	}
 
-	if (!fu_steelseries_fizz_get_crc32_fs(parent,
-					      TRUE,
-					      fs,
-					      id,
-					      &calculated_crc,
-					      &stored_crc,
-					      error)) {
-		g_prefix_error(error,
-			       "failed to get file CRC32 from FS 0x%02x ID 0x%02x: ",
-			       fs,
-			       id);
-		return FALSE;
-	}
-
-	if (calculated_crc != stored_crc) {
-		g_warning("%s: checksum mismatch, got 0x%08x, expected 0x%08x",
-			  fu_device_get_name(device),
-			  calculated_crc,
-			  stored_crc);
-	}
-
 	/* success */
 	return TRUE;
 }
@@ -251,14 +281,23 @@ static gboolean
 fu_steelseries_fizz_tunnel_poll(FuDevice *device, GError **error)
 {
 	FuDevice *parent = fu_device_get_parent(device);
+	FuDevice *proxy = fu_device_get_proxy(device);
 	guint32 calculated_crc;
 	guint32 stored_crc;
 	gboolean reached;
-	guint8 fs = STEELSERIES_FIZZ_FILESYSTEM_MOUSE;
-	guint8 id = STEELSERIES_FIZZ_MOUSE_FILESYSTEM_BACKUP_APP_ID;
+	guint8 fs = 0;
+	guint8 id = 0;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(FuDeviceLocker) locker = NULL;
 
+	if (!fu_steelseries_fizz_impl_get_fs_id(FU_STEELSERIES_FIZZ_IMPL(proxy), FALSE, &fs, error))
+		return FALSE;
+	if (!fu_steelseries_fizz_impl_get_file_id(FU_STEELSERIES_FIZZ_IMPL(proxy),
+						  FALSE,
+						  &id,
+						  error))
+		return FALSE;
+
 	/* open */
 	locker = fu_device_locker_new(parent, error);
 	if (locker == NULL)
@@ -276,7 +315,10 @@ fu_steelseries_fizz_tunnel_poll(FuDevice *device, GError **error)
 		return TRUE;
 	}
 
-	if (!fu_steelseries_fizz_get_crc32_fs(parent,
+	/* set reachable here since we ignore crc error while polling device */
+	fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_UNREACHABLE);
+
+	if (!fu_steelseries_fizz_get_crc32_fs(FU_STEELSERIES_FIZZ(parent),
 					      TRUE,
 					      fs,
 					      id,
@@ -298,8 +340,6 @@ fu_steelseries_fizz_tunnel_poll(FuDevice *device, GError **error)
 			  calculated_crc,
 			  stored_crc);
 
-	fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_UNREACHABLE);
-
 	/* success */
 	return TRUE;
 }
@@ -312,13 +352,22 @@ fu_steelseries_fizz_tunnel_write_firmware(FuDevice *device,
 					  GError **error)
 {
 	FuDevice *parent = fu_device_get_parent(device);
-	guint8 fs = STEELSERIES_FIZZ_FILESYSTEM_MOUSE;
-	guint8 id = STEELSERIES_FIZZ_MOUSE_FILESYSTEM_BACKUP_APP_ID;
+	FuDevice *proxy = fu_device_get_proxy(device);
+	guint8 fs = 0;
+	guint8 id = 0;
+
+	if (!fu_steelseries_fizz_impl_get_fs_id(FU_STEELSERIES_FIZZ_IMPL(proxy), FALSE, &fs, error))
+		return FALSE;
+	if (!fu_steelseries_fizz_impl_get_file_id(FU_STEELSERIES_FIZZ_IMPL(proxy),
+						  FALSE,
+						  &id,
+						  error))
+		return FALSE;
 
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100, NULL);
 
-	if (!fu_steelseries_fizz_write_firmware_fs(parent,
+	if (!fu_steelseries_fizz_write_firmware_fs(FU_STEELSERIES_FIZZ(parent),
 						   TRUE,
 						   fs,
 						   id,
@@ -337,14 +386,23 @@ static FuFirmware *
 fu_steelseries_fizz_tunnel_read_firmware(FuDevice *device, FuProgress *progress, GError **error)
 {
 	FuDevice *parent = fu_device_get_parent(device);
-	guint8 fs = STEELSERIES_FIZZ_FILESYSTEM_MOUSE;
-	guint8 id = STEELSERIES_FIZZ_MOUSE_FILESYSTEM_BACKUP_APP_ID;
+	FuDevice *proxy = fu_device_get_proxy(device);
+	guint8 fs = 0;
+	guint8 id = 0;
 	g_autoptr(FuFirmware) firmware = NULL;
 
+	if (!fu_steelseries_fizz_impl_get_fs_id(FU_STEELSERIES_FIZZ_IMPL(proxy), FALSE, &fs, error))
+		return NULL;
+	if (!fu_steelseries_fizz_impl_get_file_id(FU_STEELSERIES_FIZZ_IMPL(proxy),
+						  FALSE,
+						  &id,
+						  error))
+		return NULL;
+
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_READ, 100, NULL);
 
-	firmware = fu_steelseries_fizz_read_firmware_fs(parent,
+	firmware = fu_steelseries_fizz_read_firmware_fs(FU_STEELSERIES_FIZZ(parent),
 							TRUE,
 							fs,
 							id,
@@ -369,18 +427,26 @@ fu_steelseries_fizz_tunnel_set_progress(FuDevice *self, FuProgress *progress)
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 0, "reload");
 }
 
+static gchar *
+fu_steelseries_fizz_tunnel_convert_version(FuDevice *device, guint64 version_raw)
+{
+	return fu_version_from_uint16(version_raw, fu_device_get_version_format(device));
+}
+
 static void
 fu_steelseries_fizz_tunnel_class_init(FuSteelseriesFizzTunnelClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 
-	klass_device->attach = fu_steelseries_fizz_tunnel_attach;
-	klass_device->probe = fu_steelseries_fizz_tunnel_probe;
-	klass_device->setup = fu_steelseries_fizz_tunnel_setup;
-	klass_device->poll = fu_steelseries_fizz_tunnel_poll;
-	klass_device->write_firmware = fu_steelseries_fizz_tunnel_write_firmware;
-	klass_device->read_firmware = fu_steelseries_fizz_tunnel_read_firmware;
-	klass_device->set_progress = fu_steelseries_fizz_tunnel_set_progress;
+	device_class->detach = fu_steelseries_fizz_tunnel_detach;
+	device_class->attach = fu_steelseries_fizz_tunnel_attach;
+	device_class->probe = fu_steelseries_fizz_tunnel_probe;
+	device_class->setup = fu_steelseries_fizz_tunnel_setup;
+	device_class->poll = fu_steelseries_fizz_tunnel_poll;
+	device_class->write_firmware = fu_steelseries_fizz_tunnel_write_firmware;
+	device_class->read_firmware = fu_steelseries_fizz_tunnel_read_firmware;
+	device_class->set_progress = fu_steelseries_fizz_tunnel_set_progress;
+	device_class->convert_version = fu_steelseries_fizz_tunnel_convert_version;
 }
 
 static void
@@ -390,13 +456,16 @@ fu_steelseries_fizz_tunnel_init(FuSteelseriesFizzTunnel *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_CAN_VERIFY_IMAGE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_OPEN);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_AUTO_PAUSE_POLLING);
+	fu_device_register_private_flag(FU_DEVICE(self),
+					FU_STEELSERIES_DEVICE_FLAG_DETACH_BOOTLOADER);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FOR_OPEN);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_REFCOUNTED_PROXY);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_AUTO_PAUSE_POLLING);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_REPLUG_MATCH_GUID);
 	fu_device_add_protocol(FU_DEVICE(self), "com.steelseries.fizz");
 	fu_device_set_logical_id(FU_DEVICE(self), "tunnel");
-	fu_device_set_install_duration(FU_DEVICE(self), 38);				  /* 38 s */
 	fu_device_set_remove_delay(FU_DEVICE(self), FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE); /* 10 s */
-	fu_device_set_poll_interval(FU_DEVICE(self), 60000); /* 1 min */
+	fu_device_set_poll_interval(FU_DEVICE(self), FU_STEELSERIES_FIZZ_TUNNEL_POLL_INTERVAL);
 	fu_device_set_battery_threshold(FU_DEVICE(self), 20);
 	fu_device_set_firmware_gtype(FU_DEVICE(self), FU_TYPE_STEELSERIES_FIRMWARE);
 }
